@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -507,6 +507,136 @@ muse_cplmask_adapt_to_image(const cpl_mask *aMask, const cpl_image *aImage)
   return outmask;
 } /* muse_cplmask_adapt_to_image() */
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief  Compute the element-wise product of two matrices.
+  @param  aMatrix1   First matrix object.
+  @param  aMatrix2   Second matrix object.
+  @return On success the function returns a new matrix object. In case an
+          error occurs @c NULL is returned.
+
+  The function multiplies the matrices @em aMatrix1 and @em aMatrix2 element
+  by element. The input matrices must have the same dimensions. The result
+  is returned in a new matrix object.
+
+  The returned matrix has to be deallocated using cpl_matrix_delete().
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL, a parameter is NULL}
+  @error{set CPL_ERROR_INCOMPATIBLE_INPUT\, return NULL, input matrices have different sizes}
+
+ */
+/*----------------------------------------------------------------------------*/
+cpl_matrix *
+muse_cplmatrix_multiply_create(const cpl_matrix *aMatrix1,
+                               const cpl_matrix *aMatrix2)
+{
+  cpl_ensure(aMatrix1 && aMatrix2, CPL_ERROR_NULL_INPUT, NULL);
+
+  cpl_matrix *result = cpl_matrix_duplicate(aMatrix1);
+
+  if (cpl_matrix_multiply(result, aMatrix2) != CPL_ERROR_NONE) {
+    cpl_error_set_where(__func__);
+    cpl_matrix_delete(result);
+    return NULL;
+  }
+  return result;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief Select matrix elements according to a condition.
+  @param aMatrix     A matrix object.
+  @param aValue      The value to compare with the matrix elements.
+  @param aCondition  The condition function to use for comparing matrix
+                     elements with the reference value
+  @return On success the function returns an array object with the offset
+          positions of the selected matrix elements. In case an error occurs
+          @c NULL is returned.
+
+  Using the compare function @em aCondition all matrix elements are compared
+  to the given value @em aValue. The position of the selected matrix element
+  is stored in the returned index array. The returned position gives the
+  offset from the start of the matrix data buffer.
+ */
+/*----------------------------------------------------------------------------*/
+cpl_array *
+muse_cplmatrix_where(const cpl_matrix *aMatrix, double aValue,
+                      muse_cplmatrix_element_compare_func aCondition)
+{
+
+  cpl_ensure(aMatrix, CPL_ERROR_NULL_INPUT, NULL);
+  cpl_ensure(aCondition, CPL_ERROR_NULL_INPUT, NULL);
+
+  cpl_size i;
+  cpl_size count = 0;
+  cpl_size size  = cpl_matrix_get_nrow(aMatrix) * cpl_matrix_get_ncol(aMatrix);
+
+  const double *mdata = cpl_matrix_get_data_const(aMatrix);
+
+  cpl_size *selection  = cpl_malloc(size * sizeof *selection);
+  cpl_size *_selection = selection;
+
+  for (i = 0; i < size; ++i) {
+    if (aCondition(mdata[i], aValue)) {
+      *_selection++ = i;
+      ++count;
+    }
+  }
+  cpl_array *where = cpl_array_new(count, CPL_TYPE_SIZE);
+  cpl_array_copy_data_cplsize(where, selection);
+  cpl_free(selection);
+
+  return where;
+
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief  Extract the elements given by the index array
+  @param  aMatrix   A matrix object.
+  @param  aIndices  Array containing the matrix element offsets to extract.
+  @return A new matrix object containing the extracted matrix elements or
+          @c NULL in case of an error.
+
+  Creates a row vector (1xN matrix object) filled with the elements of the
+  matrix @em aMatrix which are listed in the index array @em aIndices. The
+  selected indices given by @em aIndices are offset indices, i.e. are
+  obtained from the matrix row and column indices using the relation
+  row index times number of columns plus column index.
+
+  The returned array has to be deallocated using cpl_array_delete().
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL, a parameter is NULL}
+  @error{set CPL_ERROR_INVALID_TYPE\, return NULL, aIndices type is not CPL_TYPE_SIZE}
+
+ */
+/*----------------------------------------------------------------------------*/
+cpl_matrix *
+muse_cplmatrix_extract_selected(const cpl_matrix *aMatrix,
+                                const cpl_array *aIndices)
+{
+  cpl_ensure(aMatrix, CPL_ERROR_NULL_INPUT, NULL);
+  cpl_ensure(aIndices, CPL_ERROR_NULL_INPUT, NULL);
+  cpl_ensure(cpl_array_get_type(aIndices) == CPL_TYPE_SIZE,
+             CPL_ERROR_INVALID_TYPE, NULL);
+
+  cpl_size maxpos = cpl_matrix_get_nrow(aMatrix) * cpl_matrix_get_ncol(aMatrix);
+  cpl_size sz = cpl_array_get_size(aIndices);
+  cpl_matrix *subset = cpl_matrix_new(1, sz);
+
+  double *sdata = cpl_matrix_get_data(subset);
+  const double *mdata = cpl_matrix_get_data_const(aMatrix);
+  const cpl_size *idata = cpl_array_get_data_cplsize_const(aIndices);
+
+  cpl_size i;
+  for (i = 0; i < sz; ++i) {
+    if ((idata[i] >= 0) && (idata[i] < maxpos)) {
+      *sdata++ = mdata[idata[i]];
+    }
+  }
+  return subset;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Compute the average absolute deviation of a (constant) vector.
@@ -829,7 +959,7 @@ muse_cpltable_load(const char *aFile, const char *aExtension,
   }
   cpl_msg_debug(__func__, "Loading %s['%s'] from extension %d", aFile,
                 aExtension, extension);
-  cpl_table *tbl = cpl_table_load(aFile, extension, 2);
+  cpl_table *tbl = cpl_table_load(aFile, extension, 1);
   if (muse_cpltable_check(tbl, aDefinition) != CPL_ERROR_NONE) {
     cpl_table_delete(tbl);
     return NULL;
@@ -1187,7 +1317,9 @@ muse_cplarray_erase_invalid(cpl_array *aArray)
   cpl_ensure_code(aArray != NULL, CPL_ERROR_NULL_INPUT);
   cpl_size n = cpl_array_get_size(aArray);
   cpl_size n_val = n - cpl_array_count_invalid(aArray);
+#if 0 /* DEBUG */
   cpl_msg_debug(__func__, "size = %li, %li valid", (long)n, (long)n_val);
+#endif
   if (n_val == n) {
     return CPL_ERROR_NONE;
   }
@@ -2024,12 +2156,17 @@ muse_cplparameterlist_from_propertylist(const cpl_propertylist *aHeader,
     } /* else */
     char *parname = cpl_sprintf("muse.%s.%s", recipe,
                                 cpl_propertylist_get_string(aHeader, kwname));
-    cpl_parameter *par = cpl_parameter_new_value(parname, type,
-                                                 cpl_propertylist_get_comment(aHeader, kwname),
-                                                 context);
-    cpl_free(parname);
+    const char *comment = cpl_property_get_comment(prop),
+               *comname = cpl_propertylist_get_comment(aHeader, kwname);
+    char defstr[41] = "true"; /* string with the default value */
+    if (comment) {
+      sscanf(comment, "Default: %40s", defstr);
+    }
+    cpl_parameter *par = NULL;
     switch (type) {
     case CPL_TYPE_BOOL:
+      par = cpl_parameter_new_value(parname, type, comname, context,
+                                    !strncmp(defstr, "true", 5) ? CPL_TRUE : CPL_FALSE);
       if (!strncmp(value, "true", 5)) {
         cpl_parameter_set_bool(par, CPL_TRUE);
       } else {
@@ -2037,19 +2174,25 @@ muse_cplparameterlist_from_propertylist(const cpl_propertylist *aHeader,
       }
       break;
     case CPL_TYPE_INT:
+      par = cpl_parameter_new_value(parname, type, comname, context, atoi(defstr));
       cpl_parameter_set_int(par, atoi(value));
       break;
     case CPL_TYPE_DOUBLE:
+      par = cpl_parameter_new_value(parname, type, comname, context, atof(defstr));
       cpl_parameter_set_double(par, atof(value));
       break;
     case CPL_TYPE_STRING:
+      par = cpl_parameter_new_value(parname, type, comname, context, defstr);
       cpl_parameter_set_string(par, value);
       break;
     default:
+      /* cannot do anything */
+      par = cpl_parameter_new_value(parname, type, comname, context, 0);
       cpl_msg_warning(__func__, "property of type %s (%d) found!",
                       cpl_type_get_name(type), type);
     } /* switch */
     cpl_parameterlist_append(parlist, par);
+    cpl_free(parname);
     cpl_free(kwname);
     cpl_free(kwvalue);
   } /* for npar */
@@ -2254,7 +2397,7 @@ muse_cplerrorstate_dump_some(unsigned aCurrent, unsigned aFirst, unsigned aLast)
   const cpl_boolean is_reverse = aFirst > aLast ? CPL_TRUE : CPL_FALSE;
   const char *revmsg = is_reverse ? " in reverse order" : "";
   const unsigned newest = is_reverse ? aFirst : aLast,
-                 nmax = labs(aLast - aFirst) + 1;
+                 nmax = labs((long int)aLast - (long int)aFirst) + 1;
   unsigned ndump = 20;
   if (getenv("MUSE_CPL_ERRORSTATE_NDUMP") &&
       atoi(getenv("MUSE_CPL_ERRORSTATE_NDUMP")) > 0) {
