@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *           (C) 2001 Enrico Marchetti, European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
@@ -679,9 +679,9 @@ muse_dar_check(muse_pixtable *aPixtable, double *aMaxShift,
   if (aCube) {
     *aCube = cube;
   }
-  double crpix3 = cpl_propertylist_get_double(cube->header, "CRPIX3"),
-         crval3 = cpl_propertylist_get_double(cube->header, "CRVAL3"),
-         cdelt3 = cpl_propertylist_get_double(cube->header, "CD3_3");
+  double crpix3 = muse_pfits_get_crpix(cube->header, 3),
+         crval3 = muse_pfits_get_crval(cube->header, 3),
+         cdelt3 = muse_pfits_get_cd(cube->header, 3, 3);
   cpl_errorstate state = cpl_errorstate_get(); /* header may be missing */
   double lambdaref = cpl_propertylist_get_double(aPixtable->header,
                                                  MUSE_HDR_PT_DAR_NAME),
