@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -331,10 +331,10 @@ muse_datacube_collapse(muse_datacube *aCube, cpl_table *aFilter)
       ny = cpl_image_get_size_y(cpl_imagelist_get(aCube->data, 0)),
       nz = cpl_imagelist_get_size(aCube->data);
   /* resample the filter curve on the wavelength grid */
-  double crpix3 = cpl_propertylist_get_double(aCube->header, "CRPIX3"),
-         crval3 = cpl_propertylist_get_double(aCube->header, "CRVAL3"),
-         cd33 = cpl_propertylist_get_double(aCube->header, "CD3_3");
-  const char *ctype3 = cpl_propertylist_get_string(aCube->header, "CTYPE3");
+  double crpix3 = muse_pfits_get_crpix(aCube->header, 3),
+         crval3 = muse_pfits_get_crval(aCube->header, 3),
+         cd33 = muse_pfits_get_cd(aCube->header, 3, 3);
+  const char *ctype3 = muse_pfits_get_ctype(aCube->header, 3);
   cpl_boolean loglambda = ctype3 && !strncmp(ctype3, "AWAV-LOG", 9);
   int l1 = 0, l2 = nz; /* loop boundaries for filter operation */
   double *fdata = muse_datacube_collapse_filter_buffer(aFilter, crval3, crpix3,
@@ -415,6 +415,10 @@ muse_datacube_collapse(muse_datacube *aCube, cpl_table *aFilter)
   @param    aNames      names of all filters used to create the images
   @return   CPL_ERROR_NONE or the relevant cpl_error_code on error
 
+  Saving of the BUNIT keywords is modeled after @ref muse_image_save(), i.e.
+  the data extension uses the incoming BUNIT entry (if present), the variance
+  extension gets the squared unit, and the data quality does not get any BUNIT.
+
   @error{return CPL_ERROR_NULL_INPUT, aFilename is NULL}
   @error{immediately return CPL_ERROR_NONE, aImages or aNames are NULL}
  */
@@ -434,6 +438,12 @@ muse_datacube_save_recimages(const char *aFilename, muse_imagelist *aImages,
 
     /* create small header with EXTNAME=filtername and WCS for images */
     cpl_propertylist *header = cpl_propertylist_new();
+    cpl_errorstate es = cpl_errorstate_get();
+    const char *unit = muse_pfits_get_bunit(image->header),
+               *ucomment = cpl_propertylist_get_comment(image->header, "BUNIT");
+    if (!cpl_errorstate_is_equal(es) && !unit) {
+      cpl_errorstate_set(es); /* swallow errors, if there is no unit to propagate */
+    }
     char dataext[KEYWORD_LENGTH], obj[KEYWORD_LENGTH],
          *dqext = NULL, *statext = NULL;
     snprintf(dataext, KEYWORD_LENGTH, "%s", cpl_array_get_string(aNames, i));
@@ -446,6 +456,12 @@ muse_datacube_save_recimages(const char *aFilename, muse_imagelist *aImages,
     snprintf(obj, KEYWORD_LENGTH, "%s", cpl_array_get_string(aNames, i));
     cpl_propertylist_append_string(header, "EXTNAME", dataext);
     cpl_propertylist_set_comment(header, "EXTNAME", "reconstructed image (data values)");
+    /* propagate the unit in the same way as in muse_image_save() */
+    if (unit) {
+      cpl_propertylist_append_string(header, "BUNIT", unit);
+      cpl_propertylist_set_comment(header, "BUNIT", ucomment);
+    }
+    /* update the OBJECT header with the filter name */
     muse_utils_copy_modified_header(image->header, header, "OBJECT", obj);
     cpl_propertylist_update_string(header, MUSE_HDR_FILTER,
                                    cpl_array_get_string(aNames, i));
@@ -458,16 +474,22 @@ muse_datacube_save_recimages(const char *aFilename, muse_imagelist *aImages,
     if (image->dq) {
       cpl_propertylist_update_string(header, "EXTNAME", dqext);
       cpl_propertylist_set_comment(header, "EXTNAME", "reconstructed image (bad pixel status values)");
+      cpl_propertylist_erase(header, "BUNIT"); /* no unit for data quality */
       snprintf(obj, KEYWORD_LENGTH, "%s, %s", cpl_array_get_string(aNames, i),
                EXTNAME_DQ);
       muse_utils_copy_modified_header(image->header, header, "OBJECT", obj);
       muse_utils_set_hduclass(header, "QUALITY", dataext, dqext, statext);
-      rc = cpl_image_save(image->dq, aFilename, CPL_TYPE_UNSPECIFIED, header,
+      rc = cpl_image_save(image->dq, aFilename, CPL_TYPE_INT, header,
                           CPL_IO_EXTEND);
     }
     if (image->stat) {
       cpl_propertylist_update_string(header, "EXTNAME", statext);
       cpl_propertylist_set_comment(header, "EXTNAME", "reconstructed image (variance)");
+      if (unit) {
+        char *ustat = cpl_sprintf("(%s)**2", unit); /* variance in squared units */
+        cpl_propertylist_append_string(header, "BUNIT", ustat);
+        cpl_free(ustat);
+      }
       snprintf(obj, KEYWORD_LENGTH, "%s, %s", cpl_array_get_string(aNames, i),
                EXTNAME_STAT);
       muse_utils_copy_modified_header(image->header, header, "OBJECT", obj);
@@ -507,7 +529,7 @@ muse_euro3dcube_save(muse_euro3dcube *aEuro3D, const char *aFilename)
 
   /* save primary header and the data table in the first extension */
   cpl_error_code rc = cpl_table_save(aEuro3D->dtable, aEuro3D->header,
-                                     aEuro3D->hdata, aFilename, CPL_IO_DEFAULT);
+                                     aEuro3D->hdata, aFilename, CPL_IO_CREATE);
   if (rc != CPL_ERROR_NONE) {
     cpl_msg_warning(__func__, "failed to save data part of the Euro3D table: "
                     "%s", cpl_error_get_message());
@@ -717,7 +739,7 @@ muse_datacube_save(muse_datacube *aCube, const char *aFilename)
     cpl_propertylist_append_string(header, "EXTNAME", EXTNAME_STAT);
     cpl_propertylist_set_comment(header, "EXTNAME", EXTNAME_STAT_COMMENT);
     /* add the correct BUNIT, depending on the data units */
-    if (!strncmp(cpl_propertylist_get_string(aCube->header, "BUNIT"),
+    if (!strncmp(muse_pfits_get_bunit(aCube->header),
                  kMuseFluxUnitString, strlen(kMuseFluxUnitString) + 1)) {
       /* flux calibrated data */
       cpl_propertylist_append_string(header, "BUNIT", kMuseFluxStatString);
@@ -773,8 +795,14 @@ muse_datacube_load_header(const char *aFilename)
             the DATA extension.
   @remark   Both DQ and STAT components of the file are optional.
   @warning  This function ignores the ESO-format headers like HDUCLASS.
-  @todo XXX Extensions for reconstructed images of the form <filter>_DQ and
-            <filter>_STAT are not handled properly!
+  @note     When loading extra reconstructed images from the cube file, this
+            function assumes that DQ and STAT extensions belonging to a
+            reconstructed image are in the extensions directly after the
+            main image extension for that reconstructed image. I.e., if an
+            image extension "filt1" exists, then the corresponding "filt1_DQ"
+            and/or "filt1_STAT" extensions have to follow right after it.
+            Since this is what @ref muse_datacube_save() saves to disk, it
+            should be true in most cases.
 
   @error{set CPL_ERROR_NULL_INPUT\, return NULL, aFilename is NULL}
   @error{set CPL_ERROR_ILLEGAL_INPUT\, return NULL, aFilename does not exist}
@@ -815,7 +843,22 @@ muse_datacube_load(const char *aFilename)
     muse_image *image = muse_image_new();
     image->header = cpl_propertylist_load(aFilename, ext);
     image->data = cpl_image_load(aFilename, CPL_TYPE_UNSPECIFIED, 0, ext);
-    /* XXX forget about image_DQ and image_STAT for the moment, they are unlikely... */
+    /* search for an load DQ and STAT extensions of this image, if present */
+    const char *extname = muse_pfits_get_extname(image->header);
+    char *extname2 = cpl_sprintf("%s_DQ", extname);
+    int ext2 = cpl_fits_find_extension(aFilename, extname2);
+    cpl_free(extname2);
+    if (ext2 > 0) {
+      image->dq = cpl_image_load(aFilename, CPL_TYPE_INT, 0, ext2);
+      ext = ext2; /* skip this as a normal reconstructed image */
+    }
+    extname2 = cpl_sprintf("%s_STAT", extname);
+    ext2 = cpl_fits_find_extension(aFilename, extname2);
+    cpl_free(extname2);
+    if (ext2 > 0) {
+      image->stat = cpl_image_load(aFilename, CPL_TYPE_UNSPECIFIED, 0, ext2);
+      ext = ext2; /* skip this as a normal reconstructed image */
+    }
 
     if (!cube->recnames) {
       cube->recnames = cpl_array_new(1, CPL_TYPE_STRING);
@@ -824,7 +867,7 @@ muse_datacube_load(const char *aFilename)
     }
     /* append name of this reconstructed image at the end of the names array */
     cpl_array_set_string(cube->recnames, cpl_array_get_size(cube->recnames) - 1,
-                         muse_pfits_get_extname(image->header));
+                         extname);
     if (!cube->recimages) {
       cube->recimages = muse_imagelist_new();
     }
@@ -835,6 +878,100 @@ muse_datacube_load(const char *aFilename)
   return cube;
 } /* muse_datacube_load() */
 
+/*---------------------------------------------------------------------------*/
+/**
+  @brief    Concatenate one datacube at the end of another one.
+  @param    aCube     the cube to append to
+  @param    aAppend   the cube to append
+  @return   CPL_ERROR_NONE on success, another CPL error code on failure
+
+  @note The reconstructed images present in aCube are most likely meaningless
+        after concatenating. Hence, they are removed by this function!
+  @note A logarithmic dispersion axis is currently not supported.
+
+  @error{return and set CPL_ERROR_NULL_INPUT, aCube and/or aAppend are NULL}
+  @error{return and set CPL_ERROR_ILLEGAL_INPUT,
+         image planes of the cubes have different sizes}
+  @error{return and set CPL_ERROR_ILLEGAL_INPUT,
+         sampling in dispersion direction is different and/or not adjacent}
+  @error{return and set CPL_ERROR_ILLEGAL_INPUT,
+         cpl_imagelist sizes in one of the input cubes do not match between data and stat components}
+ */
+/*---------------------------------------------------------------------------*/
+cpl_error_code
+muse_datacube_concat(muse_datacube *aCube, const muse_datacube *aAppend)
+{
+  cpl_ensure_code(aCube && aAppend, CPL_ERROR_NULL_INPUT);
+
+  cpl_size ndata = cpl_imagelist_get_size(aCube->data),
+           nstat = cpl_imagelist_get_size(aCube->stat);
+  cpl_ensure_code(ndata == nstat, CPL_ERROR_ILLEGAL_INPUT);
+  cpl_size i, n = cpl_imagelist_get_size(aAppend->data);
+  cpl_ensure_code(n == cpl_imagelist_get_size(aAppend->stat),
+                  CPL_ERROR_ILLEGAL_INPUT);
+
+  /* check sizes of the image planes */
+  cpl_size nx1 = cpl_image_get_size_x(cpl_imagelist_get(aCube->data, ndata - 1)),
+           ny1 = cpl_image_get_size_y(cpl_imagelist_get(aCube->data, ndata - 1)),
+           nx2 = cpl_image_get_size_x(cpl_imagelist_get(aAppend->data, 0)),
+           ny2 = cpl_image_get_size_y(cpl_imagelist_get(aAppend->data, 0));
+  cpl_ensure_code(nx1 == nx2 && ny1 == ny2, CPL_ERROR_ILLEGAL_INPUT);
+  const char *ctype1 = muse_pfits_get_ctype(aCube->header, 3),
+             *ctype2 = muse_pfits_get_ctype(aCube->header, 3);
+  cpl_ensure_code(ctype1 && ctype2, CPL_ERROR_UNSUPPORTED_MODE);
+  cpl_ensure_code(!strncmp(ctype1, "AWAV", 5) && !strncmp(ctype2, "AWAV", 5),
+                  CPL_ERROR_UNSUPPORTED_MODE);
+  /* compute wavelengths of last plane of first cube and first plane *
+   * of the cube to append, to check that they are adjacent          */
+  double pix1 = muse_pfits_get_crpix(aCube->header, 3),
+         val1 = muse_pfits_get_crval(aCube->header, 3),
+         cd1 = muse_pfits_get_cd(aCube->header, 3, 3),
+         pix2 = muse_pfits_get_crpix(aAppend->header, 3),
+         val2 = muse_pfits_get_crval(aAppend->header, 3),
+         cd2 = muse_pfits_get_cd(aAppend->header, 3, 3),
+         lbda1 = ((ndata - 1) + 1. - pix1) * cd1 + val1,
+         lbda2 = (0 + 1. - pix2) * cd2 + val2;
+  /* AWAV-LOG axes need more complex handling, so exclude them above */
+  cpl_msg_debug(__func__, "lambdas: %f %f (%f %f)", lbda1, lbda2, cd1, cd2);
+  cpl_ensure_code(fabs(cd1 - cd2) < DBL_EPSILON &&
+                  fabs(lbda2 - cd2 - lbda1) < DBL_EPSILON,
+                  CPL_ERROR_ILLEGAL_INPUT);
+
+  /* reconstructed images are now obsolete, remove them */
+  cpl_array_delete(aCube->recnames);
+  aCube->recnames = NULL;
+  muse_imagelist_delete(aCube->recimages);
+  aCube->recimages = NULL;
+
+  /* Check if there is a usable DQ component in both cubes. If so, *
+   * propagate them as the other two components, otherwise delete  *
+   * the incoming DQ of the first (= output) cube.                 */
+  cpl_boolean usedq = CPL_FALSE;
+  if (aCube->dq && cpl_imagelist_get_size(aCube->dq) == ndata &&
+      aAppend->dq && cpl_imagelist_get_size(aAppend->dq) == n) {
+    usedq = CPL_TRUE;
+  } else {
+    cpl_imagelist_delete(aCube->dq);
+    aCube->dq = NULL;
+  }
+
+  /* append the CPL imagelists */
+  for (i = 0; i < n; i++) {
+    cpl_imagelist_set(aCube->data,
+                      cpl_image_duplicate(cpl_imagelist_get(aAppend->data, i)),
+                      cpl_imagelist_get_size(aCube->data));
+    if (usedq) {
+      cpl_imagelist_set(aCube->dq,
+                        cpl_image_duplicate(cpl_imagelist_get(aAppend->dq, i)),
+                        cpl_imagelist_get_size(aCube->dq));
+    }
+    cpl_imagelist_set(aCube->stat,
+                      cpl_image_duplicate(cpl_imagelist_get(aAppend->stat, i)),
+                      cpl_imagelist_get_size(aCube->stat));
+  }
+  return CPL_ERROR_NONE;
+} /* muse_datacube_concat() */
+
 /*---------------------------------------------------------------------------*/
 /**
   @brief    Deallocate memory associated to a muse_datacube object.
