@@ -178,6 +178,46 @@ muse_imagelist_set(muse_imagelist *aList, muse_image *aImage, unsigned int aIdx)
   return CPL_ERROR_NONE;
 } /* muse_imagelist_set() */
 
+/*---------------------------------------------------------------------------*/
+/**
+  @brief  Unset the muse_image at given list index from the list.
+  @param  aList    the image list
+  @param  aIdx     index of the image to unset, starting at 0
+  @return The pointer to the removed muse_image or NULL on failure.
+
+  If the index points to an existing muse_image in the list, that will be
+  erased from the list. The image at that position is returned by this
+  function so that the user can deallocate it, if no longer needed.
+  The rest of the images are moved to the previous position in the list
+  and the size of the list is decreased by one.
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL, aList is NULL}
+  @error{set CPL_ERROR_ACCESS_OUT_OF_RANGE\, return NULL,
+         aIdx does not exist in the list}
+ */
+/*---------------------------------------------------------------------------*/
+muse_image *
+muse_imagelist_unset(muse_imagelist *aList, unsigned int aIdx)
+{
+  cpl_ensure(aList, CPL_ERROR_NULL_INPUT, NULL);
+  cpl_ensure(aIdx < aList->size, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
+
+  /* keep the image pointer around to return it below */
+  muse_image *image = aList->list[aIdx];
+
+  /* move the rest of the images one position backwards */
+  unsigned int k;
+  for (k = aIdx; k < aList->size - 1; k++) {
+    aList->list[k] = aList->list[k + 1];
+  }
+  /* NULL out the last pointer to be sure... */
+  aList->list[aList->size - 1] = NULL;
+  /* one image less */
+  aList->size--;
+
+  return image;
+} /* muse_imagelist_unset() */
+
 /*---------------------------------------------------------------------------*/
 /**
   @brief  Check that all images in the muse_imagelist have the same size.
