@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -193,6 +193,24 @@ muse_pfits_get_extname(const cpl_propertylist *aHeaders)
   return value;
 } /* muse_pfits_get_extname() */
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the unit string
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or NULL on error
+
+  Queries FITS header BUNIT
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_bunit(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, "BUNIT");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  return value;
+} /* muse_pfits_get_bunit() */
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    find out the size of a given axis
@@ -352,6 +370,118 @@ muse_pfits_get_exptime(const cpl_propertylist *aHeaders)
   return value;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the WCS reference point
+  @param    aHeaders       property list/headers to read from
+  @param    aAxis          the axis (the "i" of CRPIXi)
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header CRPIXi
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_crpix(const cpl_propertylist *aHeaders, unsigned int aAxis)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "CRPIX%u", aAxis);
+  const double value = cpl_propertylist_get_double(aHeaders, keyword);
+  /* default to 0.0 as per FITS Standard v3.0 */
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the WCS coordinate at the reference point
+  @param    aHeaders       property list/headers to read from
+  @param    aAxis          the axis (the "i" of CRVALi)
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header CRVALi
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_crval(const cpl_propertylist *aHeaders, unsigned int aAxis)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "CRVAL%u", aAxis);
+  const double value = cpl_propertylist_get_double(aHeaders, keyword);
+  /* default to 0.0 as per FITS Standard v3.0 */
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the WCS coordinate at the reference point
+  @param    aHeaders       property list/headers to read from
+  @param    aAxisI         the first axis (the "i" of CDi_j)
+  @param    aAxisJ         the second axis (the "j" of CDi_j)
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header CDi_j
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_cd(const cpl_propertylist *aHeaders, unsigned int aAxisI,
+                  unsigned int aAxisJ)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "CD%u_%u", aAxisI, aAxisJ);
+  const double value = cpl_propertylist_get_double(aHeaders, keyword);
+  /* default to 0.0 as per FITS Standard v3.0 */
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the WCS axis type string
+  @param    aHeaders       property list/headers to read from
+  @param    aAxis          the axis (the "i" of CTYPEi)
+  @return   the requested value or " " on error
+
+  Queries FITS header CTYPEi
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_ctype(const cpl_propertylist *aHeaders, unsigned int aAxis)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "CTYPE%u", aAxis);
+  const char *value = cpl_propertylist_get_string(aHeaders, keyword);
+  /* default to single space as per FITS Standard v3.0 */
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), " ");
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the WCS axis unit string
+  @param    aHeaders       property list/headers to read from
+  @param    aAxis          the axis (the "i" of CUNITi)
+  @return   the requested value or " " on error
+
+  Queries FITS header CUNITi
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_cunit(const cpl_propertylist *aHeaders, unsigned int aAxis)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "CUNIT%u", aAxis);
+  const char *value = cpl_propertylist_get_string(aHeaders, keyword);
+  /* default to single space as per FITS Standard v3.0 */
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), " ");
+  return value;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    find out the readout mode id
@@ -1010,6 +1140,96 @@ muse_pfits_get_parang_end(const cpl_propertylist *aHeaders)
   return value;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the x-FWHM average value from the auto-guider (in arcsec)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO OCS SGS AG FWHMX AVG
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_agx_avg(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO OCS SGS AG FWHMX AVG");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the x-FWHM root mean square from the auto-guider (in arcsec)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO OCS SGS AG FWHMX RMS
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_agx_rms(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO OCS SGS AG FWHMX RMS");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the y-FWHM average value from the auto-guider (in arcsec)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO OCS SGS AG FWHMY AVG
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_agy_avg(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO OCS SGS AG FWHMY AVG");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the y-FWHM root mean square from the auto-guider (in arcsec)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO OCS SGS AG FWHMY RMS
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_agy_rms(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO OCS SGS AG FWHMY RMS");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the image analysis FWHM from a linear fit (in arcsec)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO TEL IA FWHM
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_ia_fwhm(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL IA FWHM");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    find out the MUSE derotator mode
