@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 
 #include "muse_pixgrid.h"
 
+#include "muse_pfits.h"
 #include "muse_quality.h"
 #include "muse_utils.h"
 #include "muse_wcs.h"
@@ -182,18 +183,18 @@ muse_pixgrid_create(muse_pixtable *aPixtable, cpl_propertylist *aHeader,
              wcstype == MUSE_PIXTABLE_WCS_PIXEL, CPL_ERROR_UNSUPPORTED_MODE,
              NULL);
 
-  double crval3 = cpl_propertylist_get_double(aHeader, "CRVAL3"),
-         crpix3 = cpl_propertylist_get_double(aHeader, "CRPIX3"),
-         cd33 = cpl_propertylist_get_double(aHeader, "CD3_3");
+  double crval3 = muse_pfits_get_crval(aHeader, 3),
+         crpix3 = muse_pfits_get_crpix(aHeader, 3),
+         cd33 = muse_pfits_get_cd(aHeader, 3, 3);
   muse_wcs *wcs = muse_wcs_new(aHeader);
   wcs->iscelsph = wcstype == MUSE_PIXTABLE_WCS_CELSPH;
-  cpl_boolean loglambda = !strncmp(cpl_propertylist_get_string(aHeader, "CTYPE3"),
+  cpl_boolean loglambda = !strncmp(muse_pfits_get_ctype(aHeader, 3),
                                    "AWAV-LOG", 9);
   /* get all (relevant) table columns for easy pointer access */
   double ptxoff = 0., ptyoff = 0.;
   if (wcs->iscelsph) {
-    ptxoff = cpl_propertylist_get_double(aPixtable->header, "CRVAL1");
-    ptyoff = cpl_propertylist_get_double(aPixtable->header, "CRVAL2");
+    ptxoff = muse_pfits_get_crval(aPixtable->header, 1);
+    ptyoff = muse_pfits_get_crval(aPixtable->header, 2);
   }
   float *xpos = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_XPOS),
         *ypos = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_YPOS),
