@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,11 @@
 #include "muse_utils.h"
 #include "muse_wcs.h"
 
+#include "muse_lsf_params.h"
+#ifdef USE_LSF_PARAMS
+#include "muse_sky_old.h"
+#endif
+
 /*----------------------------------------------------------------------------*/
 /**
  * @defgroup muse_postproc       Post-processing functions
@@ -101,7 +106,12 @@ muse_postproc_properties_delete(muse_postproc_properties *aProp)
   cpl_table_delete(aProp->extinction);
   cpl_table_delete(aProp->telluric);
   cpl_propertylist_delete(aProp->wcs);
-  muse_sky_master_delete(aProp->sky);
+  muse_lsf_cube_delete_all(aProp->lsf_cube);
+#ifdef USE_LSF_PARAMS
+  muse_lsf_params_delete_all(aProp->lsf_params);
+#endif
+  cpl_table_delete(aProp->sky_lines);
+  cpl_table_delete(aProp->sky_continuum);
   muse_mask_delete(aProp->sky_mask);
   cpl_frame_delete(aProp->refframe);
   cpl_table_delete(aProp->tellregions);
@@ -316,7 +326,7 @@ muse_postproc_get_weight_type(const char *aWeightString)
          position in aHeader is more than aErrLimit away from all reference sources}
  */
 /*----------------------------------------------------------------------------*/
-static cpl_table *
+cpl_table *
 muse_postproc_load_nearest(const cpl_propertylist *aHeader,
                            const cpl_frame *aFrame, float aWarnLimit,
                            float aErrLimit, double *aRA, double *aDEC)
@@ -496,9 +506,8 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
 
   cpl_table *sky_cont = NULL,
             *sky_lines = NULL;
-  if (muse_pixtable_is_fluxcal(pt) &&
-      aProp->skymethod == MUSE_POSTPROC_SKYMETHOD_MODEL &&
-      aProp->sky && aProp->sky->lines) {
+  if ((aProp->skymethod == MUSE_POSTPROC_SKYMETHOD_MODEL && aProp->sky_lines) ||
+      (aProp->skymethod == MUSE_POSTPROC_SKYMETHOD_SIMPLE)) {
     /* Process optional sky mask */
     muse_pixtable *sky_pt = muse_pixtable_duplicate(pt);
     if (aProp->sky_mask) {
@@ -511,7 +520,7 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
     cpl_msg_indent_more();
     muse_resampling_params *params = muse_resampling_params_new(MUSE_RESAMPLE_WEIGHTED_DRIZZLE);
     params->crtype = MUSE_RESAMPLING_CRSTATS_MEDIAN;
-    params->crsigma = 15.;
+    params->crsigma = aProp->skymodel_params.crsigmac;
     muse_datacube *cube = muse_resampling_cube(sky_pt, params, NULL);
     if (!cube) {
       cpl_msg_error(__func__, "Could not create cube for whitelight image");
@@ -523,56 +532,85 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
     muse_image *whitelight = muse_datacube_collapse(cube, fwhite);
     cpl_msg_indent_less();
     muse_mask *sky_mask = muse_sky_create_skymask(whitelight,
+                                                  aProp->skymodel_params.ignore,
                                                   aProp->skymodel_params.fraction,
             /* XXX hardcode prefix for now... */  "ESO QC SCIPOST");
 
     /* Apply mask and create spectrum */
     cpl_table_select_all(sky_pt->table);
     muse_pixtable_and_selected_mask(sky_pt, sky_mask);
-    cpl_table *spectrum = muse_resampling_spectrum(sky_pt, aProp->skymodel_params.sampling);
+    cpl_table *spectrum = muse_resampling_spectrum(sky_pt,
+                                                   aProp->skymodel_params.sampling);
+    if (aProp->skymodel_params.crsigmas > 0.) {
+      cpl_msg_info(__func__, "Recreating sky spectrum with %.3f sigma level CR "
+                   "rejection", aProp->skymodel_params.crsigmas);
+      muse_sky_mark_cosmic(spectrum, sky_pt, aProp->skymodel_params.crsigmas);
+      cpl_table_delete(spectrum);
+      spectrum = muse_resampling_spectrum(sky_pt, aProp->skymodel_params.sampling);
+    } /* if aProp->skymodel_params.crsigmas positive */
 
-    /* Fit sky lines spectrum */
-    cpl_array *lambda = muse_cpltable_extract_column(spectrum, "lambda");
-    double lambda_low = cpl_array_get_min(lambda);
-    double lambda_high = cpl_array_get_max(lambda);
-    /* duplicate input sky lines, to crop and save them, for this one exposure */
-    sky_lines = cpl_table_duplicate(aProp->sky->lines);
-    muse_sky_lines_set_range(sky_lines, lambda_low-5, lambda_high+5);
+    if (aProp->skymethod == MUSE_POSTPROC_SKYMETHOD_MODEL) {
+      /* Fit sky lines spectrum */
+      cpl_array *lambda = muse_cpltable_extract_column(spectrum, "lambda");
+      double lambda_low = cpl_array_get_min(lambda);
+      double lambda_high = cpl_array_get_max(lambda);
+      sky_lines = cpl_table_duplicate(aProp->sky_lines);
+      muse_sky_lines_set_range(sky_lines, lambda_low-5, lambda_high+5);
 
-    cpl_array *data = muse_cpltable_extract_column(spectrum, "data");
-    cpl_array *stat = muse_cpltable_extract_column(spectrum, "stat");
-    /* do the fit, ignoring possible errors */
-    prestate = cpl_errorstate_get();
-    muse_sky_master *master = muse_sky_master_fit(lambda, data, stat, sky_lines);
-    if (!cpl_errorstate_is_equal(prestate)) {
-      cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
-      cpl_errorstate_set(prestate);
-    }
-    cpl_table_delete(sky_lines);
-    sky_lines = cpl_table_duplicate(master->lines); /* keep the modified lines */
+      // do the fit, ignoring possible errors
+      prestate = cpl_errorstate_get();
+      if (aProp->lsf_cube != NULL) {
+        cpl_image *lsfImage = muse_lsf_average_cube_all(aProp->lsf_cube, sky_pt);
+        muse_wcs *lsfWCS = muse_lsf_cube_get_wcs_all(aProp->lsf_cube);
+        /* Filter the image with a rectangle to model the spectrum binning */
+        muse_lsf_fold_rectangle(lsfImage, lsfWCS, aProp->skymodel_params.sampling);
 
-    /* Create continuum */
-    if (!aProp->sky->continuum) {
-      cpl_msg_info(__func__, "No sky continuum given, create it from the data");
-      /* remove the continuum from the master sky object,    *
-       * otherwise it messes up the continuum creation here! */
-      cpl_table_delete(master->continuum);
-      master->continuum = NULL;
-      muse_sky_subtract_pixtable(sky_pt, master, aProp->sky->lsf);
-      sky_cont = muse_resampling_spectrum(sky_pt, aProp->skymodel_params.csampling);
-      cpl_table_erase_column(sky_cont, "stat");
-      cpl_table_erase_column(sky_cont, "dq");
-      cpl_table_name_column(sky_cont, "data", "flux");
+        muse_sky_lines_fit(spectrum, sky_lines, lsfImage, lsfWCS);
+        if (!cpl_errorstate_is_equal(prestate)) {
+          cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
+          cpl_errorstate_set(prestate);
+        }
+
+        /* Create continuum */
+        if (aProp->sky_continuum == NULL) {
+          cpl_msg_info(__func__, "No sky continuum given, create it from the data");
+          sky_cont = muse_sky_continuum_create(spectrum, sky_lines, lsfImage, lsfWCS,
+                                               aProp->skymodel_params.csampling);
+        } else {
+          cpl_msg_info(__func__, "Using sky continuum given as input");
+        }
+
+        /* clean up */
+        cpl_image_delete(lsfImage);
+#ifdef USE_LSF_PARAMS
+      } else if (aProp->lsf_params != NULL) {// Old LSF params code
+        // do the fit, ignoring possible errors
+        muse_sky_lines_fit_old(spectrum, sky_lines);
+        if (!cpl_errorstate_is_equal(prestate)) {
+          cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
+          cpl_errorstate_set(prestate);
+        }
+
+        if (aProp->sky_continuum == NULL) {
+          cpl_msg_info(__func__, "No sky continuum given, create it from the data");
+          muse_sky_subtract_lines_old(sky_pt, sky_lines, aProp->lsf_params);
+          aProp->sky_continuum
+            = muse_resampling_spectrum(sky_pt, aProp->skymodel_params.csampling);
+          cpl_table_erase_column(aProp->sky_continuum, "stat");
+          cpl_table_erase_column(aProp->sky_continuum, "dq");
+          cpl_table_name_column(aProp->sky_continuum, "data", "flux");
+        }
+#endif
+      }
+      cpl_array_unwrap(lambda);
     } else {
-      cpl_msg_info(__func__, "Using sky continuum given as input");
-      sky_cont = cpl_table_duplicate(aProp->sky->continuum);
-    }
-
-    /* clean up */
-    cpl_array_unwrap(lambda);
-    cpl_array_unwrap(data);
-    cpl_array_unwrap(stat);
-    muse_sky_master_delete(master);
+      /* simple subtraction: pretend that the spectrum is a     *
+       * continuum and use that function for direct subtraction */
+      cpl_table *spec2 = cpl_table_duplicate(spectrum);
+      cpl_table_name_column(spec2, "data", "flux");
+      rc = muse_sky_subtract_continuum(pt, spec2);
+      cpl_table_delete(spec2);
+    } /* else: simple */
     if (aSkyOut) {
       aSkyOut->mask = sky_mask;
       aSkyOut->spectrum = spectrum;
@@ -585,35 +623,40 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
     muse_resampling_params_delete(params);
     muse_datacube_delete(cube);
     muse_pixtable_delete(sky_pt);
-  } /* if MUSE_POSTPROC_SKYMETHOD_MODEL */
+  } /* if MUSE_POSTPROC_SKYMETHOD_MODEL or MUSE_POSTPROC_SKYMETHOD_SIMPLE */
   /* In the following we do the sky subtraction, if we have the products *
    * for it. The SKYMETHOD_NONE here reflects the subtract-model option  *
    * of the muse_scipost recipe.                                         */
-  if (muse_pixtable_is_fluxcal(pt) && aProp->sky &&
-      ((aProp->skymethod == MUSE_POSTPROC_SKYMETHOD_NONE) ||
-       (aProp->skymethod == MUSE_POSTPROC_SKYMETHOD_MODEL))) {
-    /* create a new (partially duplicate) master sky that we are going to use */
-    muse_sky_master *sky = muse_sky_master_new();
-    /* leave sky->lsf NULL, since that is not used for subtraction */
-    if (aProp->sky->continuum) {
+  if ((aProp->skymethod == MUSE_POSTPROC_SKYMETHOD_NONE) ||
+      (aProp->skymethod == MUSE_POSTPROC_SKYMETHOD_MODEL)) {
+    if (aProp->sky_continuum) {
       cpl_msg_debug(__func__, "doing sky continuum subtraction with input "
                     "spectrum");
-      sky->continuum = cpl_table_duplicate(aProp->sky->continuum);
+      rc = muse_sky_subtract_continuum(pt, aProp->sky_continuum);
     } else if (sky_cont) {
       cpl_msg_debug(__func__, "doing sky continuum subtraction with created "
                     "spectrum");
-      sky->continuum = cpl_table_duplicate(sky_cont);
+      rc = muse_sky_subtract_continuum(pt, sky_cont);
     }
     if (sky_lines) {
       cpl_msg_debug(__func__, "doing sky line subtraction with created list");
-      sky->lines = cpl_table_duplicate(sky_lines);
-    } else if (aProp->sky->lines) {
+      if (aProp->lsf_cube != NULL) {
+        rc = muse_sky_subtract_lines(pt, sky_lines, aProp->lsf_cube);
+#ifdef USE_LSF_PARAMS
+      } else if (aProp->lsf_params != NULL) {
+        rc = muse_sky_subtract_lines_old(pt, sky_lines, aProp->lsf_params);
+#endif
+     }
+    } else if (aProp->sky_lines) {
       cpl_msg_debug(__func__, "doing sky line subtraction with input list");
-      sky->lines = cpl_table_duplicate(aProp->sky->lines);
+      if (aProp->lsf_cube != NULL) {
+        rc = muse_sky_subtract_lines(pt, aProp->sky_lines, aProp->lsf_cube);
+#ifdef USE_LSF_PARAMS
+      } else if (aProp->lsf_params != NULL) {
+        rc = muse_sky_subtract_lines_old(pt, aProp->sky_lines, aProp->lsf_params);
+#endif
+      }
     }
-    cpl_msg_debug(__func__, "doing sky subtraction by spectrum modeling");
-    rc = muse_sky_subtract_pixtable(pt, sky, aProp->sky->lsf);
-    muse_sky_master_delete(sky);
   } /* if sky modeling */
   /* either transfer the sky lines and continuum out or clean *
    * them up; both should still work, if one or both are NULL */
@@ -1008,9 +1051,26 @@ muse_postproc_qc_fwhm(muse_processing *aProcessing, muse_datacube *aCube)
   cpl_errorstate prestate = cpl_errorstate_get();
   cpl_apertures *apertures = cpl_apertures_extract(cim, vsigmas, &isigma);
   cpl_vector_unwrap(vsigmas);
+
+  /* add the first QC parameters, no matter what the detection did */
+  char keyword[KEYWORD_LENGTH];
+  cpl_boolean loglambda = !strncmp(muse_pfits_get_ctype(aCube->header, 3),
+                                   "AWAV-LOG", 9);
+  double crpix3 = muse_pfits_get_crpix(aCube->header, 3),
+         cd33 = muse_pfits_get_cd(aCube->header, 3, 3),
+         crval3 = muse_pfits_get_crval(aCube->header, 3),
+         lambda = loglambda
+                ? crval3 * exp((plane + 1. - crpix3) * cd33 / crval3)
+                : (plane + 1. - crpix3) * cd33 + crval3;
+  snprintf(keyword, KEYWORD_LENGTH, QC_POST_LAMBDA, prefix);
+  cpl_propertylist_update_float(aCube->header, keyword, lambda);
+
+  /* now see, if we detected anything */
   if (!apertures || !cpl_errorstate_is_equal(prestate)) {
-    /* just set up two fake entries, with number zero and negative FWHMs: */
-    char keyword[KEYWORD_LENGTH];
+    /* set up number and a fake entry (source number zero and negative FWHMs) */
+    snprintf(keyword, KEYWORD_LENGTH, QC_POST_NDET, prefix);
+    cpl_propertylist_update_int(aCube->header, keyword, 0); /* zero! */
+
     snprintf(keyword, KEYWORD_LENGTH, QC_POST_POSX, prefix, 0);
     cpl_propertylist_update_float(aCube->header, keyword, -1.);
     snprintf(keyword, KEYWORD_LENGTH, QC_POST_POSY, prefix, 0);
@@ -1025,11 +1085,10 @@ muse_postproc_qc_fwhm(muse_processing *aProcessing, muse_datacube *aCube)
                     "%.1f sigma limit on plane %d, QC parameters will not "
                     "contain useful information", dsigmas[ndsigmas-1], plane+1);
     return CPL_ERROR_DATA_NOT_FOUND;
-  }
+  } /* if no detections */
   int ndet = cpl_apertures_get_size(apertures);
-  cpl_msg_info(__func__, "Computing FWHM QC parameters for %d source%s found "
-               "down to the %.1f sigma threshold on plane %d", ndet,
-               ndet == 1 ? "" : "s", dsigmas[isigma], plane + 1);
+  snprintf(keyword, KEYWORD_LENGTH, QC_POST_NDET, prefix);
+  cpl_propertylist_update_int(aCube->header, keyword, ndet); /* zero! */
 
   /* get some kind of WCS for conversion of FWHM from pixels to arcsec, *
    * by default assume WFM and x=RA and y=DEC                           */
@@ -1040,16 +1099,10 @@ muse_postproc_qc_fwhm(muse_processing *aProcessing, muse_datacube *aCube)
   if (!cpl_errorstate_is_equal(es)) {
     cpl_errorstate_set(es); /* swallow possible errors from creating the WCS */
   }
-  if (!wcs ||
-      !strncasecmp(cpl_propertylist_get_string(aCube->header, "CTYPE1"),
-                   "PIXEL", 5)) {
-    cpl_msg_debug(__func__, "QC1 FWHM parameter estimation (%d sources): "
-                  "simple conversion to arcsec (CTYPE=%s/%s)!", ndet,
-                  cpl_propertylist_get_string(aCube->header, "CTYPE1"),
-                  cpl_propertylist_get_string(aCube->header, "CTYPE2"));
+  if (!wcs || !strncasecmp(muse_pfits_get_ctype(aCube->header, 1), "PIXEL", 5)) {
     if (muse_pfits_get_mode(aCube->header) > MUSE_MODE_WFM_AO_N) { /* NFM scaling */
-      cd11 = kMuseSpaxelSizeX_WFM;
-      cd22 = kMuseSpaxelSizeY_WFM;
+      cd11 = kMuseSpaxelSizeX_NFM;
+      cd22 = kMuseSpaxelSizeY_NFM;
     }
   } else {
     const cpl_matrix *cd = cpl_wcs_get_cd(wcs);
@@ -1058,46 +1111,86 @@ muse_postproc_qc_fwhm(muse_processing *aProcessing, muse_datacube *aCube)
     cd12 = fabs(cpl_matrix_get(cd, 0, 1)) * 3600.,
     cd21 = fabs(cpl_matrix_get(cd, 1, 0)) * 3600.,
     cd22 = fabs(cpl_matrix_get(cd, 1, 1)) * 3600.;
-    cpl_msg_debug(__func__, "QC1 FWHM parameter estimation (%d sources): "
-                  "full conversion to arcsec (CD=%f,%f,%f,%f)", ndet,
-                  cd11, cd12, cd21, cd22);
   }
   cpl_wcs_delete(wcs); /* rely on internal NULL check */
 
-  /* Compute FWHM of all apertures */
+  /* Compute FWHM of all apertures; collect them *
+   * in an image for statistical computations    */
+  cpl_image *statimage = cpl_image_new(ndet, 2, CPL_TYPE_DOUBLE);
 #if 0
   printf("index RA_FWHM DEC_FWHM\n");
 #endif
-  int n;
+  int n, nbad = 0;
   for (n = 1; n <= ndet; n++) {
     cpl_size xcen = lround(cpl_apertures_get_centroid_x(apertures, n)),
              ycen = lround(cpl_apertures_get_centroid_y(apertures, n));
     double x, y;
     cpl_image_get_fwhm(cim, xcen, ycen, &x, &y);
-
-    /* linear WCS scaling */
-    x = cd11 * x + cd12 * y;
-    y = cd22 * y + cd21 * x;
+    const char *fwhmcomment = NULL;
+    if (x < 0 || y < 0) { /* something's weird with this source */
+      x = -1.;
+      y = -1.;
+      /* do not count these in the statistics */
+      cpl_image_reject(statimage, n, 1);
+      cpl_image_reject(statimage, n, 2);
+      fwhmcomment = "[arcsec] failure determining FWHM";
+      nbad++;
+    } else {
+      /* linear WCS scaling */
+      x = cd11 * x + cd12 * y;
+      y = cd22 * y + cd21 * x;
+      cpl_image_set(statimage, n, 1, x);
+      cpl_image_set(statimage, n, 2, y);
+    }
 #if 0
     printf("%4d  %f %f\n", n, x, y);
 #endif
 
     /* generate and write FITS keywords */
-    char keyword[KEYWORD_LENGTH];
     snprintf(keyword, KEYWORD_LENGTH, QC_POST_POSX, prefix, n);
     cpl_propertylist_update_float(aCube->header, keyword, xcen);
     snprintf(keyword, KEYWORD_LENGTH, QC_POST_POSY, prefix, n);
     cpl_propertylist_update_float(aCube->header, keyword, ycen);
     snprintf(keyword, KEYWORD_LENGTH, QC_POST_FWHMX, prefix, n);
     cpl_propertylist_update_float(aCube->header, keyword, x);
+    if (fwhmcomment) {
+      cpl_propertylist_set_comment(aCube->header, keyword, fwhmcomment);
+    }
     snprintf(keyword, KEYWORD_LENGTH, QC_POST_FWHMY, prefix, n);
     cpl_propertylist_update_float(aCube->header, keyword, y);
+    if (fwhmcomment) {
+      cpl_propertylist_set_comment(aCube->header, keyword, fwhmcomment);
+    }
   } /* for n (aperture number) */
 #if 0
   fflush(stdout);
 #endif
   cpl_apertures_delete(apertures);
 
+  /* add simple FWHM statistics as well */
+  snprintf(keyword, KEYWORD_LENGTH, QC_POST_FWHM_NVALID, prefix);
+  cpl_propertylist_update_int(aCube->header, keyword, ndet - nbad);
+  cpl_stats *s = cpl_stats_new_from_image(statimage,
+                                          CPL_STATS_MEDIAN | CPL_STATS_MAD);
+  snprintf(keyword, KEYWORD_LENGTH, QC_POST_FWHM_MEDIAN, prefix);
+  if (ndet < 3) {
+    cpl_propertylist_update_float(aCube->header, keyword, 0.);
+  } else {
+    cpl_propertylist_update_float(aCube->header, keyword, cpl_stats_get_median(s));
+  }
+  snprintf(keyword, KEYWORD_LENGTH, QC_POST_FWHM_MAD, prefix);
+  if (ndet < 3) {
+    cpl_propertylist_update_float(aCube->header, keyword, 0.);
+  } else {
+    cpl_propertylist_update_float(aCube->header, keyword, cpl_stats_get_mad(s));
+  }
+  cpl_stats_delete(s);
+  cpl_image_delete(statimage);
+  cpl_msg_info(__func__, "Computed FWHM QC parameters for %d of %d source%s "
+               "found down to the %.1f sigma threshold on plane %d",
+               ndet - nbad, ndet, ndet == 1 ? "" : "s", dsigmas[isigma],
+               plane + 1);
+
   return CPL_ERROR_NONE;
 } /* muse_postproc_qc_fwhm() */
 
