@@ -490,7 +490,7 @@ muse_processing_save_image(muse_processing *aProcessing, int aIFU,
   @param  aCube         the cube to be saved
   @param  aTag          tag to apply for output frames, used as default prefix
   @param  aType         type of cube to save (MUSE_CUBE_TYPE_FITS,
-                        MUSE_CUBE_TYPE_EURO3D)
+                        MUSE_CUBE_TYPE_EURO3D, or MUSE_CUBE_TYPE_LSF)
   @return CPL_ERROR_NONE or the relevant cpl_error_code on error
 
   Save a datacube to disk. The filename consists on the prefix, a "-"
@@ -502,14 +502,13 @@ muse_processing_save_image(muse_processing *aProcessing, int aIFU,
   @error{CPL_ERROR_ILLEGAL_INPUT, an unknown aType was given}
  */
 /*---------------------------------------------------------------------------*/
-/* XXX removed MUSE_CUBE_TYPE_LSF from doxygen */
 cpl_error_code
 muse_processing_save_cube(muse_processing *aProcessing, int aIFU,
                           void *aCube, const char *aTag, muse_cube_type aType)
 {
   cpl_ensure_code(aProcessing && aCube && aTag, CPL_ERROR_NULL_INPUT);
-  cpl_ensure_code(aType == MUSE_CUBE_TYPE_EURO3D || aType == MUSE_CUBE_TYPE_FITS/* XXX ||
-                  aType == MUSE_CUBE_TYPE_LSF */, CPL_ERROR_ILLEGAL_INPUT);
+  cpl_ensure_code(aType == MUSE_CUBE_TYPE_EURO3D || aType == MUSE_CUBE_TYPE_FITS ||
+                  aType == MUSE_CUBE_TYPE_LSF, CPL_ERROR_ILLEGAL_INPUT);
 
   /* both cube structures contain the header component as  *
    * first element, so it's safe to cast it to one of them */
@@ -527,8 +526,8 @@ muse_processing_save_cube(muse_processing *aProcessing, int aIFU,
     rc = muse_datacube_save((muse_datacube *)aCube,
                             cpl_frame_get_filename(frame));
   } else {
-    //XXX rc = muse_lsf_cube_save((muse_lsf_cube *)aCube,
-    //XXX                         cpl_frame_get_filename(frame));
+    rc = muse_lsf_cube_save((muse_lsf_cube *)aCube,
+                            cpl_frame_get_filename(frame));
   }
   if (rc == CPL_ERROR_NONE) {
     cpl_frameset_insert(aProcessing->outframes, frame);
@@ -757,8 +756,9 @@ muse_processing_check_intags(muse_processing *aProcessing, const char *aTag,
   @return CPL_ERROR_NONE on success or a cpl_error_code on any error.
 
   Check the input frames for completeness. We check the min and max
-  of all input and calibration files. On error, a message will be
-  printed.
+  of all raw input files and the calibration files associated to the raw inputs.
+  On error, a message will be printed and the relevant CPL error code will be
+  returned.
 
   @error{return CPL_ERROR_NULL_INPUT, aProcessing is NULL}
   @error{return CPL_ERROR_ILLEGAL_INPUT, recipe config not found in aProcessing}
@@ -780,6 +780,7 @@ muse_processing_check_input(muse_processing *aProcessing, unsigned char aIFU)
     return CPL_ERROR_ILLEGAL_INPUT;
   }
 
+  cpl_boolean output = CPL_TRUE; /* start with outputting debug messages */
   cpl_size itag, ntags = cpl_array_get_size(aProcessing->intags);
   unsigned int errors = 0;
   for (itag = 0; itag < ntags; itag++) {
@@ -807,7 +808,7 @@ muse_processing_check_input(muse_processing *aProcessing, unsigned char aIFU)
       errors++;
       continue;
     }
-    /* check all other tags */
+    /* check all other tags related to this raw input file */
     int i;
     for (i = 0; tags[i]; i++) {
       frames = muse_frameset_find(aProcessing->inframes, tags[i], aIFU,
@@ -818,22 +819,29 @@ muse_processing_check_input(muse_processing *aProcessing, unsigned char aIFU)
       nmin = cpl_recipeconfig_get_min_count(recipeconfig, intag, tags[i]);
       nmax = cpl_recipeconfig_get_max_count(recipeconfig, intag, tags[i]);
       if (nmin >= 0 && nframes < nmin) {
-        cpl_msg_error(__func__, "Required %d, found %d frames with tag \"%s\" "
-                      "with IFU %hhu", nmin, nframes, tags[i], aIFU);
+        if (output) {
+          cpl_msg_error(__func__, "Required %d, found %d frames with tag \"%s\" "
+                        "with IFU %hhu", nmin, nframes, tags[i], aIFU);
+        }
         errors++;
       }
       if (nframes == 0 && nmin <= 0) {
-        cpl_msg_debug(__func__, "Optional frame with tag \"%s\" not given",
-                      tags[i]);
+        if (output) {
+          cpl_msg_debug(__func__, "Optional frame with tag \"%s\" not given",
+                        tags[i]);
+        }
       }
       if (nmax >= 0 && nframes > nmax) {
-        cpl_msg_error(__func__, "At most %d allowed, found %d frames with tag "
-                      "\"%s\" with IFU %hhu", nmax, nframes, tags[i], aIFU);
+        if (output) {
+          cpl_msg_error(__func__, "At most %d allowed, found %d frames with tag "
+                        "\"%s\" with IFU %hhu", nmax, nframes, tags[i], aIFU);
+        }
         errors++;
       }
       cpl_free(tags[i]);
     } /* for i (all tags) */
     cpl_free(tags);
+    output = CPL_FALSE; /* printing the output for one raw file is enough */
   } /* for itag */
 
   if (errors) {
@@ -897,6 +905,11 @@ muse_processing_sort_exposures(muse_processing *aProcessing)
 
       /* load the primary header and get DATE-OBS and the IFU number */
       cpl_propertylist *header = cpl_propertylist_load(filename, 0);
+      if (!header) {
+        cpl_msg_warning(__func__, "\"%s\" could not be loaded, it will be "
+                        "ignored!", filename);
+        continue; /* skip on to the next frame */
+      }
       const char *date = muse_pfits_get_dateobs(header);
       if (!date) {
         cpl_msg_warning(__func__, "\"%s\" does not contain the DATE-OBS "
