@@ -321,4 +321,113 @@ muse_processing_prepare_property(cpl_propertylist *aHeader, const char *aName,
   return rc;
 }
 
+/*---------------------------------------------------------------------------*/
+/**
+  @brief  Output main pipeline configuration, inputs, and parameters.
+  @param  aPlugin   the CPL plugin of the recipe in question
+
+  Output pipeline name (PACKAGE_NAME) and version (PACKAGE_VERSION) and if
+  aPlugin is non-NULL and the message or log levels are set to debug, give all
+  input frames (filenames and tags), non-default parameters, and relevant
+  environment variables.
+
+  This function is meant to be called at the beginning of a MUSE recipe.
+ */
+/*---------------------------------------------------------------------------*/
+void
+muse_processing_recipeinfo(cpl_plugin *aPlugin)
+{
+  cpl_msg_set_threadid_off();
+  cpl_msg_info(__func__, "%s v%s", PACKAGE_NAME, PACKAGE_VERSION);
+  if (!aPlugin) {
+    return;
+  }
+  if (cpl_msg_get_level() != CPL_MSG_DEBUG &&
+      cpl_msg_get_log_level() != CPL_MSG_DEBUG) {
+    return;
+  }
+  cpl_recipe *recipe = (cpl_recipe *)aPlugin;
+  cpl_size nframes = cpl_frameset_get_size(recipe->frames);
+  cpl_msg_debug(__func__, "%"CPL_SIZE_FORMAT" input frames:", nframes);
+
+  cpl_msg_indent_more();
+  cpl_size iframe;
+  for (iframe = 0; iframe < nframes; iframe++) {
+    cpl_frame *frame = cpl_frameset_get_position(recipe->frames, iframe);
+    cpl_msg_debug(__func__, "%s\t%s", cpl_frame_get_filename(frame),
+                  cpl_frame_get_tag(frame));
+  } /* for iframe (all frames) */
+  cpl_msg_indent_less();
+
+  // print non-default parameters
+  cpl_msg_debug(__func__, "non-default parameters:");
+  cpl_msg_indent_more();
+  int n = 0; /* count non-default parameters */
+  const cpl_parameter *p = cpl_parameterlist_get_first(recipe->parameters);
+  for ( ; p; p = cpl_parameterlist_get_next(recipe->parameters)) {
+    /* the alias is the short name, and we always set the *
+     * same one for both command line and configuration   */
+    const char *name = cpl_parameter_get_alias(p, CPL_PARAMETER_MODE_CLI);
+    cpl_type type = cpl_parameter_get_type(p);
+
+     switch (type) {
+     case CPL_TYPE_BOOL: {
+       cpl_boolean value = cpl_parameter_get_bool(p),
+                   dval = cpl_parameter_get_default_bool(p);
+       if (value != dval) {
+         cpl_msg_debug(__func__, "--%s = %s [%s]", name,
+                       value ? "true" : "false", dval ? "true" : "false");
+         n++;
+       }
+       break;
+     }
+     case CPL_TYPE_INT: {
+       int value = cpl_parameter_get_int(p),
+           dval = cpl_parameter_get_default_int(p);
+       if (value != dval) {
+         cpl_msg_debug(__func__, "--%s = %d [%d]", name, value, dval);
+         n++;
+       }
+       break;
+     }
+     case CPL_TYPE_DOUBLE: {
+       double value = cpl_parameter_get_double(p),
+              dval = cpl_parameter_get_default_double(p);
+       if (value != dval) {
+         cpl_msg_debug(__func__, "--%s = %g [%g]", name, value, dval);
+         n++;
+       }
+       break;
+     }
+     case CPL_TYPE_STRING: {
+       const char *value = cpl_parameter_get_string(p),
+                  *dval = cpl_parameter_get_default_string(p);
+       if (value && dval && strncmp(value, dval, strlen(dval) + 1)) {
+         cpl_msg_debug(__func__, "--%s = %s [%s]", name, value, dval);
+         n++;
+       }
+       break;
+     }
+     default:
+       cpl_msg_debug(__func__, "--%s: parameter of unknown type!", name);
+       break;
+     } /* switch */
+  } /* for ipar (all parameters) */
+  if (!n) { /* if no non-default parameters were found */
+    cpl_msg_debug(__func__, "none");
+  }
+  cpl_msg_indent_less();
+
+  cpl_msg_debug(__func__, "relevant environment variables:");
+  cpl_msg_indent_more();
+  /* print all MUSE* environment variables */
+  /* XXX not sure how to do that */
+  /* print OMP_NUM_THREADS variable */
+  char *omp_num_threads = getenv("OMP_NUM_THREADS");
+  if (omp_num_threads) {
+    cpl_msg_debug(__func__, "OMP_NUM_THREADS=%s", omp_num_threads);
+  }
+  cpl_msg_indent_less();
+} /* muse_processing_recipeinfo() */
+
 /**@}*/
