@@ -117,14 +117,14 @@ muse_quadrants_overscan_stats(muse_image *aImage, const char *aRejection,
         sigma = 5.0;
   if (reject == REJ_DCR) {
     char *rest = strchr(aRejection, ':');
-    if (strlen(aRejection) > 4 && rest) { /* try to access info after "dcr:" */
-      dcrbox1 = strtoul(++rest, &rest, 10);
-      if (strlen(rest) > 0) {
-        dcrbox2 = strtoul(++rest, &rest, 10); /* ++ to skip over the comma */
-        if (strlen(rest) > 0) {
-          dcrnpass = strtoul(++rest, &rest, 10);
-          if (strlen(rest) > 0) {
-            dcrthres = strtof(++rest, NULL);
+    if (strlen(aRejection) > 4 && rest++) { /* try to access info after "dcr:" */
+      dcrbox1 = strtoul(rest, &rest, 10);
+      if (strlen(rest++) > 0) { /* ++ to skip over the comma */
+        dcrbox2 = strtoul(rest, &rest, 10);
+        if (strlen(rest++) > 0) {
+          dcrnpass = strtoul(rest, &rest, 10);
+          if (strlen(rest++) > 0) {
+            dcrthres = strtof(rest, NULL);
           }
         }
       }
@@ -145,8 +145,8 @@ muse_quadrants_overscan_stats(muse_image *aImage, const char *aRejection,
     }
   } else if (reject == REJ_FIT) {
     char *rest = strchr(aRejection, ':');
-    if (strlen(aRejection) > 4 && rest) { /* try to access info after "fit:" */
-      sigma = strtof(++rest, NULL);
+    if (strlen(aRejection) > 4 && rest++) { /* try to access info after "fit:" */
+      sigma = strtof(rest, NULL);
     }
     /* check that box sizes in both directions stay inside overscan regions */
     if (sigma <= 0.) {
