@@ -803,7 +803,7 @@ muse_quality_copy_badpix_table(const char *aInFile, const char *aOutFile,
 
     cpl_table *tab = NULL;
     const char *xtension = cpl_propertylist_get_string(extheader, "XTENSION");
-    if (strncmp(xtension, "BINTABLE", 8)) {
+    if (xtension && strncmp(xtension, "BINTABLE", 8)) {
       if (cpl_msg_get_level() == CPL_MSG_DEBUG) {
         printf("WARNING: Not a binary table extension, skipping data section "
                "(extension %"CPL_SIZE_FORMAT")", i);
