@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,12 +27,14 @@
  *                             Includes                                       *
  *----------------------------------------------------------------------------*/
 #include <cpl.h>
+#include <string.h>
 
 #include "muse_xcombine.h"
 
 #include "muse_pfits.h"
 #include "muse_wcs.h"
 #include "muse_utils.h"
+#include "muse_data_format_z.h"
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -42,6 +44,128 @@
 
 /**@{*/
 
+/*----------------------------------------------------------------------------*/
+/**
+  @private
+  @brief   Create table with exposure information needed to compute exposure
+           weighting.
+  @param   aPixtables   the NULL-terminated array of pixel tables to weight
+  @return  the CPL table with the info on success
+
+  The columns "EXPTIME", "AG_AVG", "IA_FWHM", and "DIMM" can be used to compute
+  relative weighting of exposures. The column "AG_RMS" can be used as error
+  estimate on the "AG_AVG" column.
+
+  @note    This function only does minimal error checking.
+ */
+/*----------------------------------------------------------------------------*/
+static cpl_table *
+muse_xcombine_weights_infotable(muse_pixtable **aPixtables)
+{
+  unsigned int npt = 0;
+  while (aPixtables[npt++]) ; /* count tables, including first NULL table */
+  --npt; /* subtract NULL table */
+
+  /* create table for the values */
+  cpl_table *table = cpl_table_new(npt);
+  cpl_table_new_column(table, "EXPTIME", CPL_TYPE_DOUBLE);
+  cpl_table_new_column(table, "AGX_AVG", CPL_TYPE_DOUBLE);
+  cpl_table_new_column(table, "AGX_RMS", CPL_TYPE_DOUBLE);
+  cpl_table_new_column(table, "AGY_AVG", CPL_TYPE_DOUBLE);
+  cpl_table_new_column(table, "AGY_RMS", CPL_TYPE_DOUBLE);
+  cpl_table_new_column(table, "IA_FWHM", CPL_TYPE_DOUBLE);
+  cpl_table_new_column(table, "DIMM_START", CPL_TYPE_DOUBLE);
+  cpl_table_new_column(table, "DIMM_END", CPL_TYPE_DOUBLE);
+
+  /* fill the table with the values */
+  cpl_errorstate prestate = cpl_errorstate_get();
+  unsigned int i;
+  for (i = 0; i < npt; i++) {
+    /* exposure time */
+    cpl_errorstate state = cpl_errorstate_get();
+    double value = muse_pfits_get_exptime(aPixtables[i]->header);
+    if (cpl_errorstate_is_equal(state)) {
+      cpl_table_set_double(table, "EXPTIME", i, value);
+    }
+
+    /* AG values */
+    state = cpl_errorstate_get();
+    value = muse_pfits_get_agx_avg(aPixtables[i]->header);
+    if (cpl_errorstate_is_equal(state)) {
+      cpl_table_set_double(table, "AGX_AVG", i, value);
+    }
+    state = cpl_errorstate_get();
+    value = muse_pfits_get_agx_rms(aPixtables[i]->header);
+    if (cpl_errorstate_is_equal(state)) {
+      cpl_table_set_double(table, "AGX_RMS", i, value);
+    }
+    state = cpl_errorstate_get();
+    value = muse_pfits_get_agy_avg(aPixtables[i]->header);
+    if (cpl_errorstate_is_equal(state)) {
+      cpl_table_set_double(table, "AGY_AVG", i, value);
+    }
+    state = cpl_errorstate_get();
+    value = muse_pfits_get_agy_rms(aPixtables[i]->header);
+    if (cpl_errorstate_is_equal(state)) {
+      cpl_table_set_double(table, "AGY_RMS", i, value);
+    }
+
+    /* IA value */
+    state = cpl_errorstate_get();
+    value = muse_pfits_get_ia_fwhm(aPixtables[i]->header);
+    if (cpl_errorstate_is_equal(state)) {
+      cpl_table_set_double(table, "IA_FWHM", i, value);
+    }
+
+    /* DIMM values */
+    state = cpl_errorstate_get();
+    value = muse_pfits_get_fwhm_start(aPixtables[i]->header);
+    if (cpl_errorstate_is_equal(state) && value > 0.) {
+      cpl_table_set_double(table, "DIMM_START", i, value);
+    }
+    state = cpl_errorstate_get();
+    value = muse_pfits_get_fwhm_end(aPixtables[i]->header);
+    if (cpl_errorstate_is_equal(state) && value > 0.) {
+      cpl_table_set_double(table, "DIMM_END", i, value);
+    }
+  } /* for i (table index) */
+  cpl_errorstate_set(prestate); /* swallow all errors */
+
+  /* compute AG_AVG = (AGX + AGY) / 2 */
+  cpl_table_duplicate_column(table, "AG_AVG", table, "AGX_AVG");
+  cpl_table_add_columns(table, "AG_AVG", "AGY_AVG");
+  cpl_table_multiply_scalar(table, "AG_AVG", 0.5);
+  /* compute AG_RMS = sqrt(AGX_RMS**2 + AGY_RMS**2) */
+  cpl_table_duplicate_column(table, "AG_RMS", table, "AGX_RMS");
+  cpl_table_power_column(table, "AG_RMS", 2); /* **2 */
+  cpl_table_duplicate_column(table, "AG2_RMS", table, "AGY_RMS");
+  cpl_table_power_column(table, "AG2_RMS", 2); /* **2 */
+  cpl_table_add_columns(table, "AG_RMS", "AG2_RMS");
+  cpl_table_erase_column(table, "AG2_RMS");
+  cpl_table_power_column(table, "AG_RMS", 0.5); /* sqrt() */
+  /* compute DIMM = (DIMM_START + DIMM_END) / 2 */
+  cpl_table_duplicate_column(table, "DIMM", table, "DIMM_START");
+  cpl_table_add_columns(table, "DIMM", "DIMM_END");
+  cpl_table_multiply_scalar(table, "DIMM", 0.5);
+
+#if 0
+  printf("%s:\n", __func__);
+  const char *p, *cols[] = { "EXPTIME", "AG_AVG", "IA_FWHM", "DIMM", NULL };
+  for (i = 0, p = cols[i] ; p; p = cols[i++ + 1]) {
+    prestate = cpl_errorstate_get();
+    printf("%s: %.3f +/- %.3f (%"CPL_SIZE_FORMAT" invalid)\n", p,
+           cpl_table_get_column_mean(table, p), cpl_table_get_column_stdev(table, p),
+           cpl_table_count_invalid(table, p));
+    /* swallow errors due to completely invalid column */
+    cpl_errorstate_set(prestate);
+  }
+  cpl_table_dump(table, 0, npt, stdout);
+  fflush(stdout);
+#endif
+
+  return table;
+} /* muse_xcombine_weights_infotable() */
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief   compute the weights for combination of two or more exposures
@@ -73,8 +197,8 @@
          EXPTIME is not set in the header of the first pixel table}
   @error{weights for these tables are set to zero,
          the exposure time header is missing from one or more input pixel tables}
-  @error{FWHM-weights for these tables are set to 1.\, propagate errors\, output warning\, but return CPL_ERROR_NONE,
-         the FWHM header entries are missing from one or more input pixel tables although MUSE_XCOMBINE_FWHM is requrested}
+  @error{output warning and set CPL_ERROR_DATA_NOT_FOUND\, switch to EXPTIME-weighting\, but return CPL_ERROR_NONE,
+         all FWHM header entries are missing from one or more input pixel tables although MUSE_XCOMBINE_FWHM is requrested}
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code
@@ -94,40 +218,81 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
     return cpl_error_set(__func__, CPL_ERROR_UNSUPPORTED_MODE);
   }
 
-  cpl_msg_info(__func__, "%d tables to be weighted using %s", npt,
-               aWeighting == MUSE_XCOMBINE_EXPTIME ? "EXPTIME"
-                                                   : "EXPTIME & FWHM");
-  double exptime0 = muse_pfits_get_exptime(aPixtables[0]->header);
-  if (exptime0 == 0.0) {
+  /* create table with weighting information: exposure times, *
+   * and several seeing FWHM estimators from the FITS header  */
+  cpl_table *tinfo = muse_xcombine_weights_infotable(aPixtables);
+  int err;
+  double exptime0 = cpl_table_get_double(tinfo, "EXPTIME", 0, &err);
+  if (err || exptime0 == 0.0) {
+    cpl_table_delete(tinfo);
     return cpl_error_set(__func__, CPL_ERROR_INCOMPATIBLE_INPUT);
   }
-  double fwhm0 = aWeighting == MUSE_XCOMBINE_FWHM
-               ? (muse_pfits_get_fwhm_start(aPixtables[0]->header)
-                  + muse_pfits_get_fwhm_end(aPixtables[0]->header)) / 2.
-               : 1.;
+  /* select FWHM estimator for FWHM-based weighting */
+  double fwhm0 = 0.;
+  const char *fwhmcolumn = NULL,
+             *fwhmerrcol = NULL;
+  if (aWeighting == MUSE_XCOMBINE_FWHM) {
+    cpl_size ninvalid = cpl_table_count_invalid(tinfo, "AG_AVG");
+    if (ninvalid > 0) { /* if any entry is invalid */
+      ninvalid = cpl_table_count_invalid(tinfo, "IA_FWHM");
+      if (ninvalid > 0) { /* any more entry invalid */
+        ninvalid = cpl_table_count_invalid(tinfo, "DIMM");
+        if (!ninvalid) { /* success at last */
+          fwhmcolumn = "DIMM";
+          cpl_msg_info(__func__, "%d tables to be weighted using EXPTIME & FWHM "
+                       "(using DIMM measurements)", npt);
+        }
+      } else {
+        fwhmcolumn = "IA_FWHM";
+        cpl_msg_info(__func__, "%d tables to be weighted using EXPTIME & FWHM "
+                     "(using active optics image analysis)", npt);
+      }
+    } else {
+      fwhmcolumn = "AG_AVG";
+      fwhmerrcol = "AG_RMS";
+      cpl_msg_info(__func__, "%d tables to be weighted using EXPTIME & FWHM "
+                   "(using auto-guider info)", npt);
+    }
+    if (!fwhmcolumn) {
+      cpl_msg_warning(__func__, "%d tables to be weighted using EXPTIME.", npt);
+      cpl_msg_warning(__func__, "(FWHM-based weighting was requested but cannot"
+                      " be carried due to incomplete FITS headers in some "
+                      "exposures.)");
+      cpl_error_set_message(__func__, CPL_ERROR_DATA_NOT_FOUND, "missing FITS "
+                            "headers for FWHM-based exposure weighting");
+    } else {
+      fwhm0 = cpl_table_get_double(tinfo, fwhmcolumn, 0, &err);
+    }
+  } else {
+    /* simple exptime-time weighting */
+    cpl_msg_info(__func__, "%d tables to be weighted using EXPTIME.", npt);
+  }
 
   /* add and fill the "weight" column in all pixel tables */
   unsigned int i;
   for (i = 0; i < npt; i++) {
-    double exptime = muse_pfits_get_exptime(aPixtables[i]->header),
+    double exptime = cpl_table_get_double(tinfo, "EXPTIME", i, &err),
            weight = exptime / exptime0;
     if (!cpl_table_has_column(aPixtables[i]->table, MUSE_PIXTABLE_WEIGHT)) {
       cpl_table_new_column(aPixtables[i]->table, MUSE_PIXTABLE_WEIGHT,
                            CPL_TYPE_FLOAT);
     }
     /* modify the "weight" column depending on ambient seeing */
-    if (aWeighting == MUSE_XCOMBINE_FWHM) {
-      cpl_errorstate prestate = cpl_errorstate_get();
-      double fwhm = (muse_pfits_get_fwhm_start(aPixtables[i]->header)
-                     + muse_pfits_get_fwhm_end(aPixtables[i]->header)) / 2.;
-      if (fwhm == 0. || !cpl_errorstate_is_equal(prestate)) {
-        cpl_msg_warning(__func__, "No seeing info in table %d. Weighting it "
-                        "equal to first table!", i+1);
-        fwhm = fwhm0;
-      }
+    char *fwhmstr = NULL;
+    if (fwhmcolumn) {
+      double fwhm = cpl_table_get_double(tinfo, fwhmcolumn, i, &err),
+             fwhmerr = fwhmerrcol
+                     ? cpl_table_get_double(tinfo, fwhmerrcol, i, NULL) : 0.;
       weight *= fwhm0 / fwhm;
+      if (fwhmerrcol) {
+        fwhmstr = cpl_sprintf(", FWHM = %.2f +/- %.2f", fwhm, fwhmerr);
+      } else {
+        fwhmstr = cpl_sprintf(", FWHM = %.2f", fwhm);
+      }
     }
-    cpl_msg_debug(__func__, "Table %d, weight = %f", i+1, weight);
+    cpl_msg_debug(__func__, "Table %d, weight = %f (EXPTIME = %f%s)", i+1, weight,
+                  exptime, fwhmstr ? fwhmstr : "");
+    cpl_free(fwhmstr);
     cpl_table_fill_column_window_float(aPixtables[i]->table,
                                        MUSE_PIXTABLE_WEIGHT,
                                        0, muse_pixtable_get_nrow(aPixtables[i]),
@@ -138,14 +303,85 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
     cpl_propertylist_set_comment(aPixtables[i]->header, MUSE_HDR_PT_WEIGHTED,
                                  MUSE_HDR_PT_WEIGHTED_COMMENT);
   } /* for i (table index) */
+  cpl_table_delete(tinfo);
 
   return CPL_ERROR_NONE;
 } /* muse_xcombine_weights() */
 
+/*----------------------------------------------------------------------------*/
+/**
+  @private
+  @brief   Get offsets and scale from table row with matching DATE-OBS entry.
+  @param   aOffsets   table with coordinate offsets to apply
+  @param   aDateObs   the DATE-OBS string to compare
+  @return  a three-element double array, with the two coordinate offsets or NULL
+           on error
+  @note    The returned pointer has to be deallocated with cpl_free() after use.
+  @note    By default (if the optional MUSE_OFFSETS_FSCALE column does not
+           exist) or on error, the third element of the returned array is NAN, so
+           that no flux scaling occurs.
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL, aOffsets is NULL}
+  @error{set CPL_ERROR_ILLEGAL_INPUT\, return NULL,
+         aDateObs is NULL or has a length of less than 19 or more than 68 characters}
+  @error{set NAN for each invalid value\, output a warning,
+         the value(s) in the columns MUSE_OFFSETS_DRA and/or MUSE_OFFSETS_DDEC could not be read}
+ */
+/*----------------------------------------------------------------------------*/
+static double *
+muse_xcombine_find_offsets(const cpl_table *aOffsets, const char *aDateObs)
+{
+  const char *id = "muse_xcombine_tables"; /* pretend to be in that function */
+  cpl_ensure(aOffsets, CPL_ERROR_NULL_INPUT, NULL);
+  /* DATE-OBS format allowed by FITS are YYYY-MM-DDThh:mm:ss and optional *
+   * fractional seconds that fit within the 68 char FITS limit            */
+  cpl_ensure(aDateObs && strlen(aDateObs) >= 19 && strlen(aDateObs) <= 68,
+             CPL_ERROR_ILLEGAL_INPUT, NULL);
+
+  int ioff, noff = cpl_table_get_nrow(aOffsets);
+  for (ioff = 0; ioff < noff; ioff++) {
+    const char *dateobs = cpl_table_get_string(aOffsets, MUSE_OFFSETS_DATEOBS,
+                                               ioff);
+    /* compare DATE-OBS to the 23-char limit that's used in *
+     * normal ESO DATE keywords, that include milliseconds  */
+    if (dateobs && !strncmp(dateobs, aDateObs, 23)) {
+      double *offsets = cpl_calloc(3, sizeof(double));
+#if 0
+      cpl_msg_debug(__func__, "found:");
+      cpl_table_dump(aOffsets, ioff, 1, stdout);
+      fflush(stdout);
+#endif
+      int err;
+      offsets[0] = cpl_table_get_double(aOffsets, MUSE_OFFSETS_DRA, ioff, &err);
+      if (err) {
+        cpl_msg_warning(id, "%s for %s could not be read from %s!",
+                        MUSE_OFFSETS_DRA, aDateObs, MUSE_TAG_OFFSET_LIST);
+        offsets[0] = NAN;
+      }
+      offsets[1] = cpl_table_get_double(aOffsets, MUSE_OFFSETS_DDEC, ioff, &err);
+      if (err) {
+        cpl_msg_warning(id, "%s for %s could not be read from %s!",
+                        MUSE_OFFSETS_DDEC, aDateObs, MUSE_TAG_OFFSET_LIST);
+        offsets[1] = NAN;
+      }
+      offsets[2] = cpl_table_has_column(aOffsets, MUSE_OFFSETS_FSCALE)
+                 ? cpl_table_get_double(aOffsets, MUSE_OFFSETS_FSCALE, ioff, &err)
+                 : NAN; /* no scaling by default */
+      if (err) {
+        offsets[2] = NAN; /* fall back to no scaling */
+      }
+      return offsets;
+    }
+  } /* for ioff (all offset table rows) */
+  return NULL;
+} /* muse_xcombine_find_offsets() */
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief   combine the pixel tables of several exposures into one
   @param   aPixtables   the NULL-terminated array of pixel tables to combine
+  @param   aOffsets     a table with coordinate offsets to apply (optional, can
+                        be NULL)
   @return  a muse_pixtable * with the combined pixel table of all exposures
            or NULL on error
   @remark  The FITS headers as passed to this function through the
@@ -159,8 +395,8 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
            function.
   @warning Do not re-order the output table, otherwise the exposure ranges in
            the pixel table headers are out of sync!
-  @remark This function adds a FITS header (@ref MUSE_HDR_PT_COMBINED) with
-          the number of combined exposures, for information.
+  @remark  This function adds a FITS header (@ref MUSE_HDR_PT_COMBINED) with
+           the number of combined exposures, for information.
 
   Determine offsets from input FITS headers, loop through all pixel tables,
   position them to the RA,DEC in their headers using @ref
@@ -169,14 +405,19 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
   All used input pixel tables are deleted in this function, only the input
   pointer remains.
 
-  The behavior of this function can be tweaked using the environment variables
-  MUSE_XCOMBINE_RA_OFFSETS and MUSE_XCOMBINE_DEC_OFFSETS, so that one can align
-  exposures with different relative offsets not reflected in the RA and DEC
-  headers. If this is done, the header keywords MUSE_HDR_OFFSETi_DATEOBS,
+  The behavior of this function can be tweaked using the second argument, a
+  table listing offsets in RA and DEC for a given DATE-OBS. This can be used to
+  align exposures with different relative offsets not reflected in the RA and
+  DEC headers. If this is done, the header keywords MUSE_HDR_OFFSETi_DATEOBS,
   MUSE_HDR_OFFSETi_DRA, and MUSE_HDR_OFFSETi_DDEC are written to the header of
   the output pixel table, to show which RA and DEC offsets were applied to the
   exposure of which DATE-OBS.
 
+  Similarly, if aOffsets contains a column MUSE_OFFSETS_FSCALE, the values are
+  interpreted as scaling factors to apply to the given exposure before inserting
+  into the big pixel table. If flux scales are applied, they are stored in a
+  keyword MUSE_HDR_FLUX_SCALEi.
+
   @error{set CPL_ERROR_NULL_INPUT\, return NULL, aPixtables is NULL}
   @error{set CPL_ERROR_ILLEGAL_INPUT\, return NULL,
          there are less than 2 input pixel tables}
@@ -189,7 +430,7 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
  */
 /*----------------------------------------------------------------------------*/
 muse_pixtable *
-muse_xcombine_tables(muse_pixtable **aPixtables)
+muse_xcombine_tables(muse_pixtable **aPixtables, const cpl_table *aOffsets)
 {
   cpl_ensure(aPixtables, CPL_ERROR_NULL_INPUT, NULL);
   unsigned int npt = 0;
@@ -199,36 +440,6 @@ muse_xcombine_tables(muse_pixtable **aPixtables)
              CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
   cpl_msg_info(__func__, "%u tables to be combined", npt);
 
-  /* check for environment variables with exposure offsets (in deg) */
-  cpl_array *dra = NULL,
-            *ddec = NULL;
-  char *raenv = getenv("MUSE_XCOMBINE_RA_OFFSETS"),
-       *decenv = getenv("MUSE_XCOMBINE_DEC_OFFSETS");
-  if (raenv) {
-    dra = muse_cplarray_new_from_delimited_string(raenv, ",");
-    unsigned int nra = cpl_array_get_size(dra);
-    if (nra != npt) {
-      cpl_msg_warning(__func__, "Found %u RA offsets for %u exposures, not "
-                      "using them!", nra, npt);
-      cpl_array_delete(dra);
-      dra = NULL;
-    } else {
-      cpl_msg_info(__func__, "Using %u RA offsets", nra);
-    }
-  }
-  if (decenv) {
-    ddec = muse_cplarray_new_from_delimited_string(decenv, ",");
-    unsigned int ndec = cpl_array_get_size(ddec);
-    if (ndec != npt) {
-      cpl_msg_warning(__func__, "Found %u DEC offsets for %u exposures, not "
-                      "using them!", ndec, npt);
-      cpl_array_delete(ddec);
-      ddec = NULL;
-    } else {
-      cpl_msg_info(__func__, "Using %u DEC offsets", ndec);
-    }
-  }
-
   double timeinit = cpl_test_get_walltime(),
          cpuinit = cpl_test_get_cputime();
   muse_utils_memory_dump("muse_xcombine_tables() start");
@@ -254,33 +465,44 @@ muse_xcombine_tables(muse_pixtable **aPixtables)
   cpl_propertylist_set_comment(pt->header, keyword, comment);
 
   double ra0 =  muse_pfits_get_ra(pt->header),
-         dec0 = muse_pfits_get_dec(pt->header);
-  if (dra) {
-    double raoff = atof(cpl_array_get_string(dra, 0));
-    ra0 -= raoff;
-    /* store in the header of the output pixel table */
-    snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA, 1);
-    snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA_C, raoff * 3600.);
-    cpl_propertylist_append_double(pt->header, keyword, raoff);
-    cpl_propertylist_set_comment(pt->header, keyword, comment);
-  }
-  if (ddec) {
-    double decoff = atof(cpl_array_get_string(ddec, 0));
-    dec0 -= decoff;
-    /* store in the header of the output pixel table */
-    snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DDEC, 1);
-    snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DDEC_C, decoff * 3600.);
-    cpl_propertylist_append_double(pt->header, keyword, decoff);
-    cpl_propertylist_set_comment(pt->header, keyword, comment);
-  }
-  if (dra || ddec) {
-    /* store in the header of the output pixel table */
+         dec0 = muse_pfits_get_dec(pt->header),
+         *offsets = aOffsets
+                  ? muse_xcombine_find_offsets(aOffsets,
+                                               muse_pfits_get_dateobs(pt->header))
+                  : NULL;
+  if (offsets) {
+    if (isfinite(offsets[0]) && isfinite(offsets[1])) {
+      ra0 -= offsets[0];
+      dec0 -= offsets[1];
+      cpl_msg_debug(__func__, "Applying coordinate offsets to exposure 1: %e/%e"
+                    " deg", offsets[0], offsets[1]);
+      /* store in the header of the output pixel table */
+      snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA, 1);
+      snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA_C, offsets[0] * 3600.);
+      cpl_propertylist_append_double(pt->header, keyword, offsets[0]);
+      cpl_propertylist_set_comment(pt->header, keyword, comment);
+      snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DDEC, 1);
+      snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DDEC_C, offsets[1] * 3600.);
+      cpl_propertylist_append_double(pt->header, keyword, offsets[1]);
+      cpl_propertylist_set_comment(pt->header, keyword, comment);
+    } /* if spatial offsets */
+    if (isnormal(offsets[2])) { /* valid flux scale */
+      /* get and apply the scale */
+      cpl_msg_debug(__func__, "Scaling flux of exposure 1 by %g.", offsets[2]);
+      muse_pixtable_flux_multiply(pt, offsets[2]);
+      /* store in the header of the output pixel table */
+      snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_FLUX_SCALEi, 1);
+      cpl_propertylist_append_double(pt->header, keyword, offsets[2]);
+      cpl_propertylist_set_comment(pt->header, keyword, MUSE_HDR_FLUX_SCALEi_C);
+    } /* if scale */
+    /* store date of changed exposure in the header of the output pixel table */
     snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DATEOBS, 1);
     snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DATEOBS_C, 1);
     cpl_propertylist_append_string(pt->header, keyword,
                                    muse_pfits_get_dateobs(pt->header));
     cpl_propertylist_set_comment(pt->header, keyword, comment);
-  }
+  } /* if offsets */
+  cpl_free(offsets);
   muse_wcs_position_celestial(pt, ra0, dec0);
 
   unsigned int i, nskipped = 0;
@@ -297,39 +519,46 @@ muse_xcombine_tables(muse_pixtable **aPixtables)
                       muse_pfits_get_dateobs(aPixtables[i]->header));
     }
 
-    /* apply spherical coordinate rotation to coordinates of the first exposure */
+    /* apply spherical coordinate rotation to coordinates of this exposure */
     double ra = muse_pfits_get_ra(aPixtables[i]->header),
            dec = muse_pfits_get_dec(aPixtables[i]->header);
-    if (dra) {
-      double raoff = atof(cpl_array_get_string(dra, i));
-      ra -= raoff;
-      cpl_msg_debug(__func__, "positioning not to RA %f but to %f (dRA = %f "
-                    "deg)", ra + raoff, ra, raoff);
-      /* store in the header of the output pixel table */
-      snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA, i + 1);
-      snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA_C, raoff * 3600.);
-      cpl_propertylist_append_double(pt->header, keyword, raoff);
-      cpl_propertylist_set_comment(pt->header, keyword, comment);
-    }
-    if (ddec) {
-      double decoff = atof(cpl_array_get_string(ddec, i));
-      dec -= decoff;
-      cpl_msg_debug(__func__, "positioning not to DEC %f but to %f (dDEC = %f "
-                    "deg)", dec + decoff, dec, decoff);
-      /* store in the header of the output pixel table */
-      snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DDEC, i + 1);
-      snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DDEC_C, decoff * 3600.);
-      cpl_propertylist_append_double(pt->header, keyword, decoff);
-      cpl_propertylist_set_comment(pt->header, keyword, comment);
-    }
-    if (dra || ddec) {
+    offsets = aOffsets
+            ? muse_xcombine_find_offsets(aOffsets,
+                                         muse_pfits_get_dateobs(aPixtables[i]->header))
+            : NULL;
+    if (offsets) {
+      if (isfinite(offsets[0]) && isfinite(offsets[1])) {
+        ra -= offsets[0];
+        dec -= offsets[1];
+        cpl_msg_debug(__func__, "Applying coordinate offsets to exposure %d: "
+                      "%e/%e deg", i + 1, offsets[0], offsets[1]);
+        /* store in the header of the output pixel table */
+        snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA, i + 1);
+        snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA_C, offsets[0] * 3600.);
+        cpl_propertylist_append_double(pt->header, keyword, offsets[0]);
+        cpl_propertylist_set_comment(pt->header, keyword, comment);
+        snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DDEC, i + 1);
+        snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DDEC_C, offsets[1] * 3600.);
+        cpl_propertylist_append_double(pt->header, keyword, offsets[1]);
+        cpl_propertylist_set_comment(pt->header, keyword, comment);
+      } /* if spatial offsets */
+      if (isnormal(offsets[2])) { /* valid flux scale */
+        cpl_msg_debug(__func__, "Scaling flux of exposure %u by %g.", i + 1,
+                      offsets[2]);
+        muse_pixtable_flux_multiply(aPixtables[i], offsets[2]);
+        /* store in the header of the output pixel table */
+        snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_FLUX_SCALEi, i + 1);
+        cpl_propertylist_append_double(pt->header, keyword, offsets[2]);
+        cpl_propertylist_set_comment(pt->header, keyword, MUSE_HDR_FLUX_SCALEi_C);
+      } /* if scale */
       /* store in the header of the output pixel table */
       snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DATEOBS, i + 1);
       snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DATEOBS_C, i + 1);
       cpl_propertylist_append_string(pt->header, keyword,
                                      muse_pfits_get_dateobs(aPixtables[i]->header));
       cpl_propertylist_set_comment(pt->header, keyword, comment);
-    }
+    } /* if offsets */
+    cpl_free(offsets);
     muse_wcs_position_celestial(aPixtables[i], ra, dec);
 
     /* Shift the x/y coordinates depending on their relative zeropoint! */
@@ -379,8 +608,6 @@ muse_xcombine_tables(muse_pixtable **aPixtables)
     snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_PT_EXP_LST_COMMENT, i + 1);
     cpl_propertylist_set_comment(pt->header, keyword, comment);
   } /* for i (pixel tables) */
-  cpl_array_delete(dra);
-  cpl_array_delete(ddec);
   muse_pixtable_compute_limits(pt);
   /* update the merge-related status header */
   cpl_propertylist_update_int(pt->header, MUSE_HDR_PT_COMBINED, npt - nskipped);
