@@ -61,6 +61,11 @@ muse_create_sky_load_pixtable(muse_processing *aProcessing,
                                                          aParams->lambdamin,
                                                          aParams->lambdamax);
     cpl_table_delete(thisexp);
+    if (p == NULL) {
+      muse_pixtable_delete(pixtable);
+      pixtable = NULL;
+      break;
+    }
     /* erase pre-existing QC parameters */
     cpl_propertylist_erase_regexp(p->header, "ESO QC ", 0);
     if (pixtable == NULL) {
@@ -106,8 +111,7 @@ muse_create_sky_load_pixtable(muse_processing *aProcessing,
   cpl_table_delete(telluric);
 
   if ((pixtable != NULL) && (muse_pixtable_is_fluxcal(pixtable) != CPL_TRUE)) {
-    cpl_msg_error(__func__, "Pixel table not flux calibrated, cannot create "
-                  "sky!");
+    cpl_msg_error(__func__, "Pixel table not flux calibrated");
     muse_pixtable_delete(pixtable);
     pixtable = NULL;
   }
@@ -116,188 +120,41 @@ muse_create_sky_load_pixtable(muse_processing *aProcessing,
     cpl_table_and_selected_int(pixtable->table, MUSE_PIXTABLE_DQ,
                                CPL_NOT_EQUAL_TO, EURO3D_GOODPIXEL);
     cpl_table_erase_selected(pixtable->table);
-  }
 
-  /* do DAR correction for WFM data */
-  if (muse_pfits_get_mode(pixtable->header) <= MUSE_MODE_WFM_AO_N) {
-    cpl_msg_debug(__func__, "WFM detected: starting DAR correction");
-    cpl_error_code rc = muse_dar_correct(pixtable, aParams->lambdaref);
-    cpl_msg_debug(__func__, "DAR correction returned rc=%d: %s", rc,
-                  rc != CPL_ERROR_NONE ? cpl_error_get_message() : "");
-  } /* if WFM */
+    /* do DAR correction for WFM data */
+    if (muse_pfits_get_mode(pixtable->header) <= MUSE_MODE_WFM_AO_N) {
+      cpl_msg_debug(__func__, "WFM detected: starting DAR correction");
+      cpl_error_code rc = muse_dar_correct(pixtable, aParams->lambdaref);
+      cpl_msg_debug(__func__, "DAR correction returned rc=%d: %s", rc,
+                    rc != CPL_ERROR_NONE ? cpl_error_get_message() : "");
+    } /* if WFM */
+  }
 
   return pixtable;
 }
 
-static muse_sky_master *
-muse_create_sky_compute_master(cpl_table *spectrum, muse_processing *aProcessing)
-{
-  cpl_array *lambda = muse_cpltable_extract_column(spectrum, "lambda");
-  double lambda_low = cpl_array_get_min(lambda);
-  double lambda_high = cpl_array_get_max(lambda);
-  cpl_table *sky_lines = muse_sky_lines_load(aProcessing);
-  if (sky_lines != NULL) {
-    muse_sky_lines_set_range(sky_lines, lambda_low-5, lambda_high+5);
-  }
-
-  cpl_array *data = muse_cpltable_extract_column(spectrum, "data");
-  cpl_array *stat = muse_cpltable_extract_column(spectrum, "stat");
-  // do the fit, ignoring possible errors
-  cpl_errorstate prestate = cpl_errorstate_get();
-  muse_sky_master *res = muse_sky_master_fit(lambda, data, stat, sky_lines);
-  if (!cpl_errorstate_is_equal(prestate)) {
-    cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
-    cpl_errorstate_set(prestate);
-  }
-
-  cpl_table_delete(sky_lines);
-  cpl_array_unwrap(lambda);
-  cpl_array_unwrap(data);
-  cpl_array_unwrap(stat);
-  cpl_table_delete(spectrum);
-
-  return res;
-}
-
-static void
-muse_create_sky_save_master(cpl_propertylist *aHeader,
-                            const muse_sky_master *aMaster,
-                            muse_processing *aProcessing)
-{
-  cpl_frame *frame
-    = muse_processing_new_frame(aProcessing, -1, aHeader,
-                                "SKY_LINES", CPL_FRAME_TYPE_TABLE);
-  if (frame != NULL) {
-    const char *filename = cpl_frame_get_filename(frame);
-    int r;
-    r = cpl_propertylist_save(aHeader, filename, CPL_IO_CREATE);
-    r = muse_cpltable_append_file(aMaster->lines, filename,
-                                  "LINES", muse_sky_lines_lines_def);
-    if (r == CPL_ERROR_NONE) {
-      cpl_frameset_insert(aProcessing->outframes, frame);
-    } else {
-      cpl_frame_delete(frame);
-    }
-  }
-  frame = muse_processing_new_frame(aProcessing, -1, aHeader,
-                                    "SKY_CONTINUUM", CPL_FRAME_TYPE_TABLE);
-  if (frame != NULL) {
-    const char *filename = cpl_frame_get_filename(frame);
-    int r;
-    r = cpl_propertylist_save(aHeader, filename, CPL_IO_CREATE);
-    r = muse_cpltable_append_file(aMaster->continuum, filename,
-                                  "CONTINUUM", muse_fluxspectrum_def);
-    if (r == CPL_ERROR_NONE) {
-      cpl_frameset_insert(aProcessing->outframes, frame);
-    } else {
-      cpl_frame_delete(frame);
-    }
-  }
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    Create a header with the QC parameters for the sky.
-  @param    aMaster Computed sky parameters
-  @return   Header with the QC parameters for the sky.
- */
-/*----------------------------------------------------------------------------*/
-static cpl_error_code
-muse_create_sky_qc_sky(cpl_propertylist *aHeader,
-                       const muse_sky_master *aMaster)
-{
-  cpl_ensure_code(aHeader && aMaster, CPL_ERROR_NULL_INPUT);
-
-  cpl_ensure_code(aMaster->lines && cpl_table_get_nrow(aMaster->lines) > 0,
-                  CPL_ERROR_DATA_NOT_FOUND);
-  char keyword[KEYWORD_LENGTH];
-  int i, ngroups = cpl_table_get_column_max(aMaster->lines, "group") + 1;
-  for (i = 0; i < ngroups; i++) {
-    cpl_table_unselect_all(aMaster->lines);
-    cpl_table_or_selected_int(aMaster->lines, "group", CPL_EQUAL_TO, i);
-    cpl_table *gtable = cpl_table_extract_selected(aMaster->lines);
-    cpl_size row;
-    cpl_table_get_column_maxpos(gtable, "flux", &row);
-    const char *name = cpl_table_get_string(gtable, "name", row);
-    double wavelength = cpl_table_get_double(gtable, "lambda", row, NULL),
-           flux = cpl_table_get_double(gtable, "flux", row, NULL);
-    snprintf(keyword, KEYWORD_LENGTH, "ESO QC SKY LINE%i NAME", i+1);
-    cpl_propertylist_append_string(aHeader, keyword, name);
-    snprintf(keyword, KEYWORD_LENGTH, "ESO QC SKY LINE%i AWAV", i+1);
-    cpl_propertylist_append_double(aHeader, keyword, wavelength);
-    snprintf(keyword, KEYWORD_LENGTH, "ESO QC SKY LINE%i FLUX", i+1);
-    if (!isfinite(flux)) {
-      /* add obviously wrong value to the header,  *
-       * since CFITSIO chokes on NANs and the like */
-      cpl_propertylist_append_double(aHeader, keyword, -9999.999);
-      cpl_msg_error(__func__, "Sky-line fit failed for group %d, computed "
-                    "flux is infinite!", i+1);
-    } else {
-      cpl_propertylist_append_double(aHeader, keyword, flux);
-    }
-    cpl_table_delete(gtable);
-    double offset = aMaster->lsf[0]->offset
-                  + (aMaster->lsf[0]->refraction - 1) * wavelength;
-    snprintf(keyword, KEYWORD_LENGTH, "ESO QC SKY LINE%i OFFSET", i+1);
-    cpl_propertylist_append_double(aHeader, keyword, offset);
-  } /* for ngroups */
-  cpl_table_unselect_all(aMaster->lines);
-
-  cpl_ensure_code(aMaster->continuum && cpl_table_get_nrow(aMaster->continuum) > 0,
-                  CPL_ERROR_DATA_NOT_FOUND);
-  cpl_size row, nrows = cpl_table_get_nrow(aMaster->continuum);
-  double flux = 0.0;
-  for (row = 0; row < nrows; row++) {
-    flux += cpl_table_get_double(aMaster->continuum, "flux", row, NULL);
-  }
-  snprintf(keyword, KEYWORD_LENGTH, "ESO QC SKY CONT FLUX");
-  if (!isfinite(flux)) {
-    /* again, workaround for weird CFITSIO error */
-    cpl_propertylist_append_double(aHeader, keyword, -9999.999);
-    cpl_msg_error(__func__, "Sky-continuum contains infinite values, fit may "
-                  "have failed!");
-  } else {
-    cpl_propertylist_append_double(aHeader, keyword, flux);
-  }
-  double maxdev = 0.0,
-         prev = cpl_table_get_double(aMaster->continuum, "flux", 0, NULL),
-         l_prev = cpl_table_get_double(aMaster->continuum, "lambda", 0, NULL);
-  for (row = 1; row < nrows; row++) {
-    double cur = cpl_table_get_double(aMaster->continuum, "flux", row, NULL),
-           l_cur = cpl_table_get_double(aMaster->continuum, "lambda", row, NULL),
-           dev = fabs((cur - prev)/ (l_cur - l_prev));
-    if (maxdev < dev) {
-      maxdev = dev;
-    }
-    prev = cur;
-    l_prev = l_cur;
-  } /* for row (continuum table rows) */
-  snprintf(keyword, KEYWORD_LENGTH, "ESO QC SKY CONT MAXDEV");
-  cpl_propertylist_append_double(aHeader, keyword, maxdev);
-
-  return CPL_ERROR_NONE;
-} /* muse_create_sky_qc_sky() */
-
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Create a white-light image of the field of view.
   @param    aPixtable   Pixel table with the exposure.
-  @param    aCr         Detect cosmic rays, flag them in the pixel table, and
-                        ignore them in the whitelight image.
+  @param    aCRSigma    Sigma level for cocmic ray rejection.
   @return   A muse_image * on success or NULL on error.
+
+  If aCRSigma is a positive number, detect cosmic rays, flag them in the pixel
+  table, and ignore them in the whitelight image.
  */
 /*----------------------------------------------------------------------------*/
 static muse_image *
-muse_create_sky_whitelight_image(muse_pixtable *aPixtable, int aCr)
+muse_create_sky_whitelight_image(muse_pixtable *aPixtable, double aCRSigma)
 {
   cpl_boolean usegrid = getenv("MUSE_COLLAPSE_PIXTABLE")
                       && atoi(getenv("MUSE_COLLAPSE_PIXTABLE")) > 0;
   muse_resampling_type type = usegrid ? MUSE_RESAMPLE_NONE
                                       : MUSE_RESAMPLE_WEIGHTED_DRIZZLE;
   muse_resampling_params *params = muse_resampling_params_new(type);
-  if (aCr) {
+  if (aCRSigma > 0.) {
     params->crtype = MUSE_RESAMPLING_CRSTATS_MEDIAN;
-    params->crsigma = 15.;
+    params->crsigma = aCRSigma;
   }
   muse_pixgrid *grid = NULL;
   muse_datacube *cube = muse_resampling_cube(aPixtable, params,
@@ -324,33 +181,6 @@ muse_create_sky_whitelight_image(muse_pixtable *aPixtable, int aCr)
   return image;
 } /* muse_create_sky_whitelight_image() */
 
-/*----------------------------------------------------------------------------*/
-/**
-   @private
-   @brief Create a continuum spectrum.
-   @param aPixtable Pixel table to take lambda values from.
-   @param aMaster Master sky parameters.
-   @param aLsf Array with slice specific LSF parameters.
-
-   Replace the continuum spectrum in aMaster with a continuum that is
-   calculated from the sky pixel table data (selected rows in the pixel table)
-   and the LSF parameters.
- */
-/*----------------------------------------------------------------------------*/
-static void
-muse_create_sky_create_continuum(muse_pixtable *aPixtable,
-                                 muse_sky_master *aMaster,
-                                 muse_lsf_params **aLsf,
-                                 double binwidth) {
-  cpl_table_delete(aMaster->continuum);
-  aMaster->continuum = NULL;
-  muse_sky_subtract_pixtable(aPixtable, aMaster, aLsf);
-  aMaster->continuum = muse_resampling_spectrum(aPixtable, binwidth);
-  cpl_table_erase_column(aMaster->continuum, "stat");
-  cpl_table_erase_column(aMaster->continuum, "dq");
-  cpl_table_name_column(aMaster->continuum, "data", "flux");
-}
-
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Interpret the command line options and execute the data processing
@@ -363,6 +193,21 @@ int
 muse_create_sky_compute(muse_processing *aProcessing,
                         muse_create_sky_params_t *aParams)
 {
+  cpl_array *crarray = muse_cplarray_new_from_delimited_string(aParams->crsigma, ",");
+  double crsigmac = 15., /* for cube-based CR rejection */
+         crsigmas = 0.; /* spectrum-based CR rejection */
+  if (cpl_array_get_size(crarray) < 2) {
+    cpl_msg_warning(__func__, "Less than two values given by crsigma "
+                    "parameter, using defaults (%.3f.,%.3f)!", crsigmac,
+                    crsigmas);
+  } else {
+    crsigmac = cpl_array_get_string(crarray, 0)
+             ? atof(cpl_array_get_string(crarray, 0)) : 15.;
+    crsigmas = cpl_array_get_string(crarray, 1)
+             ? atof(cpl_array_get_string(crarray, 1)) : 0.;
+  }
+  cpl_array_delete(crarray);
+
   muse_pixtable *pixtable = muse_create_sky_load_pixtable(aProcessing, aParams);
   if (pixtable == NULL) {
     cpl_msg_error(__func__, "Could not load pixel table");
@@ -383,30 +228,33 @@ muse_create_sky_compute(muse_processing *aProcessing,
     muse_mask_delete(smask);
   }
 
-  int cube_cr = (!strncmp(aParams->cr_s, "cube", 5));
-  muse_image *whitelight = muse_create_sky_whitelight_image(pixtable, cube_cr);
+  muse_image *whitelight = muse_create_sky_whitelight_image(pixtable, crsigmac);
   if (whitelight == NULL) {
     cpl_msg_error(__func__, "Could not create whitelight image");
     return -1;
   }
   muse_processing_save_image(aProcessing, -1, whitelight, MUSE_TAG_IMAGE_FOV);
 
-  muse_mask *sky_mask = muse_sky_create_skymask(whitelight, aParams->fraction,
+  muse_mask *sky_mask = muse_sky_create_skymask(whitelight, aParams->ignore,
+                                                aParams->fraction,
                                                 "ESO QC SKY");
   muse_processing_save_mask(aProcessing, -1, sky_mask, MUSE_TAG_SKY_MASK);
 
   cpl_table_select_all(pixtable->table);
   muse_pixtable_and_selected_mask(pixtable, sky_mask);
+  cpl_table_not_selected(pixtable->table);
+  cpl_table_erase_selected(pixtable->table);
   muse_image_delete(whitelight);
   muse_mask_delete(sky_mask);
 
   cpl_table *spectrum = muse_resampling_spectrum(pixtable, aParams->sampling);
-
-  if (!strncmp(aParams->cr_s, "spectrum", 9)) {
-    muse_sky_mark_cosmic(spectrum, pixtable);
+  if (crsigmas > 0.) {
+    cpl_msg_info(__func__, "Recreating sky spectrum with %.3f sigma level CR "
+                 "rejection", crsigmas);
+    muse_sky_mark_cosmic(spectrum, pixtable, crsigmas);
     cpl_table_delete(spectrum);
     spectrum = muse_resampling_spectrum(pixtable, aParams->sampling);
-  }
+  } /* if crsigmas positive */
 
   if (spectrum == NULL) {
     muse_pixtable_delete(pixtable);
@@ -415,42 +263,98 @@ muse_create_sky_compute(muse_processing *aProcessing,
   muse_processing_save_table(aProcessing, -1, spectrum, pixtable->header,
                              MUSE_TAG_SKY_SPECTRUM, MUSE_TABLE_TYPE_CPL);
 
+
+  cpl_table *lines = muse_sky_lines_load(aProcessing);
+  if (lines != NULL) {
+    double lambda_low = cpl_table_get_column_min(spectrum, "lambda");
+    double lambda_high = cpl_table_get_column_max(spectrum, "lambda");
+    muse_sky_lines_set_range(lines, lambda_low-5, lambda_high+5);
+  }
+
+  cpl_table *continuum = muse_sky_continuum_load(aProcessing);
+
   /*
     Second step: create master sky, containing line list and
     continuum.  If a continuum was given as calibration frame, we
     ignore the computed one and replace it by the external one.
    */
-  cpl_msg_info(__func__, "Creating master sky spectrum using fits to lines "
-               "(fluxes) and residual continuum");
-  muse_sky_master *master
-    = muse_create_sky_compute_master(spectrum, aProcessing);
-  if (master == NULL) {
-    muse_pixtable_delete(pixtable);
-    return -1;
-  }
 
   cpl_errorstate prestate = cpl_errorstate_get();
+  /*
+    Create the average LSF image for the spectrum of the pixels
+    selected as sky. This is done in the following steps:
 
-  cpl_table *continuum = muse_sky_continuum_load(aProcessing);
-  muse_lsf_params **lsfParam =
-    muse_processing_lsf_params_load(aProcessing, 0);
-  if (continuum != NULL) {
-    cpl_table_delete(master->continuum);
-    master->continuum = continuum;
-  } else if (lsfParam != NULL) {
-    muse_create_sky_create_continuum(pixtable, master,
-                                     lsfParam, aParams->csampling);
+    1. Average all cubes with the pixel count as weights into a single
+       image.
+
+    2. Fold an additional rectangle to model the sky spectrum binning
+   */
+  muse_lsf_cube **lsfCube = muse_lsf_cube_load_all(aProcessing);
+  if (lsfCube != NULL) {
+    cpl_image *lsfImage = muse_lsf_average_cube_all(lsfCube, pixtable);
+    muse_wcs *lsfWCS = muse_lsf_cube_get_wcs_all(lsfCube);
+    muse_lsf_fold_rectangle(lsfImage, lsfWCS, aParams->sampling);
+
+    cpl_msg_info(__func__, "Creating master sky spectrum using fits to lines "
+                 "(fluxes) and residual continuum");
+
+    // do the fit, ignoring possible errors
+    muse_sky_lines_fit(spectrum, lines, lsfImage, lsfWCS);
+    if (!cpl_errorstate_is_equal(prestate)) {
+      cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
+      cpl_errorstate_set(prestate);
+    }
+
+    if (continuum == NULL) {
+      continuum = muse_sky_continuum_create(spectrum, lines, lsfImage, lsfWCS,
+                                            aParams->csampling);
+    }
+
+    muse_lsf_cube_delete_all(lsfCube);
+    cpl_image_delete(lsfImage);
+#ifdef USE_LSF_PARAMS
+  } else {
+    cpl_errorstate_set(prestate);
+    muse_lsf_params **lsfParams = muse_processing_lsf_params_load(aProcessing, 0);
+    if (lsfParams != NULL) {  // Old LSF params code
+
+      // do the fit, ignoring possible errors
+      muse_sky_lines_fit_old(spectrum, lines);
+      if (!cpl_errorstate_is_equal(prestate)) {
+        cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
+        cpl_errorstate_set(prestate);
+      }
+
+      if (continuum == NULL) {
+        muse_sky_subtract_lines_old(pixtable, lines, lsfParams);
+        continuum = muse_resampling_spectrum(pixtable, aParams->csampling);
+        cpl_table_erase_column(continuum, "stat");
+        cpl_table_erase_column(continuum, "dq");
+        cpl_table_name_column(continuum, "data", "flux");
+      }
+
+      muse_lsf_params_delete_all(lsfParams);
+    } else {
+      cpl_msg_error(__func__, "Could not load LSF. Continuum is not created.");
+    }
+#endif
   }
-  muse_lsf_params_delete(lsfParam);
+  /* add QC parameters for both emission lines and continuum and save them */
+  cpl_propertylist *header = cpl_propertylist_new();
+  muse_sky_qc_lines(header, lines, "ESO QC SKY");
+  muse_sky_lines_save(aProcessing, lines, header);
+  cpl_propertylist_delete(header);
 
-  cpl_propertylist *qc_header = cpl_propertylist_new();
-  muse_create_sky_qc_sky(qc_header, master);
-
-  muse_create_sky_save_master(qc_header, master, aProcessing);
+  header = cpl_propertylist_new();
+  muse_sky_qc_continuum(header, continuum, "ESO QC SKY");
+  muse_sky_save_continuum(aProcessing, continuum, header);
+  cpl_propertylist_delete(header);
 
   /* Clean up the local objects. */
-  muse_sky_master_delete(master);
-  cpl_propertylist_delete(qc_header);
+  cpl_table_delete(spectrum);
+  cpl_table_delete(lines);
+  cpl_table_delete(continuum);
   muse_pixtable_delete(pixtable);
   return cpl_errorstate_is_equal(prestate) ? 0 : -1;
 }
+
