@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -186,6 +186,7 @@ muse_geometry_load_images(muse_processing *aProcessing, unsigned short aIFU)
                                                  MUSE_TAG_MASTER_BIAS, aIFU);
     /* Merged or not, the REC1 headers we need here are in the primary HDU: */
     cpl_propertylist *hbias = cpl_propertylist_load(cpl_frame_get_filename(fbias), 0);
+    cpl_frame_delete(fbias);
     muse_basicproc_params *bpars = muse_basicproc_params_new_from_propertylist(hbias);
     cpl_propertylist_delete(hbias);
     images = muse_basicproc_load(aProcessing, aIFU, bpars);
@@ -242,7 +243,7 @@ muse_geometry_load_images(muse_processing *aProcessing, unsigned short aIFU)
 
   @error{set and return CPL_ERROR_NULL_INPUT,
          aProcessing\, aParams\, aGeo\, or aGeo->table are NULL}
-  @error{output warning\, reset range to default,
+  @error{set and return CPL_ERROR_ILLEGAL_INPUT,
          aLMin is larger or equal to aLMax}
   @error{set CPL_ERROR_ILLEGAL_OUTPUT and appropriate message but continue,
          after cutting in wavelength, no pixels are left}
@@ -258,10 +259,11 @@ muse_geometry_reconstruct_combined(muse_processing *aProcessing,
                   CPL_ERROR_NULL_INPUT);
   if (aLMin >= aLMax) {
     cpl_msg_warning(__func__, "Invalid wavelength range for reconstruction "
-                    "(%.2f..%.2f), using 6800..7200 instead!", aLMin, aLMax);
-    aLMin = 6800.;
-    aLMax = 7200.;
+                    "(%.2f..%.2f), skipping combined reconstruction!", aLMin, aLMax);
+    return CPL_ERROR_ILLEGAL_INPUT;
   }
+  cpl_msg_info(__func__, "Reconstructing combined input as cube in the "
+               "wavelength range %.2f..%.2f.", aLMin, aLMax);
   unsigned int skip = 0;
   if (getenv("MUSE_GEOMETRY_SKIP") && atoi(getenv("MUSE_GEOMETRY_SKIP")) > 0) {
     skip = atoi(getenv("MUSE_GEOMETRY_SKIP"));
@@ -287,6 +289,7 @@ muse_geometry_reconstruct_combined(muse_processing *aProcessing,
       cframe = cpl_frame_new();
       char *fn = cpl_sprintf("MASK_COMBINED-%02hhu.fits", nifu);
       cpl_frame_set_filename(cframe, fn);
+      cpl_free(fn);
     } else {
       cframe = muse_frameset_find_master(aProcessing->outframes,
                                          MUSE_TAG_MASK_COMBINED, nifu);
@@ -365,7 +368,7 @@ muse_geometry_reconstruct_combined(muse_processing *aProcessing,
 
   @error{set and return CPL_ERROR_NULL_INPUT,
          aProcessing\, aParams\, aGeo\, or aGeo->table are NULL}
-  @error{output warning\, reset range to default,
+  @error{set and return CPL_ERROR_ILLEGAL_INPUT,
          aLMin is larger or equal to aLMax}
   @error{set CPL_ERROR_ILLEGAL_OUTPUT and appropriate message but continue,
          after cutting in wavelength, no pixels are left}
@@ -380,9 +383,8 @@ muse_geometry_reconstruct(muse_processing *aProcessing,
                   CPL_ERROR_NULL_INPUT);
   if (aLMin >= aLMax) {
     cpl_msg_warning(__func__, "Invalid wavelength range for reconstruction "
-                    "(%.2f..%.2f), using 6800..7200 instead!", aLMin, aLMax);
-    aLMin = 6800.;
-    aLMax = 7200.;
+                    "(%.2f..%.2f), skipping reconstruction!", aLMin, aLMax);
+    return CPL_ERROR_ILLEGAL_INPUT;
   }
 
   /* convert all existing raw frames in the used frames to             *
@@ -452,12 +454,12 @@ muse_geometry_reconstruct(muse_processing *aProcessing,
         double frms = 1.01,
                fchisq = 1.04;
         char *rest = strchr(bpars->overscan, ':');
-        if (strlen(bpars->overscan) > 6 && rest) { /* try to access info after "vpoly:" */
-          ovscvorder = strtol(++rest, &rest, 10);
-          if (strlen(rest) > 0) {
-            frms = strtod(++rest, &rest); /* ++ to skip over the comma */
-            if (strlen(rest) > 0) {
-              fchisq = strtod(++rest, &rest);
+        if (strlen(bpars->overscan) > 6 && rest++) { /* try to access info after "vpoly:" */
+          ovscvorder = strtol(rest, &rest, 10);
+          if (strlen(rest++) > 0) { /* ++ to skip over the comma */
+            frms = strtod(rest, &rest);
+            if (strlen(rest++) > 0) {
+              fchisq = strtod(rest, &rest);
             }
           }
         } /* if */
@@ -510,9 +512,10 @@ muse_geometry_reconstruct(muse_processing *aProcessing,
     muse_datacube *cube = muse_resampling_cube(pt, rp, NULL);
     muse_resampling_params_delete(rp);
     muse_pixtable_delete(pt);
-    /* don't want all the cube stuff in the header, better derive it from the *
-     * original primary header of the raw MASK_CHECK exposure; use a trick    *
-     * so that the MASK_CHECK files appears first in the used frames and appear in the headers       */
+    /* I don't want all the cube stuff in the header, better derive it    *
+     * from the original primary header of the raw MASK_CHECK exposure.   *
+     * Use a trick so that the MASK_CHECK files appears first in the used *
+     * frames and appear in the headers.                                  */
     muse_processing_append_used(aProcessing, frame, CPL_FRAME_GROUP_RAW, 1);
     cpl_frameset_insert(usedoriginal, cpl_frame_duplicate(frame)); /* also add here */
     cpl_propertylist *header = cpl_propertylist_load(fn, 0);
@@ -584,7 +587,7 @@ muse_geometry_compute(muse_processing *aProcessing,
     trace[nifu - 1] = muse_table_load(aProcessing, MUSE_TAG_TRACE_TABLE, nifu);
     wavecal[nifu - 1] = muse_table_load(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
     if (!trace[nifu - 1] || !wavecal[nifu - 1]) {
-      cpl_msg_error(__func__, "Calibration could not be loaded for IFU %hu: %s%s",
+      cpl_msg_error(__func__, "Calibration could not be loaded for IFU %u: %s%s",
                     nifu, !trace[nifu - 1] ? " "MUSE_TAG_TRACE_TABLE : "",
                     !wavecal[nifu - 1] ? " "MUSE_TAG_WAVECAL_TABLE : "");
       cpl_table_delete(trace[nifu - 1]);
@@ -600,7 +603,7 @@ muse_geometry_compute(muse_processing *aProcessing,
     muse_imagelist *images = muse_geometry_load_images(aProcessing, nifu);
     if (!images) {
       cpl_msg_error(__func__, "Loading and basic processing of the raw input "
-                    "images failed for IFU %hu!", nifu);
+                    "images failed for IFU %u!", nifu);
       cpl_table_delete(trace[nifu - 1]);
       cpl_table_delete(wavecal[nifu - 1]);
       trace[nifu - 1] = NULL;
@@ -615,7 +618,7 @@ muse_geometry_compute(muse_processing *aProcessing,
     }
     cpl_propertylist *header;
     if (skip >= 2) { /* directly skip to existing spots table */
-      char *fn = cpl_sprintf("SPOTS_TABLE-%02hu.fits", nifu);
+      char *fn = cpl_sprintf("SPOTS_TABLE-%02u.fits", nifu);
       cpl_msg_warning(__func__, "Reading spot measurements from \"%s\"", fn);
       mspots[nifu - 1] = cpl_table_load(fn, 1, 1);
       cpl_free(fn);
@@ -670,8 +673,8 @@ muse_geometry_compute(muse_processing *aProcessing,
   } /* for nifu (all IFUs) */
   /* now use the full array for the global statistics to get a final pinhole distance */
   double pinholedy = muse_geo_compute_pinhole_global_distance(dy, 0.001, 0.5, 0.8);
-  /* XXX return the final value, but don't do anything with it, since for  *
-   *     the moment it is already set in the environment (in the function) */
+  /* return the final value, but don't do anything with it, since for  *
+   * the moment it is already set in the environment (in the function) */
   UNUSED_ARGUMENT(pinholedy);
   cpl_array_delete(dy);
 
@@ -696,9 +699,12 @@ muse_geometry_compute(muse_processing *aProcessing,
     if (!geotable) {
       geotable = geohori;
     } else {
-      if (fabs(geotable->scale - geohori->scale) > DBL_EPSILON) {
+      if (fabs(geotable->scale - geohori->scale) > 1e-10) {
+        /* for some reason, the difference it typically 2.5e-13, but we *
+         * just want to ensure that both are about 1.7050 +/- 0.0001 or *
+         * so and for that an accuracy of 1e-10 is more than enough     */
         cpl_msg_warning(__func__, "Combined and single geometry tables have "
-                        "different scales (%f / %f)!", geotable->scale,
+                        "different scales (%.15f / %.15f)!", geotable->scale,
                         geohori->scale);
       }
       cpl_table_insert(geotable->table, geohori->table,
@@ -738,6 +744,25 @@ muse_geometry_compute(muse_processing *aProcessing,
     result = rcfinal;
   }
   muse_geometry_qc_global(geofinal, headfinal);
+  /* smooth the result per slicer stack, if sigma > 0. */
+  if (aParams->smooth > 0.) {
+    /* save this state already */
+    if (geofinal) {
+      muse_processing_save_table(aProcessing, -1, geofinal->table, headfinal,
+                                 MUSE_TAG_GEOMETRY_OLD, MUSE_TABLE_TYPE_CPL);
+    }
+    rcfinal = muse_geo_correct_slices(geofinal, headfinal, aParams->smooth);
+    if (result < rcfinal) {
+      result = rcfinal;
+    }
+  } else {
+    /* add fake entries to signify that this wasn't smoothed */
+    cpl_propertylist_update_int(headfinal, QC_GEO_SMOOTH_NX, -1);
+    cpl_propertylist_update_int(headfinal, QC_GEO_SMOOTH_NY, -1);
+    cpl_propertylist_update_int(headfinal, QC_GEO_SMOOTH_NA, -1);
+    cpl_propertylist_update_int(headfinal, QC_GEO_SMOOTH_NW, -1);
+  }
+  /* now this is the really final table that we want to save */
   if (geofinal) {
     muse_processing_save_table(aProcessing, -1, geofinal->table, headfinal,
                                MUSE_TAG_GEOMETRY_TABLE, MUSE_TABLE_TYPE_CPL);
