@@ -41,7 +41,7 @@
   @param    aHeader   a FITS header to add the info to
   @param    aWC       the table with the real wavelength solution
   @param    aTT       the table with the tracing solution
- 
+
   @error{set CPL_ERROR_NULL_INPUT and message and return,
          one of aHeader\, aWC\, or aTT is NULL}
  */
@@ -82,12 +82,12 @@ muse_qi_mask_wavecal_polys(cpl_propertylist *aHeader, cpl_table *aWC,
     cpl_vector_set(ycenters, nslice - 1, ycen);
     cpl_polynomial_delete(pconst);
     cpl_polynomial_delete(pw);
-  
+
     /* shift the polynomial to a common center */
     pows = 0;
     double c0 = cpl_polynomial_get_coeff(pwcen, &pows);
     cpl_polynomial_set_coeff(pwcen, &pows, c0 - ycen);
-  
+
     unsigned int iy;
     for (iy = 1;
          iy <= (unsigned)kMuseOutputYTop;
@@ -96,11 +96,11 @@ muse_qi_mask_wavecal_polys(cpl_propertylist *aHeader, cpl_table *aWC,
       cpl_vector_set(val, ipos++, cpl_polynomial_eval_1d(pwcen, iy, NULL));
     }
   } /* for nslice */
-  
+
   /* now try to fit the common 2nd-order polynomial */
   cpl_polynomial *fit = cpl_polynomial_new(1);
   cpl_size maxdeg = 2;
-  cpl_polynomial_fit(fit, pos, CPL_FALSE, val, NULL, CPL_TRUE, NULL, &maxdeg);
+  cpl_polynomial_fit(fit, pos, NULL, val, NULL, CPL_TRUE, NULL, &maxdeg);
 #if 0 /* DEBUG */
   cpl_vector *res = cpl_vector_duplicate(val);
   double chisq;
