@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2014 European Southern Observatory
+ * Copyright (C) 2014-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -158,11 +158,22 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
       cpl_table_delete(wavecal);
       continue;
     }
+    /* search for ILLUM exposure(s) in input list, *
+     * prepare them and remove all from the list   */
+    cpl_table *tattached = muse_basicproc_get_illum(images, trace, wavecal, geo);
     /* combine the raw exposures into a master image */
     cpl_msg_debug(__func__, "combine raw files of IFU %2hhu", nifu);
     muse_image *masterimage = muse_combine_images(aCPars, images);
     muse_twilight_qc_header(masterimage, images);
     muse_imagelist_delete(images);
+    if (!masterimage) {
+      cpl_msg_warning(__func__, "Combining individual images failed for IFU %2hhu: %s",
+                      nifu, cpl_error_get_message());
+      cpl_table_delete(tattached);
+      cpl_table_delete(trace);
+      cpl_table_delete(wavecal);
+      continue;
+    }
 
     /* apply bad-pixel interpolation on this image, so that the *
      * per-IFU integrated fluxes are independent of bad pixels  */
@@ -186,8 +197,12 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
       cpl_propertylist_copy_property_regexp(pts[nifu - 1]->header,
                                             masterimage->header,
                                             QC_TWILIGHT_REGEXP, 0);
+      if (tattached) { /* test before, to not throw an error */
+        muse_basicproc_apply_illum(pts[nifu - 1], tattached);
+      } /* if attached flat given */
     } /* if pixel table created */
     muse_image_delete(masterimage);
+    cpl_table_delete(tattached);
   } /* for nifu (all IFUs) */
   cpl_table_delete(geo);
 
@@ -204,6 +219,28 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
     cpl_free(pts);
     return NULL;
   } /* if */
+
+  cpl_array *scales = NULL;
+  if (getenv("MUSE_TWILIGHT_SCALES")) {
+    scales = muse_cplarray_new_from_delimited_string(getenv("MUSE_TWILIGHT_SCALES"), ",");
+    double scale = atof(cpl_array_get_string(scales, 0));
+    cpl_table_divide_scalar(pt->table, MUSE_PIXTABLE_DATA, scale);
+    cpl_table_divide_scalar(pt->table, MUSE_PIXTABLE_STAT, scale*scale);
+    int nscales = cpl_array_get_size(scales);
+    if (nscales != kMuseNumIFUs) {
+      cpl_msg_warning(__func__, "%d external scales found instead of %d! Not "
+                      "using them.", nscales, kMuseNumIFUs);
+      cpl_array_delete(scales);
+      scales = NULL;
+    } else {
+      cpl_msg_info(__func__, "Using %d external scales.", nscales);
+    }
+#if 1
+    cpl_array_dump(scales, 0, 30, stdout);
+    fflush(stdout);
+#endif
+  } /* if */
+
   double fluxref = cpl_table_get_column_mean(pt->table, MUSE_PIXTABLE_DATA)
                  * muse_pixtable_get_nrow(pt);
   cpl_msg_debug(__func__, "pixel table of IFU  1, fluxref = %e, %"CPL_SIZE_FORMAT
@@ -212,7 +249,7 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
    * but first restricting the wavelength range and with flux        *
    * correction using the data we have in the table                  */
   cpl_propertylist_append_double(pt->header, MUSE_HDR_FLAT_FLUX_SKY"1", fluxref);
-  char *kw = cpl_sprintf(QC_TWILIGHTm_INTFLUX, 1);
+  char *kw = cpl_sprintf(QC_TWILIGHTm_INTFLUX, (unsigned char)1);
   cpl_propertylist_append_float(pt->header, kw, fluxref);
   cpl_free(kw);
   for (nifu = 2; nifu <= kMuseNumIFUs; nifu++) {
@@ -225,7 +262,14 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
     } /* if */
     double flux = cpl_table_get_column_mean(pts[nifu - 1]->table, MUSE_PIXTABLE_DATA)
                 * muse_pixtable_get_nrow(pts[nifu - 1]),
-           scale = flux / fluxref;
+           scale = 1.;
+    if (scales) {
+      scale = atof(cpl_array_get_string(scales, nifu - 1));
+      /* also recomputed the flux that is then saved in the header */
+      flux = scale * fluxref;
+    } else {
+      scale = flux / fluxref;
+    } /* else */
     kw = cpl_sprintf(MUSE_HDR_FLAT_FLUX_SKY"%hhu", nifu);
     cpl_propertylist_append_double(pt->header, kw, flux);
     cpl_free(kw);
@@ -246,6 +290,7 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
   cpl_free(pts);
   /* reset limits in the header, necessary due to manual operation on table data */
   muse_pixtable_compute_limits(pt);
+  cpl_array_delete(scales);
 
   /* now just reconstruct the cube and collapse over the full range */
   muse_datacube *cube = muse_resampling_cube(pt, aRPars, NULL);
@@ -488,9 +533,9 @@ muse_twilight_compute(muse_processing *aProcessing,
   }
 #if 0 // only for smoothing in wavelength direction (see below)
   /* also compute the wavelength of each plane */
-  double crpix3 = cpl_propertylist_get_double(cube->header, "CRPIX3"),
-         crval3 = cpl_propertylist_get_double(cube->header, "CRVAL3"),
-         cd33 = cpl_propertylist_get_double(cube->header, "CD3_3");
+  double crpix3 = muse_pfits_get_crpix(cube->header, 3),
+         crval3 = muse_pfits_get_crval(cube->header, 3),
+         cd33 = muse_pfits_get_cd(cube->header, 3, 3);
   cpl_propertylist *pbla = cpl_propertylist_new();
   cpl_propertylist_copy_property_regexp(pbla, cube->header, "^C", 0);
   cpl_propertylist_dump(pbla, stdout);
@@ -562,6 +607,7 @@ muse_twilight_compute(muse_processing *aProcessing,
   image->header = cpl_propertylist_new();
   cpl_propertylist_append_string(image->header, "OBJECT", "normalized "
                                  "white-light divided by white global fit");
+  cpl_propertylist_append_string(image->header, "BUNIT", "1"); /* unitless */
   image->data = wdiv;
   image->dq = cpl_image_new_from_mask(cpl_image_get_bpm(wdiv));
   muse_imagelist_set(cubefit->recimages, image,
@@ -647,6 +693,7 @@ muse_twilight_compute(muse_processing *aProcessing,
       image->header = cpl_propertylist_new();
       cpl_propertylist_append_string(image->header, "OBJECT",
                                      "white-light divided by white global, mask excl edges");
+      cpl_propertylist_append_string(image->header, "BUNIT", "1"); /* unitless */
       image->data = wdivX;
       image->dq = cpl_image_new_from_mask(wdivXmask);
       cpl_mask_delete(wdivXmask);
@@ -752,6 +799,8 @@ muse_twilight_compute(muse_processing *aProcessing,
     image = muse_image_new();
     image->header = cpl_propertylist_new();
     cpl_propertylist_append_string(image->header, "OBJECT", "vignetting fit");
+    /* no sensible unit for this image */
+    cpl_propertylist_append_string(image->header, "BUNIT", "");
     image->data = fit;
     image->dq = cpl_image_new_from_mask(cpl_image_get_bpm(fit));
     muse_imagelist_set(cubefit->recimages, image,
@@ -766,6 +815,8 @@ muse_twilight_compute(muse_processing *aProcessing,
     image->header = cpl_propertylist_new();
     cpl_propertylist_append_string(image->header, "OBJECT",
                                    "white image corrected by vignetting fit");
+    /* unit again like white image */
+    cpl_propertylist_append_string(image->header, "BUNIT", "count");
     image->data = cpl_image_divide_create(wdiv2, fit);
     image->dq = cpl_image_new_from_mask(cpl_image_get_bpm(image->data));
     muse_imagelist_set(cubefit->recimages, image,
@@ -781,6 +832,8 @@ muse_twilight_compute(muse_processing *aProcessing,
                  " in %d planes", npl);
     for (ipl = 0; ipl < npl; ipl++) {
       cpl_image *cimage = cpl_imagelist_get(cubefit->data, ipl);
+      // XXX the best here would be to restrict the mask to the illuminated
+      //     area instead of using the full image plane!
       cpl_image_accept_all(cimage);
       double mean1 = cpl_image_get_mean(cimage);
       cpl_image_multiply(cimage, fit);
