@@ -444,18 +444,21 @@ int main(int argc, char **argv)
   cpl_propertylist_append_string(phead, "ESO PRO REC1 ID", "muse_flat"); // Pipeline recipe (unique) identifier
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM1 NAME", "nifu"); // IFU to handle. If set to 0, all
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM1 VALUE", "-1"); // Default: 0
+  cpl_propertylist_set_comment(phead, "ESO PRO REC1 PARAM1 VALUE", "Default: 0");
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM10 NAME", "lsigma"); // Low sigma for pixel rejection
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM10 VALUE", "3"); // Default: 3
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM11 NAME", "hsigma"); // High sigma for pixel rejection
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM11 VALUE", "3"); // Default: 3
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM12 NAME", "scale"); // Scale the individual images to
-  cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM12 VALUE", "true"); // Default: true
+  cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM12 VALUE", "false"); // Default: true
+  cpl_propertylist_set_comment(phead, "ESO PRO REC1 PARAM12 VALUE", "Default: true");
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM13 NAME", "normalize"); // Normalize the master flat to
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM13 VALUE", "true"); // Default: true
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM14 NAME", "trace"); // Trace the position of the slic
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM14 VALUE", "true"); // Default: true
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM15 NAME", "nsum"); // Number of lines over which to
-  cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM15 VALUE", "32"); // Default: 32
+  cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM15 VALUE", "16"); // Default: 32
+  cpl_propertylist_set_comment(phead, "ESO PRO REC1 PARAM15 VALUE", "Default: 32");
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM16 NAME", "order"); // Order of polynomial fit to the
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM16 VALUE", "5"); // Default: 5
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM17 NAME", "edgefrac"); // Fractional change required to
@@ -466,6 +469,7 @@ int main(int argc, char **argv)
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM19 VALUE", "5"); // Default: 5
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM2 NAME", "overscan"); // If this is "none", stop when de
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM2 VALUE", "vpoly"); // Default: "vpoly"
+  cpl_propertylist_set_comment(phead, "ESO PRO REC1 PARAM2 VALUE", "Default: bla");
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM20 NAME", "samples"); // Create a table containing all
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM20 VALUE", "true"); // Default: false
   cpl_propertylist_append_string(phead, "ESO PRO REC1 PARAM3 NAME", "ovscreject"); // This influences how values ar
@@ -508,9 +512,11 @@ int main(int argc, char **argv)
   cpl_parameter *par = cpl_parameterlist_find(parlist, "muse.muse_flat.nifu");
   cpl_test_nonnull(par);
   cpl_test(cpl_parameter_get_int(par) == -1);
+  cpl_test(cpl_parameter_get_default_int(par) == 0); /* was set in the header */
   par = cpl_parameterlist_find(parlist, "muse.muse_flat.order");
   cpl_test_nonnull(par);
   cpl_test(cpl_parameter_get_int(par) == 5);
+  cpl_test(cpl_parameter_get_default_int(par) == 0); /* no default was set in the header */
   par = cpl_parameterlist_find(parlist, "muse.muse_flat.trace");
   cpl_test_nonnull(par);
   cpl_test(cpl_parameter_get_bool(par) == CPL_TRUE);
@@ -530,6 +536,11 @@ int main(int argc, char **argv)
   if (string) {
     cpl_test_zero(strncmp(string, "vpoly", 6));
   }
+  string = (char *)cpl_parameter_get_default_string(par);
+  cpl_test_nonnull(string); /* we set this in the header */
+  if (string) {
+    cpl_test_zero(strncmp(string, "bla", 4));
+  }
   par = cpl_parameterlist_find(parlist, "muse.muse_flat.commasep");
   cpl_test_nonnull(par);
   string = (char *)cpl_parameter_get_string(par);
