@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2014 European Southern Observatory
+ * Copyright (C) 2007-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,8 +55,8 @@ muse_test_flux_check_cubes(muse_datacube **aCubes, unsigned short aNCubes)
    * i.e. [2,2,*] (the central spectrum!), of the cube    */
   int nlambda = cpl_imagelist_get_size(aCubes[0]->data);
   cpl_image *image = cpl_image_new(nlambda, 4, CPL_TYPE_DOUBLE);
-  double cdelt = cpl_propertylist_get_double(aCubes[0]->header, "CD3_3"),
-         crval = cpl_propertylist_get_double(aCubes[0]->header, "CRVAL3");
+  double cdelt = muse_pfits_get_cd(aCubes[0]->header, 3, 3),
+         crval = muse_pfits_get_crval(aCubes[0]->header, 3);
   const int windows[][2] = {
     {    1, nlambda }, /* full spectrum */
     {  650, 1050 }, /* blue line-free region, ~5610...6110 Angstrom */
@@ -157,19 +157,195 @@ muse_test_flux_check_cubes(muse_datacube **aCubes, unsigned short aNCubes)
   }
 } /* muse_test_flux_check_cubes() */
 
+#define PRINT_USAGE(rc)                                                        \
+  fprintf(stderr, "\nUsage: %s [ -profile <gaussian | moffat | circle | square> ]"\
+          "[ -select <flux | distance> ] [ -smooth <none | median | ppoly> ] " \
+          "DATACUBE_STD STD_FLUX_TABLE EXTINCTION_TABLE\n"                     \
+          "  The output STD_RESPONSE is written to a file starting with the "  \
+          "same basename as DATACUBE_STD.\n\n\n", argv[0]);                    \
+  return (rc);
+
+int
+muse_test_flux_integrate_from_cube(int argc, char **argv)
+{
+  cpl_errorstate state = cpl_errorstate_get();
+
+  /* defaults */
+  char *incube = NULL,
+       *inref = NULL,
+       *inext = NULL;
+  muse_flux_profile_type profile = MUSE_FLUX_PROFILE_MOFFAT;
+  muse_flux_selection_type select = MUSE_FLUX_SELECT_BRIGHTEST;
+  muse_flux_smooth_type smooth = MUSE_FLUX_SMOOTH_PPOLY;
+
+  /* argument processing */
+  int i;
+  for (i = 1; i < argc; i++) {
+    if (strncmp(argv[i], "-profile", 9) == 0) {
+      /* skip to next arg to first slice number */
+      i++;
+      if (i < argc) {
+        if (!strncmp(argv[i], "moffat", 7)) {
+          profile = MUSE_FLUX_PROFILE_MOFFAT;
+        } else if (!strncmp(argv[i], "gaussian", 9)) {
+          profile = MUSE_FLUX_PROFILE_GAUSSIAN;
+        } else if (!strncmp(argv[i], "circle", 7)) {
+          profile = MUSE_FLUX_PROFILE_CIRCLE;
+        } else if (!strncmp(argv[i], "square", 7)) {
+          profile = MUSE_FLUX_PROFILE_EQUAL_SQUARE;
+        }
+      } else {
+        PRINT_USAGE(2);
+      }
+    } else if (strncmp(argv[i], "-select", 8) == 0) {
+      /* skip to next arg to last slice number */
+      i++;
+      if (i < argc) {
+        if (strncmp(argv[i], "flux", 5) == 0) {
+          select = MUSE_FLUX_SELECT_BRIGHTEST;
+        } else if (strncmp(argv[i], "distance", 9) == 0) {
+          select = MUSE_FLUX_SELECT_NEAREST;
+        }
+      } else {
+        PRINT_USAGE(3);
+      }
+    } else if (strncmp(argv[i], "-smooth", 8) == 0) {
+      /* skip to next arg to last slice number */
+      i++;
+      if (i < argc) {
+        if (strncmp(argv[i], "none", 5) == 0) {
+          smooth = MUSE_FLUX_SMOOTH_NONE;
+        } else if (strncmp(argv[i], "median", 7) == 0) {
+          smooth = MUSE_FLUX_SMOOTH_MEDIAN;
+        } else if (strncmp(argv[i], "ppoly", 6) == 0) {
+          smooth = MUSE_FLUX_SMOOTH_PPOLY;
+        }
+      } else {
+        PRINT_USAGE(4);
+      }
+    } else if (strncmp(argv[i], "-", 1) == 0) { /* unallowed options */
+      PRINT_USAGE(9);
+    } else {
+      if (incube && inref && inext) {
+        break; /* we have the possible names, skip the rest */
+      }
+      if (!incube) {
+        incube = argv[i]; /* set the name for the input cube */
+      } else if (!inref) {
+        inref = argv[i]; /* set the name for the input reference table */
+      } else {
+        inext = argv[i]; /* set the name for the input extinction table */
+      }
+    }
+  } /* for i (all arguments) */
+
+  muse_datacube *cube = muse_datacube_load(incube);
+  if (!cube) {
+    PRINT_USAGE(10);
+  }
+  /* copy stuff from muse_flux_integrate_std() */
+  int nplane = cpl_imagelist_get_size(cube->data) / 2; /* central plane */
+  cpl_image *cim = cpl_imagelist_get(cube->data, nplane);
+  cpl_image_reject_value(cim, CPL_VALUE_NAN); /* mark NANs */
+  double dsigmas[] = { 50., 30., 10., 5. };
+  cpl_vector *vsigmas = cpl_vector_wrap(sizeof(dsigmas) / sizeof(double),
+                                        dsigmas);
+  cpl_size isigma = -1;
+  cpl_apertures *apertures = cpl_apertures_extract(cim, vsigmas, &isigma);
+  int napertures = apertures ? cpl_apertures_get_size(apertures) : 0;
+  if (napertures < 1) {
+    double med_dist,
+           median = cpl_image_get_median_dev(cim, &med_dist),
+           threshold = median + 5. * med_dist;
+    cpl_msg_debug(__func__, "failed: %f +/- %f --> %f", median, med_dist,
+                  threshold);
+    cpl_vector_unwrap(vsigmas);
+    cpl_apertures_delete(apertures);
+    muse_datacube_delete(cube);
+    PRINT_USAGE(11);
+  }
+  cpl_msg_debug(__func__, "The %.1f sigma threshold was used to find %d source%s",
+                cpl_vector_get(vsigmas, isigma), napertures, napertures == 1 ? "" : "s");
+  cpl_vector_unwrap(vsigmas);
+  /* now do the flux integration and save the resulting file */
+  muse_image *intimage = muse_flux_integrate_cube(cube, apertures, profile);
+  cpl_apertures_delete(apertures);
+  char *dot = strrchr(incube, '.');
+  *dot = '\0'; /* replace the dot */
+  char *fn = cpl_sprintf("%s_STD_FLUXES.fits", incube);
+  muse_image_save(intimage, fn);
+  cpl_msg_info(__func__, "Wrote intimage as \"%s\".", fn);
+  cpl_free(fn);
+
+  /* load the tables */
+  cpl_frame *refframe = cpl_frame_new();
+  cpl_frame_set_filename(refframe, inref);
+  cpl_table *reftable = muse_postproc_load_nearest(cube->header, refframe,
+                                                   20., 60., NULL, NULL),
+            *exttable = cpl_table_load(inext, 1, 1);
+  cpl_frame_delete(refframe);
+  if (!reftable || !exttable) {
+    muse_datacube_delete(cube);
+    muse_image_delete(intimage);
+    PRINT_USAGE(12);
+  }
+  muse_flux_reference_table_check(reftable);
+  double airmass = muse_astro_airmass(cube->header);
+  muse_flux_object *fo = muse_flux_object_new();
+  fo->cube = cube;
+  fo->intimage = intimage;
+  fo->raref = muse_pfits_get_ra(cube->header);
+  fo->decref = muse_pfits_get_dec(cube->header);
+  muse_flux_response_compute(fo, select, airmass, reftable,
+                             NULL, exttable);
+  muse_flux_get_response_table(fo, smooth);
+  fn = cpl_sprintf("%s_STD_RESPONSE.fits", incube);
+  cpl_table_save(fo->response, cube->header, NULL, fn, CPL_IO_CREATE);
+  cpl_msg_info(__func__, "Wrote response as \"%s\".", fn);
+  cpl_free(fn);
+  muse_flux_get_telluric_table(fo);
+  fn = cpl_sprintf("%s_STD_TELLURIC.fits", incube);
+  cpl_table_save(fo->telluric, cube->header, NULL, fn, CPL_IO_CREATE);
+  cpl_msg_info(__func__, "Wrote telluric as \"%s\".", fn);
+  cpl_free(fn);
+  muse_flux_object_delete(fo);
+  cpl_table_delete(reftable);
+  cpl_table_delete(exttable);
+
+  cpl_test(cpl_errorstate_is_equal(state));
+  return CPL_ERROR_NONE;
+} /* muse_test_flux_integrate_from_cube() */
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    test program to check that the functions from the muse_flux module
             do what they should when called with the necessary data
+
+  This program explicitely tests
+    muse_flux_object_new
+    muse_flux_object_delete
+    muse_flux_reference_table_check
+    muse_flux_response_interpolate
+    muse_flux_integrate_cube
+    muse_flux_integrate_std
+    muse_flux_response_compute
+    muse_flux_get_response_table
+    muse_flux_get_telluric_table
+    muse_flux_calibrate
  */
 /*----------------------------------------------------------------------------*/
 int main(int argc, char **argv)
 {
-  UNUSED_ARGUMENTS(argc, argv);
   cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
 
-  cpl_table *ref = cpl_table_load(BASEFILENAME"_reference_gd71sm.fits", 1, 0),
-            *extinction = cpl_table_load(BASEFILENAME"_extinction.fits", 1, 0);
+  if (argc > 1) {
+    /* If there are arguments, misuse this test program as flux integrator. */
+    cpl_test(muse_test_flux_integrate_from_cube(argc, argv) == CPL_ERROR_NONE);
+    return cpl_test_end(0);
+  }
+
+  cpl_table *ref = cpl_table_load(BASEFILENAME"_reference_gd71sm.fits", 1, 1),
+            *extinction = cpl_table_load(BASEFILENAME"_extinction.fits", 1, 1);
 
   /* test muse_flux_reference_table_check() */
   cpl_test(muse_flux_reference_table_check(ref) == CPL_ERROR_NONE);
@@ -177,7 +353,7 @@ int main(int argc, char **argv)
   cpl_test(muse_flux_reference_table_check(NULL) == CPL_ERROR_NULL_INPUT);
   cpl_errorstate_set(state);
   /* test with HST table */
-  cpl_table *refhst = cpl_table_load(BASEFILENAME"_gd153_stisnic_004.fits", 1, 0),
+  cpl_table *refhst = cpl_table_load(BASEFILENAME"_gd153_stisnic_004.fits", 1, 1),
             *refhst2 = cpl_table_duplicate(refhst);
   cpl_test(muse_flux_reference_table_check(refhst) == CPL_ERROR_NONE);
   cpl_test(cpl_table_has_column(refhst, "lambda"));
@@ -266,20 +442,28 @@ int main(int argc, char **argv)
   double ftot = cpl_stats_get_flux(s),
          fmedian = cpl_stats_get_median(s);
   cpl_stats_delete(s);
-  /* before starting to mess with the pixel table, try       *
-   * integrating the flux using the other profile types, too */
-  unsigned int i;
-  for (i = MUSE_FLUX_PROFILE_GAUSSIAN; i <= MUSE_FLUX_PROFILE_EQUAL_SQUARE; i++) {
-    muse_flux_object *fobj2 = muse_flux_object_new();
+  /* extract the source on the same sigma was done above     *
+   * (the cube is 3961 pixels long, so use plane index 1980) */
+  cpl_image *plane = cpl_imagelist_get(fluxobj->cube->data, 1980);
+  cpl_apertures *ap = cpl_apertures_extract_sigma(plane, 50.);
+  /* Before starting to mess with the pixel table, try integrating the    *
+   * flux using the other profile types, too. Since the above tested      *
+   * muse_flux_integrate_std(), we can now use muse_flux_integrate_cube() *
+   * on the existing cube which is much faster.                           */
+  int i;
+  for (i = MUSE_FLUX_PROFILE_EQUAL_SQUARE; i >= MUSE_FLUX_PROFILE_MOFFAT; i--) {
+    if (i == MUSE_FLUX_PROFILE_MOFFAT) {
+      /* in the last iteration, for the moffat profile, test the (success) *
+       * case where the DIMM measurements are missing from the cube header */
+      cpl_propertylist_erase_regexp(fluxobj->cube->header, "ESO TEL AMBI FWHM ", 0);
+    }
     state = cpl_errorstate_get();
-    rc = muse_flux_integrate_std(pt, i, fobj2);
-    cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
-    cpl_test_nonnull(fobj2->cube);
-    cpl_test_nonnull(fobj2->intimage);
-    cpl_test(cpl_imagelist_get_size(fobj2->cube->data)
-             == cpl_image_get_size_x(fobj2->intimage->data));
-    s = cpl_stats_new_from_image_window(fobj2->intimage->data, smode,
-                                        201, 1, 3801, 1);
+    muse_image *intimage = muse_flux_integrate_cube(fluxobj->cube, ap, i);
+    cpl_test(cpl_errorstate_is_equal(state));
+    cpl_test_nonnull(intimage);
+    cpl_test(cpl_imagelist_get_size(fluxobj->cube->data)
+             == cpl_image_get_size_x(intimage->data));
+    s = cpl_stats_new_from_image_window(intimage->data, smode, 201, 1, 3801, 1);
     /* should be at least comparable to Moffat integration */
     cpl_msg_debug(__func__, "flux %g / %g\n\tmedian %g / %g",
                   ftot, cpl_stats_get_flux(s), fmedian, cpl_stats_get_median(s));
@@ -289,7 +473,37 @@ int main(int argc, char **argv)
     cpl_test_rel(ftot, cpl_stats_get_flux(s), limit1);
     cpl_test_rel(fmedian, cpl_stats_get_median(s), limit2);
     cpl_stats_delete(s);
-    muse_flux_object_delete(fobj2);
+    muse_image_delete(intimage);
+
+    /* for one of the calls, test the case of a NAN-converted cube */
+    if (i == MUSE_FLUX_PROFILE_GAUSSIAN) {
+      /* duplicate the cube */
+      muse_datacube *cube2 = cpl_calloc(1, sizeof(muse_datacube));
+      cube2->header = cpl_propertylist_duplicate(fluxobj->cube->header);
+      cube2->data = cpl_imagelist_duplicate(fluxobj->cube->data);
+      cube2->dq = cpl_imagelist_duplicate(fluxobj->cube->dq);
+      cube2->stat = cpl_imagelist_duplicate(fluxobj->cube->stat);
+      muse_datacube_convert_dq(cube2);
+
+      state = cpl_errorstate_get();
+      intimage = muse_flux_integrate_cube(cube2, ap, i);
+      cpl_test(cpl_errorstate_is_equal(state));
+      cpl_test_nonnull(intimage);
+      cpl_test(cpl_imagelist_get_size(fluxobj->cube->data)
+               == cpl_image_get_size_x(intimage->data));
+      s = cpl_stats_new_from_image_window(intimage->data, smode, 201, 1, 3801, 1);
+      /* should be at least comparable to Moffat integration */
+      cpl_msg_debug(__func__, "flux %g / %g\n\tmedian %g / %g",
+                    ftot, cpl_stats_get_flux(s), fmedian, cpl_stats_get_median(s));
+      /* the Gaussian integrates less flux than the other methods... */
+      limit1 = 0.18,
+      limit2 = 0.15;
+      cpl_test_rel(ftot, cpl_stats_get_flux(s), limit1);
+      cpl_test_rel(fmedian, cpl_stats_get_median(s), limit2);
+      cpl_stats_delete(s);
+      muse_image_delete(intimage);
+      muse_datacube_delete(cube2);
+    } /* if MUSE_FLUX_PROFILE_GAUSSIAN */
   } /* for i (all other integration profiles) */
   /* test failure cases of muse_flux_integrate_std(), too */
   muse_flux_object *fobj3 = muse_flux_object_new();
@@ -304,6 +518,36 @@ int main(int argc, char **argv)
   rc = muse_flux_integrate_std(pt, MUSE_FLUX_PROFILE_MOFFAT, NULL);
   cpl_test(rc == CPL_ERROR_NULL_INPUT && !cpl_errorstate_is_equal(state));
   cpl_errorstate_set(state);
+  /* test with pixel table with constant values, to trigger detection failure */
+  muse_pixtable *ptc = muse_pixtable_duplicate(pt);
+  muse_pixtable_restrict_wavelength(ptc, 6950., 7050.);
+  cpl_table_fill_column_window_float(ptc->table, MUSE_PIXTABLE_DATA,
+                                     0, cpl_table_get_nrow(ptc->table), 100);
+  fobj3 = muse_flux_object_new();
+  state = cpl_errorstate_get();
+  rc = muse_flux_integrate_std(ptc, MUSE_FLUX_PROFILE_MOFFAT, fobj3);
+  cpl_test(rc == CPL_ERROR_DATA_NOT_FOUND && !cpl_errorstate_is_equal(state));
+  cpl_errorstate_set(state);
+  muse_flux_object_delete(fobj3);
+  muse_pixtable_delete(ptc);
+  /* and failure cases for muse_flux_integrate_cube() */
+  state = cpl_errorstate_get();
+  muse_image *image = muse_flux_integrate_cube(NULL, ap, MUSE_FLUX_PROFILE_MOFFAT);
+  cpl_test(!cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_test_null(image);
+  image = muse_flux_integrate_cube(fluxobj->cube, NULL, MUSE_FLUX_PROFILE_MOFFAT);
+  cpl_test(!cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_test_null(image);
+  image = muse_flux_integrate_cube(fluxobj->cube, ap, MUSE_FLUX_PROFILE_EQUAL_SQUARE + 1);
+  cpl_test(!cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() ==CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_test_null(image);
+  cpl_apertures_delete(ap);
 
   /* test success of muse_flux_response_compute() for reasonable data */
   state = cpl_errorstate_get();
