@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2014 European Southern Observatory
+ * Copyright (C) 2007-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ int main(int argc, char **argv)
   UNUSED_ARGUMENTS(argc, argv);
   cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
 
-  cpl_table *geo = cpl_table_load(BASEFILENAME"_regular.fits", 1, 0);
+  cpl_table *geo = cpl_table_load(BASEFILENAME"_regular.fits", 1, 1);
   cpl_test_nonnull(geo);
   if (!geo) {
     cpl_msg_error(__func__, "could not load \"%s_regular.fits\"", BASEFILENAME);
@@ -97,7 +97,7 @@ int main(int argc, char **argv)
   cpl_table_delete(geo);
 
   /* muse_geo_lines_get() */
-  cpl_table *tlines = cpl_table_load(BASEFILENAME"_linelist.fits", 1, 0);
+  cpl_table *tlines = cpl_table_load(BASEFILENAME"_linelist.fits", 1, 1);
   state = cpl_errorstate_get();
   cpl_vector *vlines = muse_geo_lines_get(tlines);
   cpl_test_nonnull(vlines);
@@ -126,10 +126,10 @@ int main(int argc, char **argv)
   cpl_table_delete(tlines);
 
   /* muse_geo_measure_spots() */
-  cpl_table *tr22 = cpl_table_load(BASEFILENAME"_trace22.fits", 1, 0),
-            *tr23 = cpl_table_load(BASEFILENAME"_trace23.fits", 1, 0),
-            *wv22 = cpl_table_load(BASEFILENAME"_wave22.fits", 1, 0),
-            *wv23 = cpl_table_load(BASEFILENAME"_wave23.fits", 1, 0);
+  cpl_table *tr22 = cpl_table_load(BASEFILENAME"_trace22.fits", 1, 1),
+            *tr23 = cpl_table_load(BASEFILENAME"_trace23.fits", 1, 1),
+            *wv22 = cpl_table_load(BASEFILENAME"_wave22.fits", 1, 1),
+            *wv23 = cpl_table_load(BASEFILENAME"_wave23.fits", 1, 1);
   if (!tr22 || !tr23 || !wv22 || !wv23) {
     cpl_error_set_message(__func__, cpl_error_get_code(),
                           "failed to load file(s): %s%s%s%s",
@@ -230,8 +230,8 @@ int main(int argc, char **argv)
   muse_imagelist_delete(list4);
   cpl_vector_delete(lines);
 
-  cpl_table *sp22 = cpl_table_load(BASEFILENAME"_spots22.fits", 1, 0),
-            *sp23 = cpl_table_load(BASEFILENAME"_spots23.fits", 1, 0);
+  cpl_table *sp22 = cpl_table_load(BASEFILENAME"_spots22.fits", 1, 1),
+            *sp23 = cpl_table_load(BASEFILENAME"_spots23.fits", 1, 1);
   if (!sp22 || !sp23) {
     cpl_error_set_message(__func__, cpl_error_get_code(),
                           "failed to load file(s): %s%s",
@@ -580,8 +580,8 @@ int main(int argc, char **argv)
 
   /* muse_geo_refine_horizontal() */
   /* load another test dataset to test this function */
-  cpl_table *geocomb = cpl_table_load(BASEFILENAME"_geo_comb01to03.fits", 1, 0),
-            *spotscomb = cpl_table_load(BASEFILENAME"_spots01to03.fits", 1, 0);
+  cpl_table *geocomb = cpl_table_load(BASEFILENAME"_geo_comb01to03.fits", 1, 1),
+            *spotscomb = cpl_table_load(BASEFILENAME"_spots01to03.fits", 1, 1);
   /* add new necessary ScaleFOV columns */
   cpl_table_new_column(spotscomb, "ScaleFOV", CPL_TYPE_DOUBLE);
   cpl_table_fill_column_window(spotscomb, "ScaleFOV", 0, cpl_table_get_nrow(spotscomb), 60./35.);
@@ -870,15 +870,13 @@ int main(int argc, char **argv)
   muse_geo_table_delete(geovkeep2);
   muse_geo_table_delete(geovkeep);
 
-#if 1
+#if 0
   state = cpl_errorstate_get();
   const char *fn = "muse_test_geo_final.fits";
   cpl_msg_info(__func__, "Saving final table as \"%s\"", fn);
   cpl_table_save(geov->table, NULL, NULL, fn, CPL_IO_CREATE);
   cpl_test(cpl_errorstate_is_equal(state));
 #endif
-
-  muse_geo_table_delete(geov);
   cpl_table_delete(tr22);
   cpl_table_delete(tr23);
   cpl_table_delete(wv22);
@@ -886,5 +884,79 @@ int main(int argc, char **argv)
   cpl_table_delete(sp22);
   cpl_table_delete(sp23);
 
+  /* muse_geo_correct_slices() */
+  state = cpl_errorstate_get();
+  geovkeep = muse_geo_table_duplicate(geov);
+  rc = muse_geo_correct_slices(geov, NULL, 1.5);
+  cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
+  muse_geo_table_delete(geov);
+  /* again with header */
+  geov = muse_geo_table_duplicate(geovkeep);
+  cpl_propertylist *header = cpl_propertylist_new();
+  rc = muse_geo_correct_slices(geov, header, 1.5);
+  cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
+  cpl_test(cpl_propertylist_get_size(header) == 4);
+#if 0
+  state = cpl_errorstate_get();
+  fn = "muse_test_geo_smoothed.fits";
+  cpl_msg_info(__func__, "Saving final table as \"%s\"", fn);
+  cpl_table_save(geov->table, header, NULL, fn, CPL_IO_CREATE);
+  cpl_test(cpl_errorstate_is_equal(state));
+#endif
+  cpl_propertylist_delete(header);
+  muse_geo_table_delete(geov);
+  /* test failures */
+  state = cpl_errorstate_get();
+  rc = muse_geo_correct_slices(NULL, NULL, 1.5);
+  cpl_test(rc == CPL_ERROR_NULL_INPUT && !cpl_errorstate_is_equal(state));
+  cpl_errorstate_set(state);
+  rc = muse_geo_correct_slices(geovkeep, NULL, 0.); /* non-positive sigma */
+  cpl_test(rc == CPL_ERROR_ILLEGAL_INPUT && !cpl_errorstate_is_equal(state));
+  cpl_errorstate_set(state);
+  geov = muse_geo_table_duplicate(geovkeep);
+  cpl_table_delete(geov->table);
+  geov->table = NULL; /* simulate missing table component */
+  rc = muse_geo_correct_slices(geov, NULL, 1.5);
+  cpl_test(rc == CPL_ERROR_NULL_INPUT && !cpl_errorstate_is_equal(state));
+  cpl_errorstate_set(state);
+  muse_geo_table_delete(geov);
+  /* test wrong columns */
+  const char *cols[] = { MUSE_GEOTABLE_FIELD, MUSE_GEOTABLE_CCD, MUSE_GEOTABLE_SKY,
+                         MUSE_GEOTABLE_X, MUSE_GEOTABLE_X"err",
+                         MUSE_GEOTABLE_Y, MUSE_GEOTABLE_Y"err",
+                         MUSE_GEOTABLE_ANGLE, MUSE_GEOTABLE_ANGLE"err",
+                         MUSE_GEOTABLE_WIDTH, MUSE_GEOTABLE_WIDTH"err", NULL
+                         },
+             **col;
+  for (col = cols; *col; col++) {
+    const char *colname = *col;
+    cpl_msg_debug(__func__, "column name %s", colname);
+
+    geov = muse_geo_table_duplicate(geovkeep);
+    /* the column type decides which tests to run */
+    cpl_type coltype = cpl_table_get_column_type(geov->table, colname);
+    cpl_table_erase_column(geov->table, colname);
+    state = cpl_errorstate_get();
+    rc = muse_geo_correct_slices(geov, NULL, 1.5);
+    cpl_test(rc == CPL_ERROR_DATA_NOT_FOUND && !cpl_errorstate_is_equal(state));
+    cpl_errorstate_set(state);
+    if (coltype != CPL_TYPE_DOUBLE) {
+      muse_geo_table_delete(geov);
+      continue;
+    }
+    /* for columns that are double originally, test int and float as well */
+    cpl_table_new_column(geov->table, colname, CPL_TYPE_INT);
+    state = cpl_errorstate_get();
+    rc = muse_geo_correct_slices(geov, NULL, 1.5);
+    cpl_test(rc == CPL_ERROR_INCOMPATIBLE_INPUT&& !cpl_errorstate_is_equal(state));
+    cpl_errorstate_set(state);
+    cpl_table_cast_column(geov->table, colname, colname, CPL_TYPE_FLOAT);
+    rc = muse_geo_correct_slices(geov, NULL, 1.5);
+    cpl_test(rc == CPL_ERROR_INCOMPATIBLE_INPUT&& !cpl_errorstate_is_equal(state));
+    cpl_errorstate_set(state);
+    muse_geo_table_delete(geov);
+  } /* for cols (relevant table columns */
+  muse_geo_table_delete(geovkeep);
+
   return cpl_test_end(0);
 }
