@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2014 European Southern Observatory
+ * Copyright (C) 2007-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -367,7 +367,7 @@ int main(int argc, char **argv)
   cpl_test_nonnull(imagel->header);
   const char *object = cpl_propertylist_get_string(imagel->header, "OBJECT"),
              *extname = cpl_propertylist_get_string(imagel->header, "EXTNAME"),
-             *unit = cpl_propertylist_get_string(imagel->header, "BUNIT");
+             *unit = muse_pfits_get_bunit(imagel->header);
   cpl_test_zero(strncmp(object, OBJECT_STRING, strlen(OBJECT_STRING) + 1));
   cpl_test_zero(strncmp(extname, "CHAN22", 7));
   cpl_test_zero(strncmp(unit, "count", 6));
@@ -382,14 +382,16 @@ int main(int argc, char **argv)
   char *fcontents = cpl_calloc(sb.st_size + 1, 1); /* file-sized buffer */
   FILE *fin = fopen(file, "r"), /* read-only */
        *fout = fopen(file2, "w"); /* wrote to this filename */
-  fread(fcontents, sb.st_size, 1, fin);
+  size_t nitems = fread(fcontents, sb.st_size, 1, fin);
+  cpl_test(nitems == 1);
   fclose(fin);
   int i = 0;
   do {
     i += 80; /* no data section to where we want to go, just step 80 chars */
   } while (i < sb.st_size - 80 && strncmp(fcontents + i, "BUNIT", 5));
   fcontents[i] = 'X'; /* make BUNIT into XUNIT */
-  fwrite(fcontents, sb.st_size, 1, fout);
+  nitems = fwrite(fcontents, sb.st_size, 1, fout);
+  cpl_test(nitems == 1);
   cpl_free(fcontents);
   fclose(fout);
   /* now do that actual test */
@@ -589,8 +591,7 @@ int main(int argc, char **argv)
   cpl_test(cpl_image_get_absflux(image_test->data) == 0.0);
   cpl_test(cpl_image_get_absflux(image_test->stat)
            == cpl_image_get_absflux(image->stat));
-  cpl_test_zero(strncmp(cpl_propertylist_get_string(image_test->header, "BUNIT"),
-                        "adu", 4));
+  cpl_test_zero(strncmp(muse_pfits_get_bunit(image_test->header), "adu", 4));
   muse_image_delete(image_test);
   cpl_errorstate prev_state = cpl_errorstate_get();
   /* load non-existing extension from existing file */
@@ -774,7 +775,7 @@ int main(int argc, char **argv)
   image->stat = cpl_image_duplicate(image->data);
   /* image->dq is not required for this test */
   cpl_test(muse_image_adu_to_count(image) == CPL_ERROR_NONE);
-  cpl_test_zero(strncmp(cpl_propertylist_get_string(image->header, "BUNIT"), "count", 6));
+  cpl_test_zero(strncmp(muse_pfits_get_bunit(image->header), "count", 6));
   cpl_test_rel(cpl_image_get(image->data, 1, 1, &rej), 0.5, FLT_EPSILON);
   cpl_test_rel(cpl_image_get(image->data, 1, 2, &rej), 2.0, FLT_EPSILON);
   cpl_test_rel(cpl_image_get(image->data, 2, 1, &rej), 1.1, FLT_EPSILON);
