@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2014 European Southern Observatory
+ * Copyright (C) 2007-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -215,6 +215,7 @@ muse_test_pixtable_delete_exptable(cpl_table *aExpTable)
     muse_pixtable_load_merge_channels
     muse_pixtable_duplicate
     muse_pixtable_get_type
+    muse_pixtable_flux_multiply
     muse_pixtable_wcs_check (only failure cases)
     muse_pixtable_erase_ifu_slice
     muse_pixtable_is_fluxcal
@@ -331,9 +332,9 @@ int main(int argc, char **argv)
   cpl_errorstate_set(ps);
 
   /* test muse_pixtable_create() */
-  cpl_table *wavecaltable = cpl_table_load(BASEFILENAME"_create_wavecal_in.fits", 1, 0);
-  cpl_table *tracetable = cpl_table_load(BASEFILENAME"_create_trace_in.fits", 1, 0);
-  cpl_table *geotable = cpl_table_load(BASEFILENAME"_create_geotable.fits", 1, 0);
+  cpl_table *wavecaltable = cpl_table_load(BASEFILENAME"_create_wavecal_in.fits", 1, 1);
+  cpl_table *tracetable = cpl_table_load(BASEFILENAME"_create_trace_in.fits", 1, 1);
+  cpl_table *geotable = cpl_table_load(BASEFILENAME"_create_geotable.fits", 1, 1);
   /* just take any image into which the normal MUSE pattern fits */
   muse_image *inimage = muse_image_load(BASEFILENAME"_create_data.fits");
 
@@ -591,6 +592,34 @@ int main(int argc, char **argv)
   ps = cpl_errorstate_get();
   cpl_test(muse_pixtable_get_type(pt2) == MUSE_PIXTABLE_TYPE_UNKNOWN);
   cpl_test(cpl_errorstate_is_equal(ps)); /* no new error set */
+
+  /* test muse_pixtable_flux_multiply() */
+  ps = cpl_errorstate_get();
+  cpl_size nrow2 = muse_pixtable_get_nrow(pt2);
+  float d1 = cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, 0, NULL),
+        d2 = cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, nrow2 / 2, NULL),
+        d3 = cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, nrow2 - 1, NULL);
+  cpl_error_code rc = muse_pixtable_flux_multiply(pt2, 1.33);
+  cpl_test(cpl_errorstate_is_equal(ps) && rc == CPL_ERROR_NONE);
+  cpl_test_rel(cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, 0, NULL), d1 * 1.33, 5e-8);
+  cpl_test_rel(cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, nrow2 / 2, NULL), d2 * 1.33, 5e-8);
+  cpl_test_rel(cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, nrow2 - 1, NULL), d3 * 1.33, 5e-8);
+  rc = muse_pixtable_flux_multiply(pt2, 1. / 1.33); /* reverse it */
+  cpl_test(cpl_errorstate_is_equal(ps) && rc == CPL_ERROR_NONE);
+  cpl_test_rel(cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, 0, NULL), d1, DBL_EPSILON);
+  cpl_test_rel(cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, nrow2 / 2, NULL), d2, DBL_EPSILON);
+  cpl_test_rel(cpl_table_get(pt2->table, MUSE_PIXTABLE_DATA, nrow2 - 1, NULL), d3, DBL_EPSILON);
+  /* failure cases */
+  ps = cpl_errorstate_get();
+  rc = muse_pixtable_flux_multiply(NULL, 2.);
+  cpl_test(!cpl_errorstate_is_equal(ps) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(ps);
+  cpl_table_delete(pt2->table);
+  pt2->table = NULL;
+  ps = cpl_errorstate_get();
+  rc = muse_pixtable_flux_multiply(pt2, 2.);
+  cpl_test(!cpl_errorstate_is_equal(ps) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(ps);
   muse_pixtable_delete(pt2);
 
   /* test muse_pixtable_duplicate() */
@@ -610,7 +639,7 @@ int main(int argc, char **argv)
   cpl_test_zero(setenv("MUSE_PIXTABLE_SAVE_AS_TABLE", "1", 1)); /* bintable */
   char *fn = cpl_sprintf("%s/%s", dir, "muse_pixtable_as_table.fits");
   ps = cpl_errorstate_get();
-  cpl_error_code rc = muse_pixtable_save(pt, fn);
+  rc = muse_pixtable_save(pt, fn);
   cpl_test(cpl_errorstate_is_equal(ps) && rc == CPL_ERROR_NONE);
   muse_pixtable_delete(pt);
   struct stat sb;
