@@ -74,7 +74,7 @@ prepare_test(char *aDir)
   cpl_propertylist_append_int(hdr, "ESO DET READ CURID", 1);
   cpl_propertylist_append_string(hdr, "ESO DET READ CURNAME", "SCI1.0");
   char *file = cpl_sprintf("%s/raw.fits", aDir);
-  cpl_image_save(NULL, file, CPL_TYPE_SHORT, hdr, CPL_IO_DEFAULT);
+  cpl_image_save(NULL, file, CPL_TYPE_SHORT, hdr, CPL_IO_CREATE);
   cpl_propertylist_delete(hdr);
   /* extension header */
   hdr = cpl_propertylist_new();
@@ -178,7 +178,7 @@ frames_sort_test(const char *aDir, cpl_recipe *aRecipe)
       cpl_free(chan);
 
       char *outfile = cpl_sprintf("%s/test_table_%1d_%02d.fits", aDir, n, i);
-      cpl_table_save(table, header, NULL, outfile, CPL_IO_DEFAULT);
+      cpl_table_save(table, header, NULL, outfile, CPL_IO_CREATE);
 
       /* create frame for this file and add it to (some of) the framesets */
       cpl_frame *frame = cpl_frame_new();
@@ -314,6 +314,12 @@ int main(int argc, char **argv)
   retval = exec_func(plugin);
   cpl_test_eq(retval, CPL_ERROR_NONE);
 
+  // run plugin with debug-level output
+  cpl_msg_set_level(CPL_MSG_DEBUG);
+  retval = exec_func(plugin);
+  cpl_test_eq(retval, CPL_ERROR_NONE);
+  cpl_msg_set_level(CPL_MSG_INFO);
+
   // before removing it, use the still existing recipe to do
   // the frames_sort_test that needs a new processing structure
   frames_sort_test(dir, recipe);
