@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2014 European Southern Observatory
+ * Copyright (C) 2007-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -97,12 +97,14 @@ cube_resampling_test_image_stats(cpl_image *aImage, const char *aName)
     muse_resampling_params_set_wcs
     muse_resampling_params_delete
     muse_resampling_cube
+    muse_resampling_euro3d
     muse_resampling_collapse_pixgrid
     muse_datacube_collapse
     muse_datacube_convert_dq
     muse_datacube_save
     muse_datacube_save_recimages
-    muse_resampling_euro3d
+    muse_datacube_load
+    muse_datacube_concat
     muse_euro3dcube_collapse
     muse_euro3dcube_save
     muse_datacube_delete
@@ -292,6 +294,9 @@ int main(int argc, char **argv)
         muse_image *fov = muse_datacube_collapse(cube, fwhite);
         cpl_test_nonnull(fov);
         cpl_test_nonnull(fov->data);
+        cpl_test_nonnull(fov->header);
+        const char *fovunit = muse_pfits_get_bunit(fov->header);
+        cpl_test_eq_string(fovunit, muse_pfits_get_bunit(cube->header));
         name = cpl_sprintf("FITS NAXIS=3 FOV (%s l%d %d)", unit, tl, t);
         cube_resampling_test_image_stats(fov->data, name);
         cpl_free(name);
@@ -352,6 +357,8 @@ int main(int argc, char **argv)
         /* test the the headers now also worked for the reconstructed image */
         cpl_propertylist *him = cpl_propertylist_load(fn, 4),
                          *himdq = cpl_propertylist_load(fn, 5);
+        cpl_test_eq_string(fovunit, muse_pfits_get_bunit(him));
+        cpl_test_zero(cpl_propertylist_has(himdq, "BUNIT")); /* no BUNIT stored for DQ! */
         cpl_test(!strcmp("white", cpl_propertylist_get_string(him, "EXTNAME")) &&
                  !strcmp("white_DQ", cpl_propertylist_get_string(himdq, "EXTNAME")));
         /* test for ESO format headers produced by muse_image_save() */
@@ -494,6 +501,76 @@ int main(int argc, char **argv)
 #endif
           cpl_free(fn2);
 
+          /* now create two cubes, which align nicely in wavelength, *
+           * using OUTPUT_WCS-like setups                            */
+          hwcs = cpl_propertylist_duplicate(cube->header);
+          cpl_propertylist_update_int(hwcs, "NAXIS3", 4); /* force dispersion axis */
+          cpl_wcs *cwcs1 = cpl_wcs_new_from_propertylist(hwcs);
+          double crval3 = muse_pfits_get_crval(hwcs, 3),
+                 cd3_3 = muse_pfits_get_cd(hwcs, 3, 3);
+          cpl_propertylist_update_double(hwcs, "CRVAL3", crval3 + 4 * cd3_3);
+          cpl_wcs *cwcs2 = cpl_wcs_new_from_propertylist(hwcs);
+          cpl_propertylist_delete(hwcs);
+          muse_resampling_params *pwcs1 = muse_resampling_params_new(t),
+                                 *pwcs2 = muse_resampling_params_new(t);
+          pwcs1->tlambda = tl;
+          pwcs1->wcs = cwcs1;
+          printf("CRVAL (1):\n");
+          cpl_array_dump(cpl_wcs_get_crval(cwcs1), 0, 12, stdout);
+          printf("CRPIX (1):\n");
+          cpl_array_dump(cpl_wcs_get_crpix(cwcs1), 0, 12, stdout);
+          printf("CD (1):\n");
+          cpl_matrix_dump(cpl_wcs_get_cd(cwcs1), stdout);
+          fflush(stdout);
+          pwcs2->tlambda = tl;
+          pwcs2->wcs = cwcs2;
+          printf("CRVAL (2):\n");
+          cpl_array_dump(cpl_wcs_get_crval(cwcs2), 0, 12, stdout);
+          printf("CRPIX (2):\n");
+          cpl_array_dump(cpl_wcs_get_crpix(cwcs2), 0, 12, stdout);
+          printf("CD (2):\n");
+          cpl_matrix_dump(cpl_wcs_get_cd(cwcs2), stdout);
+          fflush(stdout);
+          muse_datacube *cube1 = muse_resampling_cube(pt, pwcs1, NULL),
+                        *cube2 = muse_resampling_cube(pt, pwcs2, NULL);
+          muse_resampling_params_delete(pwcs1);
+          muse_resampling_params_delete(pwcs2);
+          cpl_test_nonnull(cube1);
+          cpl_test_nonnull(cube2);
+          cpl_test_eq(cpl_imagelist_get_size(cube1->data), 4);
+          cpl_test_eq(cpl_imagelist_get_size(cube2->data), 4);
+          if (tl == MUSE_RESAMPLING_DISP_AWAV) {
+            /* concat should go without problems and create cube twice the length */
+            cpl_test(muse_datacube_concat(cube1, cube2) == CPL_ERROR_NONE);
+            cpl_test_eq(cpl_imagelist_get_size(cube1->data), 8);
+
+            /* doing it again, should create problems, *
+             * since the cubes do not align any more   */
+            prestate = cpl_errorstate_get();
+            cpl_test(muse_datacube_concat(cube1, cube2) == CPL_ERROR_ILLEGAL_INPUT);
+            cpl_test(!cpl_errorstate_is_equal(prestate));
+            cpl_test_eq(cpl_imagelist_get_size(cube1->data), 8); /* still same size */
+            cpl_errorstate_set(prestate);
+            /* NULL arguments should also cause failures */
+            cpl_test(muse_datacube_concat(cube1, NULL) == CPL_ERROR_NULL_INPUT);
+            cpl_test(!cpl_errorstate_is_equal(prestate));
+            cpl_errorstate_set(prestate);
+            prestate = cpl_errorstate_get();
+            cpl_test(muse_datacube_concat(NULL, cube2) == CPL_ERROR_NULL_INPUT);
+            cpl_test(!cpl_errorstate_is_equal(prestate));
+            cpl_errorstate_set(prestate);
+            muse_datacube_delete(cube1);
+            muse_datacube_delete(cube2);
+          } else if (tl == MUSE_RESAMPLING_DISP_AWAV_LOG) {
+            /* concat should tail for log axes  */
+            prestate = cpl_errorstate_get();
+            cpl_test(muse_datacube_concat(cube1, cube2) == CPL_ERROR_UNSUPPORTED_MODE);
+            cpl_test(!cpl_errorstate_is_equal(prestate));
+            cpl_errorstate_set(prestate);
+            muse_datacube_delete(cube1);
+            muse_datacube_delete(cube2);
+          }
+
           /* also try to convert the DQ to NANs and save again */
           fn2 = cpl_sprintf("cube_resampled_%s_l%d_%d_with_recimage_NANs.fits",
                             unit, tl, t);
