@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2014 European Southern Observatory
+ * Copyright (C) 2007-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -223,6 +223,233 @@ muse_test_wcs_check_celestial_transformation(cpl_propertylist *aWCS,
   cpl_wcs_delete(cwcs);
 } /* muse_test_wcs_check_celestial_transformation() */
 
+#define PRINT_USAGE(rc)                                                        \
+  fprintf(stderr, "\nUsage: %s [ -centroid <gaussian | moffat | box> ] "       \
+          "[ -detsigma <float> ] [ -radius <float> ] [ -faccuracy <float> ] "  \
+          "[ -niter <int> ] [ -rejsigma <float> ] [ -rotcenter <float>,<float ]"\
+          " DATACUBE_ASTROMETRY ASTROMETRY_REFERENCE\n"                        \
+          "  The output ASTROMETRY_WCS is written to a file starting with the" \
+          " same basename as DATACUBE_ASTROMETRY.\n\n\n", argv[0]);            \
+  return (rc);
+
+int
+muse_test_wcs_calibrate_from_cube(int argc, char **argv)
+{
+  cpl_errorstate state = cpl_errorstate_get();
+
+  /* defaults */
+  char *incube = NULL,
+       *intable = NULL;
+  muse_wcs_centroid_type centroid = MUSE_WCS_CENTROID_MOFFAT;
+  double detsigma = 3.0,
+         radius = 5.,
+         faccuracy = 5.,
+         rejsigma = 3.,
+         crpix1 = -0.01,
+         crpix2 = -1.20;
+  int niter = 2;
+
+  /* argument processing */
+  int i;
+  for (i = 1; i < argc; i++) {
+    if (strncmp(argv[i], "-centroid", 10) == 0) {
+      /* skip to next arg to first slice number */
+      i++;
+      if (i < argc) {
+        if (!strncmp(argv[i], "moffat", 7)) {
+          centroid = MUSE_WCS_CENTROID_MOFFAT;
+        } else if (!strncmp(argv[i], "gaussian", 9)) {
+          centroid = MUSE_WCS_CENTROID_GAUSSIAN;
+        } else if (!strncmp(argv[i], "box", 4)) {
+          centroid = MUSE_WCS_CENTROID_BOX;
+        }
+      } else {
+        PRINT_USAGE(2);
+      }
+    } else if (strncmp(argv[i], "-detsigma", 10) == 0) {
+      /* skip to next arg to last slice number */
+      i++;
+      if (i < argc) {
+        detsigma = atof(argv[i]);
+      } else {
+        PRINT_USAGE(3);
+      }
+    } else if (strncmp(argv[i], "-radius", 8) == 0) {
+      /* skip to next arg to last slice number */
+      i++;
+      if (i < argc) {
+        radius = atof(argv[i]);
+      } else {
+        PRINT_USAGE(4);
+      }
+    } else if (strncmp(argv[i], "-faccuracy", 11) == 0) {
+      /* skip to next arg to last slice number */
+      i++;
+      if (i < argc) {
+        faccuracy = atof(argv[i]);
+      } else {
+        PRINT_USAGE(5);
+      }
+    } else if (strncmp(argv[i], "-niter", 7) == 0) {
+      /* skip to next arg to last slice number */
+      i++;
+      if (i < argc) {
+        niter = atoi(argv[i]);
+      } else {
+        PRINT_USAGE(6);
+      }
+    } else if (strncmp(argv[i], "-rejsigma", 10) == 0) {
+      /* skip to next arg to last slice number */
+      i++;
+      if (i < argc) {
+        rejsigma = atof(argv[i]);
+      } else {
+        PRINT_USAGE(7);
+      }
+    } else if (strncmp(argv[i], "-rotcenter", 11) == 0) {
+      /* skip to next arg to last slice number */
+      i++;
+      if (i < argc) {
+        cpl_array *a = muse_cplarray_new_from_delimited_string(argv[i], ",");
+        crpix1 = cpl_array_get_string(a, 0) ? atof(cpl_array_get_string(a, 0))
+                                            : crpix1;
+        crpix2 = cpl_array_get_string(a, 1) ? atof(cpl_array_get_string(a, 1))
+                                            : crpix2;
+        cpl_array_delete(a);
+      } else {
+        PRINT_USAGE(8);
+      }
+    } else if (strncmp(argv[i], "-", 1) == 0) { /* unallowed options */
+      PRINT_USAGE(9);
+    } else {
+      if (incube && intable) {
+        break; /* we have the possible names, skip the rest */
+      }
+      if (!incube) {
+        incube = argv[i]; /* set the name for the input cube */
+      } else {
+        intable = argv[i] /* set the name for the input reference table */;
+      }
+    }
+  } /* for i (all arguments) */
+
+  muse_datacube *cube = muse_datacube_load(incube);
+  if (!cube) {
+    PRINT_USAGE(10);
+  }
+
+  cpl_frame *refframe = cpl_frame_new();
+  cpl_frame_set_filename(refframe, intable);
+  cpl_table *reftable = muse_postproc_load_nearest(cube->header, refframe,
+                                                   20., 60., NULL, NULL);
+  cpl_frame_delete(refframe);
+
+  /* get the image from the cube */
+  cpl_errorstate es = cpl_errorstate_get();
+  muse_image *detimage = NULL;
+  const char *extname = NULL;
+  for (i = 0; i < cpl_array_get_size(cube->recnames); i++) {
+    const char *name = cpl_array_get_string(cube->recnames, i);
+    cpl_msg_debug(__func__, "image extension \"%s\"", name);
+    if (name && !strncmp(name, MUSE_WCS_DETIMAGE_EXTNAME,
+                         strlen(MUSE_WCS_DETIMAGE_EXTNAME) + 1)) {
+      detimage = muse_imagelist_get(cube->recimages, i);
+    }
+  } /* for i (all cube image extensions) */
+  if (!detimage) {
+    cpl_errorstate_set(es); /* swallow access beyond boundaries */
+    /* fall back to white-light image */
+    cpl_msg_warning(__func__, "No \"%s\" image in cube, trying to reconstruct "
+                    "it.", MUSE_WCS_DETIMAGE_EXTNAME);
+    /* code copied from muse_wcs_locate_sources() */
+    int iplane, irefplane = cpl_imagelist_get_size(cube->data) / 2;
+    muse_imagelist *list = muse_imagelist_new();
+    unsigned int ilist = 0;
+    for (iplane = irefplane - 1; iplane <= irefplane + 1; iplane++) {
+      muse_image *image = muse_image_new();
+      image->data = cpl_image_duplicate(cpl_imagelist_get(cube->data, iplane));
+      cpl_image_reject_value(image->data, CPL_VALUE_NAN); /* mark NANs */
+      image->dq = cpl_image_new_from_mask(cpl_image_get_bpm(image->data));
+      image->stat = cpl_image_duplicate(cpl_imagelist_get(cube->stat, iplane));
+      muse_imagelist_set(list, image, ilist++);
+    } /* for iplane (planes around ref. wavelength) */
+    detimage = muse_combine_median_create(list);
+    /* fill existing empty header with the header of the *
+     * cube, but without the third dimension in the WCS  */
+    cpl_propertylist_copy_property_regexp(detimage->header, cube->header,
+                                          "^C...*3$|^CD3_.$", 1);
+    muse_imagelist_delete(list);
+    if (!detimage) {
+      cpl_table_delete(reftable);
+      muse_datacube_delete(cube);
+      PRINT_USAGE(12);
+    } /* if recreating detimage failed */
+  } else {
+    extname = muse_pfits_get_extname(detimage->header);
+  } /* else detimage from cube */
+  es = cpl_errorstate_get();
+  if (extname) {
+    cpl_msg_info(__func__, "Loaded image \"%s\" from cube \"%s\"", extname, incube);
+  } else {
+    cpl_errorstate_set(es); /* swallow data not found */
+    cpl_msg_info(__func__, "Recreated detection image from cube \"%s\"", incube);
+  }
+
+  /* now we loaded everything, do the processing */
+  muse_wcs_object *wcs = muse_wcs_object_new();
+  /* hardcode defaults here as set in muse_astrometry.xml */
+  wcs->crpix1 = crpix1;
+  wcs->crpix2 = crpix2;
+  wcs->ra = muse_pfits_get_ra(cube->header);
+  wcs->dec = muse_pfits_get_dec(cube->header);
+  wcs->detected = muse_wcs_centroid_stars(detimage, detsigma, centroid);
+  if (!reftable) {
+    muse_datacube_delete(cube);
+    muse_wcs_object_delete(wcs);
+    if (!extname) {
+      muse_image_delete(detimage);
+    }
+    PRINT_USAGE(11);
+  }
+  /* use defaults from the muse_astrometry recipe */
+  muse_wcs_solve(wcs, reftable, radius, faccuracy, niter, rejsigma);
+
+  /* prepare input filename for creation of output names */
+  char *dot = strrchr(incube, '.');
+  *dot = '\0'; /* replace the dot */
+  if (!extname) {
+    /* reconstructed from cube, need to save and delete the image */
+    char *fn = cpl_sprintf("%s_%s.fits", incube, MUSE_WCS_DETIMAGE_EXTNAME);
+    cpl_error_code rc = muse_image_save(detimage, fn);
+    if (rc == CPL_ERROR_NONE) {
+      cpl_msg_info(__func__, "Saved reconstructed detection image to \"%s\".", fn);
+    } else {
+      cpl_msg_error(__func__, "Failed reconstructed detection image to \"%s\": "
+                    "%s", fn, cpl_error_get_message());
+    }
+    cpl_free(fn);
+    muse_image_delete(detimage);
+  } /* if no extname */
+  char *object = cpl_sprintf("Astrometric calibration (%s)",
+                             cpl_propertylist_get_string(cube->header, "OBJECT"));
+  cpl_propertylist_update_string(wcs->wcs, "OBJECT", object);
+  char *fn = cpl_sprintf("%s_%s.fits", incube, MUSE_TAG_ASTROMETRY_WCS);
+  cpl_error_code rc = cpl_propertylist_save(wcs->wcs, fn, CPL_IO_CREATE);
+  if (rc == CPL_ERROR_NONE) {
+    cpl_msg_info(__func__, "Saved solution to \"%s\".", fn);
+  } else {
+    cpl_msg_error(__func__, "Failed saving solution to \"%s\": %s", fn,
+                  cpl_error_get_message());
+  }
+  cpl_free(fn);
+  cpl_free(object);
+  muse_wcs_object_delete(wcs);
+  cpl_table_delete(reftable);
+  muse_datacube_delete(cube);
+
+  return cpl_errorstate_is_equal(state) ? 0 : 50;
+} /* muse_test_wcs_calibrate_from_cube() */
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Test program to check that all the functions of the muse_wcs module
@@ -231,6 +458,7 @@ muse_test_wcs_check_celestial_transformation(cpl_propertylist *aWCS,
   This program explicitely tests
     muse_wcs_object_new
     muse_wcs_object_delete
+    muse_wcs_centroid_stars
     muse_wcs_locate_sources
     muse_wcs_solve
     muse_wcs_create_default
@@ -254,57 +482,103 @@ muse_test_wcs_check_celestial_transformation(cpl_propertylist *aWCS,
 /*----------------------------------------------------------------------------*/
 int main(int argc, char **argv)
 {
-  UNUSED_ARGUMENTS(argc, argv);
   cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
 
+  if (argc > 1) {
+    /* If there are arguments, misuse this program as astrometry calibrator. */
+    cpl_test(muse_test_wcs_calibrate_from_cube(argc, argv) == 0);
+    return cpl_test_end(0);
+  }
+
   /*********************************************
    * check WCS creation from input pixel table *
    *********************************************/
   muse_pixtable *ptin = muse_pixtable_load(BASEFILENAME"_pt_in.fits");
   cpl_table *wcsref = cpl_table_load(BASEFILENAME"_scene.fits", 1, 1);
   cpl_test(ptin && wcsref);
-  cpl_error_code rc;
-  /* loop through the possible WCS centroid types */
+  /* run once with muse_wcs_locate_sources() to create cube and image */
+  muse_wcs_object *wcsobj = muse_wcs_object_new();
+  cpl_test_nonnull(wcsobj);
+  cpl_error_code rc = muse_wcs_locate_sources(ptin, 3., MUSE_WCS_CENTROID_MOFFAT,
+                                              wcsobj);
+  cpl_test(rc == CPL_ERROR_NONE);
+  cpl_test_nonnull(wcsobj->cube);
+  cpl_test_nonnull(wcsobj->detected);
+  /* check that the white-light image was saved (as the 1st image) */
+  cpl_test(cpl_array_get_string(wcsobj->cube->recnames, 0) &&
+           !strncmp(cpl_array_get_string(wcsobj->cube->recnames, 0),
+                    "white", 6));
+  /* check that the detection image was saved (as the 2nd image) */
+  cpl_test(cpl_array_get_string(wcsobj->cube->recnames, 1) &&
+           !strncmp(cpl_array_get_string(wcsobj->cube->recnames, 1),
+                    MUSE_WCS_DETIMAGE_EXTNAME,
+                    strlen(MUSE_WCS_DETIMAGE_EXTNAME) + 1));
+  /* loop through the possible centroid types, use muse_wcs_centroid_stars() *
+   * for this so that it doesn't need to recreate the whole cube every time  */
   muse_wcs_centroid_type type;
   const char *typestring[] = {
     "GAUSSIAN",
     "MOFFAT",
     "BOX"
   };
-  muse_wcs_object *wcsobj = NULL;
   for (type = MUSE_WCS_CENTROID_GAUSSIAN; type <= MUSE_WCS_CENTROID_BOX; type++) {
     cpl_msg_debug(__func__, "\n____ centroid type %s _____", typestring[type]);
-    wcsobj = muse_wcs_object_new();
-    cpl_test_nonnull(wcsobj);
     /* for the last one, replace the normal DAR *
      * header with the one from dar_check       */
     if (type == MUSE_WCS_CENTROID_BOX) {
       cpl_propertylist_erase_regexp(ptin->header, MUSE_HDR_PT_DAR_NAME, 0);
       cpl_propertylist_append_float(ptin->header, MUSE_HDR_PT_DAR_CORR, 0.001);
     }
-    rc = muse_wcs_locate_sources(ptin, 3., type, wcsobj);
-    cpl_test(rc == CPL_ERROR_NONE);
-    cpl_test_nonnull(wcsobj->detected);
+    cpl_errorstate es = cpl_errorstate_get();
+    muse_image *image = muse_imagelist_get(wcsobj->cube->recimages, 1);
+    cpl_table *detected = muse_wcs_centroid_stars(image, 3., type);
+    cpl_test(cpl_errorstate_is_equal(es));
+    cpl_test_nonnull(detected);
 #if 0
     char *fn = cpl_sprintf("muse_test_wcs_detected_%s.fits", typestring[type]);
-    cpl_table_save(wcsobj->detected, NULL, NULL, fn, CPL_IO_CREATE);
+    cpl_table_save(detected, NULL, NULL, fn, CPL_IO_CREATE);
     cpl_free(fn);
 #endif
-    cpl_test(cpl_table_get_nrow(wcsobj->detected) == 63);
+    cpl_test(cpl_table_get_nrow(detected) == 63);
     /* check position of selected isolated objects, reference *
      * positions measured using IRAF imexamine 'a'            */
-    muse_test_wcs_check_object_detection(wcsobj->detected,  26.47,  41.08);
-    muse_test_wcs_check_object_detection(wcsobj->detected,  62.98, 121.62);
-    muse_test_wcs_check_object_detection(wcsobj->detected, 169.56,  33.6);
-    muse_test_wcs_check_object_detection(wcsobj->detected, 117.26,  26.92);
-    muse_test_wcs_check_object_detection(wcsobj->detected, 233.57,  41.53);
-    muse_test_wcs_check_object_detection(wcsobj->detected, 211.7,  151.3);
-    muse_test_wcs_check_object_detection(wcsobj->detected, 163.29, 152.43);
-    muse_test_wcs_check_object_detection(wcsobj->detected,  90.38, 168.80);
-    muse_test_wcs_check_object_detection(wcsobj->detected,  30.85, 253.93);
-    muse_test_wcs_check_object_detection(wcsobj->detected, 187.42, 233.65);
-    muse_test_wcs_check_object_detection(wcsobj->detected, 237.70, 270.73);
-    muse_test_wcs_check_object_detection(wcsobj->detected, 226.48, 231.46);
+    muse_test_wcs_check_object_detection(detected,  26.47,  41.08);
+    muse_test_wcs_check_object_detection(detected,  62.98, 121.62);
+    muse_test_wcs_check_object_detection(detected, 169.56,  33.6);
+    muse_test_wcs_check_object_detection(detected, 117.26,  26.92);
+    muse_test_wcs_check_object_detection(detected, 233.57,  41.53);
+    muse_test_wcs_check_object_detection(detected, 211.7,  151.3);
+    muse_test_wcs_check_object_detection(detected, 163.29, 152.43);
+    muse_test_wcs_check_object_detection(detected,  90.38, 168.80);
+    muse_test_wcs_check_object_detection(detected,  30.85, 253.93);
+    muse_test_wcs_check_object_detection(detected, 187.42, 233.65);
+    muse_test_wcs_check_object_detection(detected, 237.70, 270.73);
+    muse_test_wcs_check_object_detection(detected, 226.48, 231.46);
+
+    /* for one, also test with an image without dq component but with NANs */
+    if (type == MUSE_WCS_CENTROID_GAUSSIAN) {
+      muse_image *image2 = muse_image_duplicate(image);
+      muse_image_dq_to_nan(image2);
+      es = cpl_errorstate_get();
+      cpl_table *detected2 = muse_wcs_centroid_stars(image2, 3., type);
+      cpl_test(cpl_errorstate_is_equal(es));
+      cpl_test_nonnull(detected);
+      /* rerun selected tests */
+      cpl_test(cpl_table_get_nrow(detected) == 63);
+      muse_test_wcs_check_object_detection(detected,  26.47,  41.08);
+      muse_test_wcs_check_object_detection(detected, 233.57,  41.53);
+      muse_test_wcs_check_object_detection(detected,  30.85, 253.93);
+      muse_image_delete(image2);
+      cpl_table_delete(detected2);
+    }
+
+    /* see how to use this table further; for that replace it  *
+     * into the WCS object, but keep the existing table around */
+    cpl_table *saved = wcsobj->detected;
+    wcsobj->detected = detected;
+    /* remove the previous solution */
+    cpl_propertylist_delete(wcsobj->wcs);
+    wcsobj->wcs = NULL;
     rc = muse_wcs_solve(wcsobj, wcsref, 5., 5., 2, 3.);
     cpl_msg_debug(__func__, "Solving WCS returned rc=%d: %s", rc,
                   rc != CPL_ERROR_NONE ? cpl_error_get_message() : "");
@@ -320,9 +594,9 @@ int main(int argc, char **argv)
     /* should both be about 0.2''/px */
     cpl_test_abs(xsc, 0.2/3600., klimitsINM[type][0]);
     cpl_test_abs(ysc, 0.2/3600., klimitsINM[type][1]);
-    if (type != MUSE_WCS_CENTROID_BOX) {
-       muse_wcs_object_delete(wcsobj);
-    }
+    /* reinstate the original detected sources table */
+    cpl_table_delete(detected);
+    wcsobj->detected = saved;
   } /* for type */
   /* test some failure cases of muse_wcs_locate_sources() */
   cpl_errorstate state = cpl_errorstate_get();
@@ -361,10 +635,30 @@ int main(int argc, char **argv)
   /* create pixel table with flat data, should fail object detection */
   cpl_table_fill_column_window_float(pt2->table, MUSE_PIXTABLE_DATA,
                                      0, cpl_table_get_nrow(pt2->table), 100);
-  muse_datacube_delete(wcsobj->cube); /* don't want to leak the previous cube */
+  muse_datacube *safe = wcsobj->cube; /* don't want to leak the previous cube */
+  wcsobj->cube = NULL;
   rc = muse_wcs_locate_sources(pt2, 3., MUSE_WCS_CENTROID_BOX, wcsobj);
   cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_DATA_NOT_FOUND);
   cpl_errorstate_set(state);
+  muse_datacube_delete(wcsobj->cube); /* delete constant-value cube again */
+  wcsobj->cube = safe;
+  /* test some failure cases of muse_wcs_centroid_stars() */
+  cpl_table *bla = muse_wcs_centroid_stars(NULL, 3., MUSE_WCS_CENTROID_MOFFAT);
+  cpl_test(!cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_test_null(bla);
+  muse_image *image = muse_imagelist_get(wcsobj->cube->recimages, 1);
+  state = cpl_errorstate_get();
+  bla = muse_wcs_centroid_stars(image, 0., MUSE_WCS_CENTROID_MOFFAT);
+  cpl_test(!cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_test_null(bla);
+  bla = muse_wcs_centroid_stars(image, 3., MUSE_WCS_CENTROID_BOX + 1);
+  cpl_test(!cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
   /* test some failure cases of muse_wcs_solve() */
   rc = muse_wcs_solve(NULL, wcsref, 5., 5., 2, 3.);
   cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
@@ -411,10 +705,8 @@ int main(int argc, char **argv)
   state = cpl_errorstate_get();
   cpl_propertylist *pwcs = muse_wcs_create_default();
   cpl_test(cpl_errorstate_is_equal(state));
-  cpl_test(!strncmp("RA---TAN", cpl_propertylist_get_string(pwcs, "CTYPE1"), 9) &&
-           !strncmp("DEC--TAN", cpl_propertylist_get_string(pwcs, "CTYPE2"), 9));
-  cpl_propertylist_append_double(pwcs, "RA", 0.); /* muse_wcs_new() needs RA and DEC */
-  cpl_propertylist_append_double(pwcs, "DEC", 0.);
+  cpl_test(!strncmp("RA---TAN", muse_pfits_get_ctype(pwcs, 1), 9) &&
+           !strncmp("DEC--TAN", muse_pfits_get_ctype(pwcs, 2), 9));
   state = cpl_errorstate_get();
   muse_wcs *wcs = muse_wcs_new(pwcs);
   cpl_test(cpl_errorstate_is_equal(state));
@@ -699,7 +991,7 @@ int main(int argc, char **argv)
    ***************************************************************************/
   /* use values from WCS Paper II, example 1 as references */
   p = muse_test_wcs_create_example_1_header();
-  double x,y;
+  double x, y;
   cpl_test(muse_wcs_projplane_from_celestial(p, 47.503264, 62.795111, &x, &y)
            == CPL_ERROR_NONE);
   cpl_test(fabs(x - 0.765000) < kLimitPPl && fabs(y - (-0.765000)) < kLimitPPl);
@@ -845,16 +1137,35 @@ int main(int argc, char **argv)
   cpl_test(unsetenv("MUSE_DEBUG_WCS") == 0);
   cpl_propertylist_delete(p);
 
-  /* create and delete a wcs with a bad header */
+  /* create and delete a wcs with a bad header (singular CD matrix) */
   p = muse_test_wcs_create_typical_tan_header();
   cpl_propertylist_update_double(p, "CD1_1", 0.);
   cpl_propertylist_update_double(p, "CD2_2", 0.);
   state = cpl_errorstate_get();
   wcs = muse_wcs_new(p);
   cpl_test(!cpl_errorstate_is_equal(state) &&
-           cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+           cpl_error_get_code() == CPL_ERROR_SINGULAR_MATRIX);
   cpl_errorstate_set(state);
+  cpl_test_nonnull(wcs);
   cpl_free(wcs);
+
+  /* use the same bad header to test other functions   *
+   * that depend on the invertability of the CD matrix */
+  state = cpl_errorstate_get();
+  x = y = 0.;
+  cpl_test(muse_wcs_pixel_from_projplane(p, 10., -10., &x, &y)
+           == CPL_ERROR_SINGULAR_MATRIX);
+  cpl_test(!cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_SINGULAR_MATRIX);
+  cpl_errorstate_set(state);
+  cpl_test(isnan(x) && isnan(y));
+  x = y = 0.;
+  cpl_test(muse_wcs_pixel_from_celestial(p, 10., -10, &x, &y)
+           == CPL_ERROR_SINGULAR_MATRIX);
+  cpl_test(!cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_SINGULAR_MATRIX);
+  cpl_errorstate_set(state);
+  cpl_test(isnan(x) && isnan(y));
   cpl_propertylist_delete(p);
 
   return cpl_test_end(0);
