@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2014 European Southern Observatory
+ * Copyright (C) 2014-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -123,6 +123,11 @@ int main(int argc, char **argv)
   cpl_table_new_column(table, "P2", CPL_TYPE_INT);
 
   /* check ranges and grids of the cubes involved, save in the table */
+  int irecout = -1, /* output receipe parameter set */
+      iparout1 = -1, /* and output parameter numbers for low */
+      iparout2 = -1; /* and high wavelength limits */
+  double lminout = 99999.,  /* variables to store final output limit for low */
+         lmaxout = -99999.; /* and high wavelength limits */
   char *ctype1ref = NULL, *ctype2ref = NULL,
        *cunit1ref = NULL, *cunit2ref = NULL;
   double crpix1ref = NAN, crpix2ref = NAN, crval1ref = NAN, crval2ref = NAN,
@@ -146,24 +151,56 @@ int main(int argc, char **argv)
     cpl_propertylist *header = cpl_propertylist_load(iname, 0);
     double lmin = -FLT_MAX,
            lmax = FLT_MAX;
-    /* loop through the recipe parameters stored in the cube */
+    /* search for last set of recipe parameters stored in the cube,  */
     cpl_errorstate es = cpl_errorstate_get();
+    int irec = 0;
+    do {
+      irec++;
+      char *hname = cpl_sprintf("ESO PRO REC%d ID", irec);
+      /* recipe name should exist at least, so search that */
+      const char *name = cpl_propertylist_get_string(header, hname);
+      cpl_free(hname);
+      if (!name) {
+        break;
+      }
+      cpl_msg_debug(idstring, "REC = %d, ID = %s", irec, name);
+    } while (cpl_errorstate_is_equal(es));
+    cpl_errorstate_set(es);
+    cpl_msg_info(idstring, "Using recipe parameters ESO PRO REC%d to determine "
+                 "wavelength ranges", --irec);
+    if (irecout < 0) {
+       irecout = irec; /* remember output receipe parameter set */
+    }
+
+    /* loop through the recipe parameters stored in the cube */
     int ipar = 0;
     do {
       ipar++;
-      char *hname = cpl_sprintf("ESO PRO REC1 PARAM%d NAME", ipar);
+      char *hname = cpl_sprintf("ESO PRO REC%d PARAM%d NAME", irec, ipar);
       const char *name = cpl_propertylist_get_string(header, hname);
       cpl_free(hname);
       cpl_boolean islmin = name && !strncmp(name, "lambdamin", 10),
                   islmax = name && !strncmp(name, "lambdamax", 10);
       if (islmin || islmax) {
-        char *hval = cpl_sprintf("ESO PRO REC1 PARAM%d VALUE", ipar);
+        char *hval = cpl_sprintf("ESO PRO REC%d PARAM%d VALUE", irec, ipar);
         if (islmin) {
+          if (iparout1 < 0) {
+            iparout1 = ipar; /* remember parameter number */
+          }
           lmin = atof(cpl_propertylist_get_string(header, hval));
           cpl_msg_debug(idstring, "lmin = %f", lmin);
+          if (lmin < lminout) {
+            lminout = lmin; /* remember minimum output limit */
+          }
         } else {
+          if (iparout2 < 0) {
+            iparout2 = ipar; /* remember parameter number */
+          }
           lmax = atof(cpl_propertylist_get_string(header, hval));
           cpl_msg_debug(idstring, "lmax = %f", lmax);
+          if (lmax > lmaxout) {
+            lmaxout = lmax; /* remember maximum output limit */
+          }
         }
         cpl_free(hval);
       } /* if */
@@ -173,12 +210,12 @@ int main(int argc, char **argv)
 
     /* load DATA header and get lambda solution */
     header = cpl_propertylist_load(iname, iext);
-    double crpix3 = cpl_propertylist_get_double(header, "CRPIX3"),
-           crval3 = cpl_propertylist_get_double(header, "CRVAL3"),
-           cd33 = cpl_propertylist_get_double(header, "CD3_3");
+    double crpix3 = muse_pfits_get_crpix(header, 3),
+           crval3 = muse_pfits_get_crval(header, 3),
+           cd33 = muse_pfits_get_cd(header, 3, 3);
     int naxis3 = cpl_propertylist_get_int(header, "NAXIS3");
 
-    const char *cunit3 = cpl_propertylist_get_string(header, "CUNIT3"),
+    const char *cunit3 = muse_pfits_get_cunit(header, 3),
                *lunit = cpl_table_get_column_unit(table, "LMIN");
     if (!lunit) {
       cpl_table_set_column_unit(table, "LMIN", cunit3);
@@ -189,7 +226,7 @@ int main(int argc, char **argv)
       cpl_propertylist_delete(header);
       continue;
     }
-    const char *ctype3 = cpl_propertylist_get_string(header, "CTYPE3");
+    const char *ctype3 = muse_pfits_get_ctype(header, 3);
     if (!ctype3ref) {
       ctype3ref = cpl_strdup(ctype3);
       crpix3ref = crpix3;
@@ -218,18 +255,18 @@ int main(int argc, char **argv)
     }
 
     /* check spatial WCS against first cube */
-    const char *ctype1 = cpl_propertylist_get_string(header, "CTYPE1"),
-               *ctype2 = cpl_propertylist_get_string(header, "CTYPE2"),
-               *cunit1 = cpl_propertylist_get_string(header, "CUNIT1"),
-               *cunit2 = cpl_propertylist_get_string(header, "CUNIT2");
-    double crpix1 = cpl_propertylist_get_double(header, "CRPIX1"),
-           crpix2 = cpl_propertylist_get_double(header, "CRVAL2"),
-           crval1 = cpl_propertylist_get_double(header, "CRVAL1"),
-           crval2 = cpl_propertylist_get_double(header, "CRVAL2"),
-           cd11 = cpl_propertylist_get_double(header, "CD1_1"),
-           cd12 = cpl_propertylist_get_double(header, "CD1_2"),
-           cd21 = cpl_propertylist_get_double(header, "CD2_1"),
-           cd22 = cpl_propertylist_get_double(header, "CD2_2");
+    const char *ctype1 = muse_pfits_get_ctype(header, 1),
+               *ctype2 = muse_pfits_get_ctype(header, 2),
+               *cunit1 = muse_pfits_get_cunit(header, 1),
+               *cunit2 = muse_pfits_get_cunit(header, 2);
+    double crpix1 = muse_pfits_get_crpix(header, 1),
+           crpix2 = muse_pfits_get_crval(header, 2),
+           crval1 = muse_pfits_get_crval(header, 1),
+           crval2 = muse_pfits_get_crval(header, 2),
+           cd11 = muse_pfits_get_cd(header, 1, 1),
+           cd12 = muse_pfits_get_cd(header, 1, 2),
+           cd21 = muse_pfits_get_cd(header, 2, 1),
+           cd22 = muse_pfits_get_cd(header, 2, 2);
     if (!ctype1ref) {
       ctype1ref = cpl_strdup(ctype1);
       ctype2ref = cpl_strdup(ctype2);
@@ -436,9 +473,25 @@ int main(int argc, char **argv)
   } /* for ic (all other valid cubes) */
   cpl_table_delete(table);
 
-  /* clean up the output header of the recipe options that are now obsolete */
-  cpl_propertylist_erase_regexp(cube->header, "ESO PRO REC1 ", 0);
+  /* reset the recipe parameters in the output header with the now full range */
+  cpl_msg_info(idstring, "Resetting recipe parameters ESO PRO REC%d to determine "
+               "wavelength ranges", irecout);
+  char *hname = cpl_sprintf("ESO PRO REC%d PARAM%d VALUE", irecout, iparout1),
+       *hvalue = cpl_sprintf("%g", lminout),
+       *hcomment = cpl_sprintf("set with %s!", idstring);
+  cpl_propertylist_update_string(cube->header, hname, hvalue);
+  cpl_propertylist_set_comment(cube->header, hname, hcomment);
+  cpl_free(hname);
+  cpl_free(hvalue);
+  hname = cpl_sprintf("ESO PRO REC%d PARAM%d VALUE", irecout, iparout2),
+  hvalue = cpl_sprintf("%g", lmaxout),
+  cpl_propertylist_update_string(cube->header, hname, hvalue);
+  cpl_propertylist_set_comment(cube->header, hname, hcomment);
+  cpl_free(hname);
+  cpl_free(hvalue);
+  cpl_free(hcomment);
 
+  /* save output, then check for errors, and end */
   int rc = muse_datacube_save(cube, oname);
   muse_datacube_delete(cube);
   if (rc == CPL_ERROR_NONE) {
